/*
 * arch/arm/mach-tcc93xx/clock.c
 *
 * Clock driver for TCC93xx
 *
 * Copyright (C) 2010 Telechips, Inc.
 *
 * This software is licensed under the terms of the GNU General Public
 * License version 2, as published by the Free Software Foundation, and
 * may be copied, distributed, and modified under those terms.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 */

/*===========================================================================

                         INCLUDE FILES FOR MODULE

===========================================================================*/

#include <linux/kernel.h>
#include <linux/init.h>
#include <linux/module.h>
#include <linux/mutex.h>
#include <linux/list.h>
#include <linux/err.h>
#include <linux/clk.h>
#include <linux/spinlock.h>

#include <mach/clock.h>
#include <mach/globals.h>
#include <mach/tca_ckc.h>
#include <linux/delay.h>
#include <bsp.h>
#include <asm/io.h> 

/*===========================================================================

                 DEFINITIONS AND DECLARATIONS FOR MODULE

===========================================================================*/

//#define USE_CLOCK_PMU_CTRL

static LIST_HEAD(clock_list);
static DEFINE_SPINLOCK(clock_lock);
static DEFINE_MUTEX(clock_mutex);

struct freqclk_table {
	unsigned int freq;
	int clk_src;
};

#define CALLED_BY_CLOCK_C_FILE
#if defined(CONFIG_DRAM_MDDR)
#include "tcc_clocktbl_lpddr.h"
#elif defined(CONFIG_DRAM_DDR2)
#include "tcc_clocktbl_ddr2.h"
#elif defined(CONFIG_DRAM_DDR3)
#include "tcc_clocktbl_ddr3.h"
#else
#error Select the DDR type
#endif

static int _clk_set_rate(struct clk *clk, unsigned long rate);

#define NUM_FREQS		ARRAY_SIZE(clk_table)
#define iomap_p2v(x)	io_p2v(x)


/*===========================================================================

                                Clock Sources

===========================================================================*/

/*------------------------------ xin ------------------------------*/
static struct clk clk_xin = {
	.name = "xin",
	.rate = 12000,
};

/*----------------------------- PLL0 ------------------------------*/
static int pll0_clk_set_rate(struct clk *clk, unsigned long rate)
{
#ifndef CONFIG_MEM_CLK_SYNC_MODE
	if (clk->rate == rate)
		return 0;

	tca_ckc_setpll(rate/100, 0);
#endif
	return 0;
}

static struct clk clk_pll0 = {
	.name = "pll0",
	.flags = CLK_ALWAYS_ENABLED,
	.set_rate = pll0_clk_set_rate,
};

/*----------------------------- PLL1 ------------------------------*/
static struct clk clk_pll1 = {
	.name = "pll1",
	.flags = CLK_ALWAYS_ENABLED,
};

/*----------------------------- PLL2 ------------------------------*/
static struct clk clk_pll2 = {
	.name = "pll2",
	.flags = CLK_ALWAYS_ENABLED,
};

/*----------------------------- PLL3 ------------------------------*/
static struct clk clk_pll3 = {
	.name = "pll3",
	.flags = CLK_ALWAYS_ENABLED,
};

/*----------------------------- PLL4 ------------------------------*/
static struct clk clk_pll4 = {
	.name = "pll4",
	.flags = CLK_ALWAYS_ENABLED,
};

/*----------------------------- PLL5 ------------------------------*/
static struct clk clk_pll5 = {
	.name = "pll5",
	.flags = CLK_ALWAYS_ENABLED,
};


/*===========================================================================

                               System Clocks

===========================================================================*/

/*------------------------------ CPU ------------------------------*/
static unsigned long cpu_clk_get_rate(struct clk *clk)
{
	return tca_ckc_getcpu() * 100;
}

static int cpu_clk_set_rate(struct clk *clk, unsigned long rate)
{
#ifndef CONFIG_MEM_CLK_SYNC_MODE
	int n;
	unsigned long pll;

	if (clk->rate == rate)
		return 0;

	pll = tca_ckc_getpll(0);

	rate /= 100;
	n = (rate*16) / pll;
	if (!n) ++n;	// n must be greater than 0

	tca_ckc_setcpu(n);
#endif
	return 0;
}

static struct clk clk_cpu = {
	.name = "cpu",
	.parent = &clk_pll0,
	.flags = CLK_ALWAYS_ENABLED,
	.get_rate = cpu_clk_get_rate,
	.set_rate = cpu_clk_set_rate,
};

/*-------------------------- Memory Bus ---------------------------*/
static int membus_clk_set_rate(struct clk *clk, unsigned long rate)
{
	struct clk *pclk;
	
	list_for_each_entry(pclk, &clock_list, list) {
		if (pclk->parent == clk && pclk->rate > rate)
                rate = pclk->rate;
	}
	if (clk->rate != rate) {
		tca_ckc_set_mem_freq(rate / 1000);
	}
	clk->rate = rate;
	return 0;
}

static struct clk clk_membus = {
	.name = "membus",
	.parent = &clk_pll3,
	.flags = CLK_ALWAYS_ENABLED,
	.set_rate = membus_clk_set_rate,
};

/*------------------------------ SMU ------------------------------*/
static int smubus_clk_set_rate(struct clk *clk, unsigned long rate)
{
#ifndef CONFIG_MEM_CLK_SYNC_MODE
	tca_ckc_setfbusctrl(CLKCTRL7,  ENABLE, NORMAL_MD, rate, DIRECTPLL0); /*FBUS_SMU*/
	clk->rate = rate;
#endif
	return 0;
}

static struct clk clk_smu = {
	.name = "smu",
	.parent = &clk_pll0,
	.flags = CLK_ALWAYS_ENABLED,	
	.set_rate = smubus_clk_set_rate,
};


/*===========================================================================

                             Display Bus Clock

===========================================================================*/

/*---------------------------- DDI Bus ----------------------------*/
static int ddi_clk_enable(struct clk *clk)
{
#if defined(USE_CLOCK_PMU_CTRL)
	if(tca_ckc_getpmupwroff(PMU_DDIBUS) == 1) {
		tca_ckc_setpmupwroff(PMU_DDIBUSPRE, 0);
		tca_ckc_setpmupwroff(PMU_DDIBUS, 0);
		tca_ckc_setpmupwroff(PMU_DDIBUSISOLATION, 1);
		ndelay(100); // min 50ns
		tca_ckc_setswreset(RESET_DDIBUS, 0);
	}
#endif

	tca_ckc_enable(CLKCTRL1, 1);

	return 0;
}

static void ddi_clk_disable(struct clk *clk)
{
	tca_ckc_enable(CLKCTRL1, 0);

#if defined(USE_CLOCK_PMU_CTRL)
	tca_ckc_setswreset(RESET_DDIBUS, 1);
	ndelay(100); // min 50ns
	tca_ckc_setpmupwroff(PMU_DDIBUSISOLATION, 0);
	tca_ckc_setpmupwroff(PMU_DDIBUSPRE, 1);
	tca_ckc_setpmupwroff(PMU_DDIBUS, 1);
#endif
}

static int ddibus_clk_set_rate(struct clk *clk, unsigned long rate)
{
	int i;
	struct clk *pclk;

	list_for_each_entry(pclk, &clock_list, list) {
		if (pclk->parent == clk && pclk->rate > rate) {
                rate = pclk->rate;
		}
	}

	for(i = 0; i < NUM_FREQS ; i++) {
		if(rate <= clk_table[i].freq) {
			rate = clk_table[i].freq;
			break;
		}
	}

	if (clk->rate != rate) {
		tca_ckc_setfbusctrl(CLKCTRL1, NOCHANGE, NORMAL_MD, clk_table[i].freq, clk_table[i].clk_src); /*FBUS_DDI*/
		clk->rate = rate;
	}

	return 0;
}

static struct clk clk_ddi = {
	.name = "ddi",
	.parent = &clk_pll2,
	.flags = CLK_AUTO_OFF,
	.enable = ddi_clk_enable,
	.disable = ddi_clk_disable,
	.set_rate = ddibus_clk_set_rate,
};

/*----------------- video image quality enhancer ------------------*/
static int viqe_clk_enable(struct clk *clk)
{
	tca_ckc_setddipwdn(DDIPWDN_VIQE, 1);
	return 0;
}

static void viqe_clk_disable(struct clk *clk)
{
	tca_ckc_setddipwdn(DDIPWDN_VIQE, 0);
}

static struct clk clk_viqe = {
	.name = "viqe",
	.parent = &clk_ddi,
	.flags = CLK_AUTO_OFF,
	.enable = viqe_clk_enable,
	.disable = viqe_clk_disable,
};

/*---------------------------- LCDC 0 -----------------------------*/
static int lcdc0_clk_enable(struct clk *clk)
{
	tca_ckc_setddipwdn(DDIPWDN_LCDC0, 1);
	tca_ckc_pclk_enable(PERI_LCD0, 1);
	return 0;
}

static void lcdc0_clk_disable(struct clk *clk)
{
	tca_ckc_pclk_enable(PERI_LCD0, 0);
	tca_ckc_setddipwdn(DDIPWDN_LCDC0, 0);
}

static int lcdc0_clk_set_rate(struct clk *clk, unsigned long rate)
{
	if (clk->rate != rate) {
		tca_ckc_setperi(PERI_LCD0, ENABLE, rate, PCDIRECTPLL1);
		clk->rate = rate;
	}

	return 0;
}

static unsigned long lcdc0_clk_get_rate(struct clk *clk)
{
	return (unsigned long)tca_ckc_getperi(PERI_LCD0);
}

static struct clk clk_lcdc0 = {
	.name = "lcdc0",
	.parent = &clk_ddi,
	.flags = CLK_AUTO_OFF,
	.enable = lcdc0_clk_enable,
	.disable = lcdc0_clk_disable, 	
	.set_rate = lcdc0_clk_set_rate,
	.get_rate = lcdc0_clk_get_rate,
};

/*---------------------------- LCDC 1 -----------------------------*/
static int lcdc1_clk_enable(struct clk *clk)
{
	tca_ckc_setddipwdn(DDIPWDN_LCDC1, 1);
	tca_ckc_pclk_enable(PERI_LCD1, 1);
	return 0;
}

static void lcdc1_clk_disable(struct clk *clk)
{
	tca_ckc_pclk_enable(PERI_LCD1, 0);
	tca_ckc_setddipwdn(DDIPWDN_LCDC1, 0);
}

static int lcdc1_clk_set_rate(struct clk *clk, unsigned long rate)
{
	if (clk->rate != rate) {
		tca_ckc_setperi(PERI_LCD1, ENABLE, rate, PCDIRECTPLL1);
		clk->rate = rate;
	}

	return 0;
}

static unsigned long lcdc1_clk_get_rate(struct clk *clk)
{
	return (unsigned long)tca_ckc_getperi(PERI_LCD1);
}

static struct clk clk_lcdc1 = {
	.name = "lcdc1",
	.parent = &clk_ddi,
	.flags = CLK_AUTO_OFF,
	.enable = lcdc1_clk_enable,
	.disable = lcdc1_clk_disable,
	.set_rate = lcdc1_clk_set_rate,
	.get_rate = lcdc1_clk_get_rate,
};

/*---------------------------- LCDSI 0 ----------------------------*/
static int lcdsi0_clk_enable(struct clk *clk)
{
	tca_ckc_setddipwdn(DDIPWDN_LCDSI0, 1);
	tca_ckc_pclk_enable(PERI_LCDSI0, 1);
	return 0;
}

static void lcdsi0_clk_disable(struct clk *clk)
{
	tca_ckc_pclk_enable(PERI_LCDSI0, 0);
	tca_ckc_setddipwdn(DDIPWDN_LCDSI0, 0);
}

static int lcdsi0_clk_set_rate(struct clk *clk, unsigned long rate)
{
	if (clk->rate != rate) {
		tca_ckc_setperi(PERI_LCDSI0, ENABLE, rate, PCDIRECTPLL1);
		clk->rate = rate;
	}

	return 0;
}

static unsigned long lcdsi0_clk_get_rate(struct clk *clk)
{
	return (unsigned long)tca_ckc_getperi(PERI_LCDSI0);
}

static struct clk clk_lcdsi0 = {
	.name = "lcdsi0",
	.parent = &clk_ddi,
	.flags = CLK_AUTO_OFF,
	.enable = lcdsi0_clk_enable,
	.disable = lcdsi0_clk_disable,
	.set_rate = lcdsi0_clk_set_rate,
	.get_rate = lcdsi0_clk_get_rate,
};

/*---------------------------- LCDSI 1 ----------------------------*/
static int lcdsi1_clk_enable(struct clk *clk)
{
	tca_ckc_setddipwdn(DDIPWDN_LCDSI1, 1);
	tca_ckc_pclk_enable(PERI_LCDSI1, 1);
	return 0;
}

static void lcdsi1_clk_disable(struct clk *clk)
{
	tca_ckc_pclk_enable(PERI_LCDSI1, 0);
	tca_ckc_setddipwdn(DDIPWDN_LCDSI1, 0);
}

static int lcdsi1_clk_set_rate(struct clk *clk, unsigned long rate)
{
	if (clk->rate != rate) {
		tca_ckc_setperi(PERI_LCDSI1, ENABLE, rate, PCDIRECTPLL1);
		clk->rate = rate;
	}

	return 0;
}

static unsigned long lcdsi1_clk_get_rate(struct clk *clk)
{
	return (unsigned long)tca_ckc_getperi(PERI_LCDSI1);
}


static struct clk clk_lcdsi1 = {
	.name = "lcdsi1",
	.parent = &clk_ddi,
	.flags = CLK_AUTO_OFF,
	.enable = lcdsi1_clk_enable,
	.disable = lcdsi1_clk_disable,
	.set_rate = lcdsi1_clk_set_rate,
	.get_rate = lcdsi1_clk_get_rate,
};

/*------------------------ Memory Scaler 0 ------------------------*/
static int m2m0_clk_enable(struct clk *clk)
{
	tca_ckc_setddipwdn(DDIPWDN_MSCL0, 1);
	return 0;
}

static void m2m0_clk_disable(struct clk *clk)
{
	tca_ckc_setddipwdn(DDIPWDN_MSCL0, 0);
}

static struct clk clk_m2m0 = {
	.name = "m2m0",
	.parent = &clk_ddi,
	.flags = CLK_AUTO_OFF,
	.enable = m2m0_clk_enable,
	.disable = m2m0_clk_disable,
};

/*------------------------ Memory Scaler 1 ------------------------*/
static int m2m1_clk_enable(struct clk *clk)
{
	tca_ckc_setddipwdn(DDIPWDN_MSCL1, 1);
	return 0;
}

static void m2m1_clk_disable(struct clk *clk)
{
	tca_ckc_setddipwdn(DDIPWDN_MSCL1, 0);
}

static struct clk clk_m2m1 = {
	.name = "m2m1",
	.parent = &clk_ddi,
	.flags = CLK_AUTO_OFF,
	.enable = m2m1_clk_enable,
	.disable = m2m1_clk_disable,
};

/*--------------------------- DDI Cache ---------------------------*/
static int ddicache_clk_enable(struct clk *clk)
{
	tca_ckc_setddipwdn(DDIPWDN_DDIC, 1);
	return 0;
}

static void ddicache_clk_disable(struct clk *clk)
{
	tca_ckc_setddipwdn(DDIPWDN_DDIC, 0);
}

static struct clk clk_ddi_cache = {
	.name = "ddi_cache",
	.parent = &clk_ddi,
	.flags = CLK_AUTO_OFF,
	.enable = ddicache_clk_enable,
	.disable = ddicache_clk_disable,
};

/*----------------------------- HDMI ------------------------------*/
static int hdmi_clk_enable(struct clk *clk)
{
	tca_ckc_setpmupwroff(PMU_HDMIPHY, 1);
	tca_ckc_setddipwdn(DDIPWDN_HDMI, 1);
	tca_ckc_pclk_enable(PERI_HDMI, 1);
	return 0;
}

static void hdmi_clk_disable(struct clk *clk)
{
	tca_ckc_pclk_enable(PERI_HDMI, 0);
	tca_ckc_setddipwdn(DDIPWDN_HDMI, 0);
	tca_ckc_setpmupwroff(PMU_HDMIPHY, 0);
}

static int hdmi_clk_set_rate(struct clk *clk, unsigned long rate)
{
	if (clk->rate != rate) {
		tca_ckc_setperi(PERI_HDMI, ENABLE, rate, PCDIRECTPLL1);
		clk->rate = rate;
	}

	return 0;
}

static unsigned long hdmi_clk_get_rate(struct clk *clk)
{
	return (unsigned long)tca_ckc_getperi(PERI_HDMI);
}

static struct clk clk_hdmi = {
	.name = "hdmi",
	.parent = &clk_ddi,
	.flags = CLK_AUTO_OFF,
	.enable = hdmi_clk_enable,
	.disable = hdmi_clk_disable,
	.set_rate = hdmi_clk_set_rate,
	.get_rate = hdmi_clk_get_rate,
};


/*===========================================================================

                             Graphic Bus Clock

===========================================================================*/

/*--------------------------- GPU Clock ---------------------------*/
static int gpu_clk_enable(struct clk *clk)
{
#if defined(USE_CLOCK_PMU_CTRL)
	if(tca_ckc_getpmupwroff(PMU_GRAPHICBUS) == 1) {
		tca_ckc_setpmupwroff(PMU_GRAPHICBUSPRE, 0);
		tca_ckc_setpmupwroff(PMU_GRAPHICBUS, 0);
		tca_ckc_setpmupwroff(PMU_GRAPHICBUSISOLATION, 1);
		ndelay(100); // min 50ns
		tca_ckc_setswreset(RESET_GRAPBUS, 0);
	}	
#endif

	tca_ckc_enable(CLKCTRL3, 1);
	return 0;
}

static void gpu_clk_disable(struct clk *clk)
{
	tca_ckc_enable(CLKCTRL3, 0);

#if defined(USE_CLOCK_PMU_CTRL)
	tca_ckc_setswreset(RESET_GRAPBUS, 1);
	ndelay(100); // min 50ns
	tca_ckc_setpmupwroff(PMU_GRAPHICBUSISOLATION, 0);
	tca_ckc_setpmupwroff(PMU_GRAPHICBUSPRE, 1);
	tca_ckc_setpmupwroff(PMU_GRAPHICBUS, 1);
#endif
}

static int gpubus_clk_set_rate(struct clk *clk, unsigned long rate)
{
	int i;
	struct clk *pclk;

	list_for_each_entry(pclk, &clock_list, list) {
		if (pclk->parent == clk && pclk->rate > rate) {
                rate = pclk->rate;
		}
	}

	for(i = 0; i < NUM_FREQS ; i++) {
		if(rate <= clk_table[i].freq) {
			rate = clk_table[i].freq;
			break;
		}
	}

	if (clk->rate != rate) {
		tca_ckc_setfbusctrl(CLKCTRL3, NOCHANGE, NORMAL_MD, clk_table[i].freq, clk_table[i].clk_src); /*FBUS_GRP*/
		clk->rate = rate;
	}

	return 0;
}	

static struct clk clk_gpu = {
	.name = "gpu",
	.parent = &clk_pll2,
	.flags = CLK_AUTO_OFF,
	.enable = gpu_clk_enable,
	.disable = gpu_clk_disable,
	.set_rate = gpubus_clk_set_rate,
};

/*----------------------------- MALI ------------------------------*/
static int mali_clk_enable(struct clk *clk)
{
	tca_ckc_setgrppwdn(1);
	return 0;
}

static void mali_clk_disable(struct clk *clk)
{
	tca_ckc_setgrppwdn(0);
}

static struct clk clk_mali = {
	.name = "mali_clk",
	.parent = &clk_gpu,
	.flags = CLK_AUTO_OFF,
	.enable = mali_clk_enable,
	.disable = mali_clk_disable,
};


/*===========================================================================

                              I/O Bus Clock

===========================================================================*/

/*---------------------------- I/O Bus ----------------------------*/
static int iobus_clk_set_rate(struct clk *clk, unsigned long rate)
{
	int i;

	for(i = 0; i < NUM_FREQS ; i++) {
		if(rate <= clk_table[i].freq) {
			rate = clk_table[i].freq;
			break;
		}
	}

	if (clk->rate != rate) {
		tca_ckc_setfbusctrl(CLKCTRL4,  ENABLE, NORMAL_MD, clk_table[i].freq, clk_table[i].clk_src); /*FBUS_IO*/
		clk->rate = rate;
	}

	return 0;
}

static struct clk clk_io = {
	.name = "iobus",
	.parent = &clk_pll1,
	.flags = CLK_ALWAYS_ENABLED,
	.set_rate = iobus_clk_set_rate,
};

/*--------------------- Prefetch/Write Buffer ---------------------*/
static int pwb_clk_enable(struct clk *clk)
{
	tca_ckc_setiobus(RB_PREFETCHWRITEBUFFER, 1);
	return 0;
}

static void pwb_clk_disable(struct clk *clk)
{
	tca_ckc_setiobus(RB_PREFETCHWRITEBUFFER, 0);
}

static struct clk clk_pwb = {
	.name = "prefetch",
	.parent = &clk_io,
	.flags = CLK_AUTO_OFF,
	.enable = pwb_clk_enable,
	.disable = pwb_clk_disable,
};

/*----------------------------- EHI 0 -----------------------------*/
static int ehi0_clk_enable(struct clk *clk)
{
	tca_ckc_setiobus(RB_EHI0, 1);
	tca_ckc_pclk_enable(PERI_EHI0, 1);
	return 0;
}

static void ehi0_clk_disable(struct clk *clk)
{
	tca_ckc_pclk_enable(PERI_EHI0, 0);
	tca_ckc_setiobus(RB_EHI0, 0);
}

static int ehi0_clk_set_rate(struct clk *clk, unsigned long rate)
{
	if (clk->rate != rate) {
		tca_ckc_setperi(PERI_EHI0, ENABLE, rate, PCDIRECTPLL1);
		clk->rate = rate;
	}

	return 0;
}

static unsigned long ehi0_clk_get_rate(struct clk *clk)
{
	return (unsigned long)tca_ckc_getperi(PERI_EHI0);
}

static struct clk clk_ehi0 = {
	.name = "ehi0",
	.parent = &clk_io,
	.flags = CLK_AUTO_OFF,
	.enable = ehi0_clk_enable,
	.disable = ehi0_clk_disable,
	.set_rate = ehi0_clk_set_rate,
	.get_rate = ehi0_clk_get_rate,
};

/*----------------------------- EHI 1 -----------------------------*/
static int ehi1_clk_enable(struct clk *clk)
{
	tca_ckc_setiobus(RB_EHI1, 1);
	tca_ckc_pclk_enable(PERI_EHI1, 1);
	return 0;
}

static void ehi1_clk_disable(struct clk *clk)
{
	tca_ckc_pclk_enable(PERI_EHI1, 0);
	tca_ckc_setiobus(RB_EHI1, 0);
}

static int ehi1_clk_set_rate(struct clk *clk, unsigned long rate)
{
	if (clk->rate != rate) {
		tca_ckc_setperi(PERI_EHI1, ENABLE, rate, PCDIRECTPLL1);
		clk->rate = rate;
	}

	return 0;
}

static unsigned long ehi1_clk_get_rate(struct clk *clk)
{
	return (unsigned long)tca_ckc_getperi(PERI_EHI1);
}

static struct clk clk_ehi1 = {
	.name = "ehi1",
	.parent = &clk_io,
	.flags = CLK_AUTO_OFF,
	.enable = ehi1_clk_enable,
	.disable = ehi1_clk_disable,
	.set_rate = ehi1_clk_set_rate,
	.get_rate = ehi1_clk_get_rate,
};

/*---------------------------- USB OTG ----------------------------*/
static int usb_otg_clk_enable(struct clk *clk)
{
	tca_ckc_setiobus(RB_USB20OTG, 1);
	return 0;
}

static void usb_otg_clk_disable(struct clk *clk)
{
	tca_ckc_setiobus(RB_USB20OTG, 0);
}

static struct clk clk_usb_otg = {
	.name = "usb_otg",
	.parent = &clk_io,
	.flags = CLK_AUTO_OFF,
	.enable = usb_otg_clk_enable,
	.disable = usb_otg_clk_disable,
};

/*------------------------------ NFC ------------------------------*/
static int nfc_clk_enable(struct clk *clk)
{
	tca_ckc_setiobus(RB_NFCCONTROLLER, 1);
	tca_ckc_pclk_enable(PERI_NFC, 1);
	return 0;
}

static void nfc_clk_disable(struct clk *clk)
{
	tca_ckc_pclk_enable(PERI_NFC, 0);
	tca_ckc_setiobus(RB_NFCCONTROLLER, 0);
}

static int nfc_clk_set_rate(struct clk *clk, unsigned long rate)
{
	if (clk->rate != rate) {
		tca_ckc_setperi(PERI_NFC, ENABLE, rate, PCDIRECTPLL1);
		clk->rate = rate;
	}

	return 0;
}

static unsigned long nfc_clk_get_rate(struct clk *clk)
{
	return (unsigned long)tca_ckc_getperi(PERI_NFC);
}

static struct clk clk_nfc = {
	.name = "nfc",
	.parent = &clk_io,
	.flags = CLK_ALWAYS_ENABLED,
	.enable = nfc_clk_enable,
	.disable = nfc_clk_disable,
	.set_rate = nfc_clk_set_rate,
	.get_rate = nfc_clk_get_rate,
};

/*--------------------------- SD/MMC CH ---------------------------*/
static int sdhc_clk_enable(struct clk *clk)
{
	tca_ckc_setiobus(RB_SDMMCPORTMUXCONTROLLER,1);
	return 0;
}

static void sdhc_clk_disable(struct clk *clk)
{
	tca_ckc_setiobus(RB_SDMMCPORTMUXCONTROLLER,0);
}

static struct clk clk_sdhc = {
	.name = "sdhc",
	.parent = &clk_io,
	.flags = CLK_AUTO_OFF,
	.enable = sdhc_clk_enable,
	.disable = sdhc_clk_disable,
};

/*--------------------------- SD/MMC 0 ----------------------------*/
static int sdhc0_clk_enable(struct clk *clk)
{
	tca_ckc_setiobus(RB_SDMMC0CONTROLLER,1);
	tca_ckc_pclk_enable(PERI_SDMMC0,1);
	return 0;
}

static void sdhc0_clk_disable(struct clk *clk)
{
	tca_ckc_pclk_enable(PERI_SDMMC0,0);
	tca_ckc_setiobus(RB_SDMMC0CONTROLLER,0);
}

static int sdhc0_clk_set_rate(struct clk *clk, unsigned long rate)
{
	if (clk->rate != rate) {
		tca_ckc_setperi(PERI_SDMMC0, ENABLE, rate, PCDIRECTPLL1);
		clk->rate = rate;
	}

	return 0;
}

static unsigned long sdhc0_clk_get_rate(struct clk *clk)
{
	return (unsigned long)tca_ckc_getperi(PERI_SDMMC0);
}

static struct clk clk_sdhc0 = {
	.name = "sdhc0",
	.parent = &clk_sdhc,
	.flags = CLK_AUTO_OFF,
	.enable = sdhc0_clk_enable,
	.disable = sdhc0_clk_disable,
	.set_rate = sdhc0_clk_set_rate,
	.get_rate = sdhc0_clk_get_rate,
};

/*--------------------------- SD/MMC 1 ----------------------------*/
static int sdhc1_clk_enable(struct clk *clk)
{
	tca_ckc_setiobus(RB_SDMMC1CONTROLLER,1);
	tca_ckc_pclk_enable(PERI_SDMMC1,1);
	return 0;
}

static void sdhc1_clk_disable(struct clk *clk)
{
	tca_ckc_pclk_enable(PERI_SDMMC1,0);
	tca_ckc_setiobus(RB_SDMMC1CONTROLLER,0);
}

static int sdhc1_clk_set_rate(struct clk *clk, unsigned long rate)
{
	if (clk->rate != rate) {
		tca_ckc_setperi(PERI_SDMMC1, ENABLE, rate, PCDIRECTPLL1);
		clk->rate = rate;
	}

	return 0;
}

static unsigned long sdhc1_clk_get_rate(struct clk *clk)
{
	return (unsigned long)tca_ckc_getperi(PERI_SDMMC1);
}

static struct clk clk_sdhc1 = {
	.name = "sdhc1",
	.parent = &clk_sdhc,
	.flags = CLK_AUTO_OFF,
	.enable = sdhc1_clk_enable,
	.disable = sdhc1_clk_disable,
	.set_rate = sdhc1_clk_set_rate,
	.get_rate = sdhc1_clk_get_rate,
};

/*--------------------------- SD/MMC 2 ----------------------------*/
static int sdhc2_clk_enable(struct clk *clk)
{
	tca_ckc_setiobus(RB_SDMMC2CONTROLLER,1);
	tca_ckc_pclk_enable(PERI_SDMMC2,1);
	return 0;
}

static void sdhc2_clk_disable(struct clk *clk)
{
	tca_ckc_pclk_enable(PERI_SDMMC2,0);
	tca_ckc_setiobus(RB_SDMMC2CONTROLLER,0);
}

static int sdhc2_clk_set_rate(struct clk *clk, unsigned long rate)
{
	if (clk->rate != rate) {
		tca_ckc_setperi(PERI_SDMMC2, ENABLE, rate, PCDIRECTPLL1);
		clk->rate = rate;
	}

	return 0;
}

static unsigned long sdhc2_clk_get_rate(struct clk *clk)
{
	return (unsigned long)tca_ckc_getperi(PERI_SDMMC2);
}

static struct clk clk_sdhc2 = {
	.name = "sdhc2",
	.parent = &clk_sdhc,
	.flags = CLK_AUTO_OFF,
	.enable = sdhc2_clk_enable,
	.disable = sdhc2_clk_disable,
	.set_rate = sdhc2_clk_set_rate,
	.get_rate = sdhc2_clk_get_rate,
};

/*---------------------------- GDMA 0 -----------------------------*/
static int gdma0_clk_enable(struct clk *clk)
{
	tca_ckc_setiobus(RB_GDMA0CONTROLLER, 1);
	return 0;
}

static void gdma0_clk_disable(struct clk *clk)
{
	tca_ckc_setiobus(RB_GDMA0CONTROLLER, 0);
}

static struct clk clk_gdma0 = {
	.name = "gdma0",
	.parent = &clk_io,
	.flags = CLK_ALWAYS_ENABLED,
	.enable = gdma0_clk_enable,
	.disable = gdma0_clk_disable,
};

/*---------------------------- GDMA 1 -----------------------------*/
static int gdma1_clk_enable(struct clk *clk)
{
	tca_ckc_setiobus(RB_GDMA1CONTROLLER, 1);
	return 0;
}

static void gdma1_clk_disable(struct clk *clk)
{
	tca_ckc_setiobus(RB_GDMA1CONTROLLER, 0);
}

static struct clk clk_gdma1 = {
	.name = "gdma1",
	.parent = &clk_io,
	.flags = CLK_ALWAYS_ENABLED,
	.enable = gdma1_clk_enable,
	.disable = gdma1_clk_disable,
};

/*---------------------------- GDMA 2 -----------------------------*/
static int gdma2_clk_enable(struct clk *clk)
{
	tca_ckc_setiobus(RB_GDMA2CONTROLLER, 1);
	return 0;
}

static void gdma2_clk_disable(struct clk *clk)
{
	tca_ckc_setiobus(RB_GDMA2CONTROLLER, 0);
}

static struct clk clk_gdma2 = {
	.name = "gdma2",
	.parent = &clk_io,
	.flags = CLK_ALWAYS_ENABLED,
	.enable = gdma2_clk_enable,
	.disable = gdma2_clk_disable,
};

/*------------------------------ PWM ------------------------------*/
static int pwm_clk_enable(struct clk *clk)
{
	tca_ckc_setiobus(RB_PWMCONTROLLER, 1);
	tca_ckc_pclk_enable(PERI_PDM, 1);
	return 0;
}

static void pwm_clk_disable(struct clk *clk)
{
	tca_ckc_pclk_enable(PERI_PDM, 0);
	tca_ckc_setiobus(RB_PWMCONTROLLER, 0);
}

static int pwm_clk_set_rate(struct clk *clk, unsigned long rate)
{
	if (clk->rate != rate) {
		tca_ckc_setperi(PERI_PDM, ENABLE, rate, PCDIRECTXIN);
		clk->rate = rate;
	}

	return 0;
}

static unsigned long pwm_clk_get_rate(struct clk *clk)
{
	return (unsigned long)tca_ckc_getperi(PERI_PDM);
}

static struct clk clk_pwm = {
	.name = "pwm",
	.parent = &clk_io,
	.flags = CLK_AUTO_OFF,
	.enable = pwm_clk_enable,
	.disable = pwm_clk_disable,
	.set_rate = pwm_clk_set_rate,
	.get_rate = pwm_clk_get_rate,
};

/*------------------------- Overlay Mixer -------------------------*/
static int overlay_clk_enable(struct clk *clk)
{
	tca_ckc_setiobus(RB_2DCONTROLLER,1);
	return 0;
}

static void overlay_clk_disable(struct clk *clk)
{
	tca_ckc_setiobus(RB_2DCONTROLLER,0);
}

static struct clk clk_overlay = {
	.name = "overlay",
	.parent = &clk_io,
	.flags = CLK_AUTO_OFF,
	.enable = overlay_clk_enable,
	.disable = overlay_clk_disable,
};

/*--------------------------- SD/MMC 3 ----------------------------*/
static int sdhc3_clk_enable(struct clk *clk)
{
	tca_ckc_setiobus(RB_SDMMC3CONTROLLER,1);
	tca_ckc_pclk_enable(PERI_SD3,1);
	return 0;
}

static void sdhc3_clk_disable(struct clk *clk)
{
	tca_ckc_pclk_enable(PERI_SD3,0);
	tca_ckc_setiobus(RB_SDMMC3CONTROLLER,0);
}

static int sdhc3_clk_set_rate(struct clk *clk, unsigned long rate)
{
	if (clk->rate != rate) {
		tca_ckc_setperi(PERI_SD3, ENABLE, rate, PCDIRECTPLL1);
		clk->rate = rate;
	}

	return 0;
}

static unsigned long sdhc3_clk_get_rate(struct clk *clk)
{
	return (unsigned long)tca_ckc_getperi(PERI_SD3);
}

static struct clk clk_sdhc3 = {
	.name = "sdhc3",
	.parent = &clk_sdhc,
	.flags = CLK_AUTO_OFF,
	.enable = sdhc3_clk_enable,
	.disable = sdhc3_clk_disable,
	.set_rate = sdhc3_clk_set_rate,
	.get_rate = sdhc3_clk_get_rate,
};

/*---------------------------- Remote -----------------------------*/
static int remote_clk_enable(struct clk *clk)
{
	tca_ckc_setiobus(RB_REMOTECONTROLLER, 1);
	return 0;
}

static void remote_clk_disable(struct clk *clk)
{
	tca_ckc_setiobus(RB_REMOTECONTROLLER, 0);
}

static struct clk clk_remote = {
	.name = "remote",
	.parent = &clk_io,
	.flags = CLK_AUTO_OFF,
	.enable = remote_clk_enable,
	.disable = remote_clk_disable,
};

/*---------------------------- TSIF 0 -----------------------------*/
static int tsif0_clk_enable(struct clk *clk)
{
	tca_ckc_setiobus(RB_TSIF0CONTROLLER, 1);
	return 0;
}

static void tsif0_clk_disable(struct clk *clk)
{
	tca_ckc_setiobus(RB_TSIF0CONTROLLER, 0);
}

static struct clk clk_tsif0 = {
	.name = "tsif0",
	.parent = &clk_io,
	.flags = CLK_AUTO_OFF,
	.enable = tsif0_clk_enable,
	.disable = tsif0_clk_disable,
};

/*---------------------------- TSIF 1 -----------------------------*/
static int tsif1_clk_enable(struct clk *clk)
{
	tca_ckc_setiobus(RB_TSIF1CONTROLLER, 1);
	return 0;
}

static void tsif1_clk_disable(struct clk *clk)
{
	tca_ckc_setiobus(RB_TSIF1CONTROLLER, 0);
}

static struct clk clk_tsif1 = {
	.name = "tsif1",
	.parent = &clk_io,
	.flags = CLK_AUTO_OFF,
	.enable = tsif1_clk_enable,
	.disable = tsif1_clk_disable,
};

/*---------------------------- ADMA 0 -----------------------------*/
static int adma0_clk_enable(struct clk *clk)
{
    tca_ckc_setiobus(RB_ADMA0CONTROLLER, 1);
	tca_ckc_set_iobus_swreset(RB_ADMA0CONTROLLER, 0);
	tca_ckc_set_iobus_swreset(RB_ADMA0CONTROLLER, 1);
    return 0;
}

static void adma0_clk_disable(struct clk *clk)
{
    tca_ckc_setiobus(RB_ADMA0CONTROLLER, 0);
}

static struct clk clk_adma0 = {
	.name = "adma0",
	.parent = &clk_io,
    .flags = CLK_AUTO_OFF,
    .enable = adma0_clk_enable,
    .disable = adma0_clk_disable,
};

/*----------------------------- DAI 0 -----------------------------*/
static int dai0_clk_enable(struct clk *clk)
{
    tca_ckc_setiobus(RB_DAI0CONTROLLER, 1);
    tca_ckc_pclk_enable(PERI_AUD, 1);
    return 0;
}

static void dai0_clk_disable(struct clk *clk)
{
    tca_ckc_pclk_enable(PERI_AUD, 0);
    tca_ckc_setiobus(RB_DAI0CONTROLLER, 0);
}

static int dai0_clk_set_rate(struct clk *clk, unsigned long rate)
{
	if (clk->rate != rate) {
		tca_ckc_setperi(PERI_AUD, ENABLE, rate, PCDIRECTPLL1);
		clk->rate = rate;
	}

	return 0;
}

static unsigned long dai0_clk_get_rate(struct clk *clk)
{
	return (unsigned long)tca_ckc_getperi(PERI_AUD);
}

static struct clk clk_dai0 = {
	.name = "dai0",
	.parent = &clk_io,
    .flags = CLK_AUTO_OFF,
    .enable = dai0_clk_enable,
    .disable = dai0_clk_disable,
    .set_rate = dai0_clk_set_rate,
    .get_rate = dai0_clk_get_rate,
};

/*---------------------------- ADMA 1 -----------------------------*/
static int adma1_clk_enable(struct clk *clk)
{
    tca_ckc_setiobus(RB_ADMA1CONTROLLER, 1);
	tca_ckc_set_iobus_swreset(RB_ADMA1CONTROLLER, 0);
	tca_ckc_set_iobus_swreset(RB_ADMA1CONTROLLER, 1);
    return 0;
}

static void adma1_clk_disable(struct clk *clk)
{
    tca_ckc_setiobus(RB_ADMA1CONTROLLER, 0);
}

static struct clk clk_adma1 = {
	.name = "adma1",
	.parent = &clk_io,
    .flags = CLK_AUTO_OFF,
    .enable = adma1_clk_enable,
    .disable = adma1_clk_disable,
};

/*----------------------------- DAI 1 -----------------------------*/
static int dai1_clk_enable(struct clk *clk)
{
    tca_ckc_setiobus(RB_DAI1CONTROLLER, 1);
    tca_ckc_pclk_enable(PERI_AUD1, 1);
    return 0;
}

static void dai1_clk_disable(struct clk *clk)
{
    tca_ckc_pclk_enable(PERI_AUD1, 0);
    tca_ckc_setiobus(RB_DAI1CONTROLLER, 0);
}

static int dai1_clk_set_rate(struct clk *clk, unsigned long rate)
{
	if (clk->rate != rate) {
		tca_ckc_setperi(PERI_AUD1, ENABLE, rate, PCDIRECTPLL1);
		clk->rate = rate;
	}

	return 0;
}

static unsigned long dai1_clk_get_rate(struct clk *clk)
{
	return (unsigned long)tca_ckc_getperi(PERI_AUD1);
}

static struct clk clk_dai1 = {
	.name = "dai1",
	.parent = &clk_io,
    .flags = CLK_AUTO_OFF,
    .enable = dai1_clk_enable,
    .disable = dai1_clk_disable,
    .set_rate = dai1_clk_set_rate,
    .get_rate = dai1_clk_get_rate,
};

/*---------------------------- UART 0 -----------------------------*/
static int uart0_clk_enable(struct clk *clk)
{
	tca_ckc_setiobus(RB_UARTCONTROLLER0, 1);
	tca_ckc_pclk_enable(PERI_UART0, 1);
	return 0;
}

static void uart0_clk_disable(struct clk *clk)
{
	tca_ckc_pclk_enable(PERI_UART0, 0);
	tca_ckc_setiobus(RB_UARTCONTROLLER0, 0);
}

static int uart0_clk_set_rate(struct clk *clk, unsigned long rate)
{
	if (clk->rate != rate) {
		tca_ckc_setperi(PERI_UART0, ENABLE, rate, PCDIRECTPLL1);
		clk->rate = rate;
	}

	return 0;
}

static unsigned long uart0_clk_get_rate(struct clk *clk)
{
	return (unsigned long)tca_ckc_getperi(PERI_UART0);
}

static struct clk clk_uart0  = {
	.name = "uart0",
	.parent = &clk_io,
	.flags = CLK_AUTO_OFF,
	.enable = uart0_clk_enable,
	.disable = uart0_clk_disable,
	.set_rate = uart0_clk_set_rate,
	.get_rate = uart0_clk_get_rate,
};

/*---------------------------- UART 1 -----------------------------*/
static int uart1_clk_enable(struct clk *clk)
{
	tca_ckc_setiobus(RB_UARTCONTROLLER1, 1);
	tca_ckc_pclk_enable(PERI_UART1, 1);
	return 0;
}

static void uart1_clk_disable(struct clk *clk)
{
	tca_ckc_pclk_enable(PERI_UART1, 0);
	tca_ckc_setiobus(RB_UARTCONTROLLER1, 0);
}

static int uart1_clk_set_rate(struct clk *clk, unsigned long rate)
{
	if (clk->rate != rate) {
		tca_ckc_setperi(PERI_UART1, ENABLE, rate, PCDIRECTPLL1);
		clk->rate = rate;
	}

	return 0;
}

static unsigned long uart1_clk_get_rate(struct clk *clk)
{
	return (unsigned long)tca_ckc_getperi(PERI_UART1);
}

static struct clk clk_uart1 = {
	.name = "uart1",
	.parent = &clk_io,
	.flags = CLK_AUTO_OFF,
	.enable = uart1_clk_enable,
	.disable = uart1_clk_disable,
	.set_rate = uart1_clk_set_rate,
	.get_rate = uart1_clk_get_rate,
};

/*---------------------------- UART 2 -----------------------------*/
static int uart2_clk_enable(struct clk *clk)
{
	tca_ckc_setiobus(RB_UARTCONTROLLER2, 1);
	tca_ckc_pclk_enable(PERI_UART2, 1);
	return 0;
}

static void uart2_clk_disable(struct clk *clk)
{
	tca_ckc_pclk_enable(PERI_UART2, 0);
	tca_ckc_setiobus(RB_UARTCONTROLLER2, 0);
}

static int uart2_clk_set_rate(struct clk *clk, unsigned long rate)
{
	if (clk->rate != rate) {
		tca_ckc_setperi(PERI_UART2, ENABLE, rate, PCDIRECTPLL1);
		clk->rate = rate;
	}

	return 0;
}

static unsigned long uart2_clk_get_rate(struct clk *clk)
{
	return (unsigned long)tca_ckc_getperi(PERI_UART2);
}

static struct clk clk_uart2 = {
	.name = "uart2",
	.parent = &clk_io,
	.flags = CLK_AUTO_OFF,
	.enable = uart2_clk_enable,
	.disable = uart2_clk_disable,
	.set_rate = uart2_clk_set_rate,
	.get_rate = uart2_clk_get_rate,
};

/*---------------------------- UART 3 -----------------------------*/
static int uart3_clk_enable(struct clk *clk)
{
	tca_ckc_setiobus(RB_UARTCONTROLLER3, 1);
	tca_ckc_pclk_enable(PERI_UART3, 1);
	return 0;
}

static void uart3_clk_disable(struct clk *clk)
{
	tca_ckc_pclk_enable(PERI_UART3, 0);
	tca_ckc_setiobus(RB_UARTCONTROLLER3, 0);
}

static int uart3_clk_set_rate(struct clk *clk, unsigned long rate)
{
	if (clk->rate != rate) {
		tca_ckc_setperi(PERI_UART3, ENABLE, rate, PCDIRECTPLL1);
		clk->rate = rate;
	}

	return 0;
}

static unsigned long uart3_clk_get_rate(struct clk *clk)
{
	return (unsigned long)tca_ckc_getperi(PERI_UART3);
}

static struct clk clk_uart3 = {
	.name = "uart3",
	.parent = &clk_io,
	.flags = CLK_AUTO_OFF,
	.enable = uart3_clk_enable,
	.disable = uart3_clk_disable,
	.set_rate = uart3_clk_set_rate,
	.get_rate = uart3_clk_get_rate,
};

/*---------------------------- UART 4 -----------------------------*/
static int uart4_clk_enable(struct clk *clk)
{
	tca_ckc_setiobus(RB_UARTCONTROLLER4, 1);
	tca_ckc_pclk_enable(PERI_UART4, 1);
	return 0;
}

static void uart4_clk_disable(struct clk *clk)
{
	tca_ckc_pclk_enable(PERI_UART4, 0);
	tca_ckc_setiobus(RB_UARTCONTROLLER4, 0);
}

static int uart4_clk_set_rate(struct clk *clk, unsigned long rate)
{
	if (clk->rate != rate) {
		tca_ckc_setperi(PERI_UART4, ENABLE, rate, PCDIRECTPLL1);
		clk->rate = rate;
	}

	return 0;
}

static unsigned long uart4_clk_get_rate(struct clk *clk)
{
	return (unsigned long)tca_ckc_getperi(PERI_UART4);
}

static struct clk clk_uart4 = {
	.name = "uart4",
	.parent = &clk_io,
	.flags = CLK_AUTO_OFF,
	.enable = uart4_clk_enable,
	.disable = uart4_clk_disable,
	.set_rate = uart4_clk_set_rate,
	.get_rate = uart4_clk_get_rate,
};

/*---------------------------- UART 5 -----------------------------*/
static int uart5_clk_enable(struct clk *clk)
{
	tca_ckc_setiobus(RB_UARTCONTROLLER5, 1);
	tca_ckc_pclk_enable(PERI_UART5, 1);
	return 0;
}

static void uart5_clk_disable(struct clk *clk)
{
	tca_ckc_pclk_enable(PERI_UART5, 0);
	tca_ckc_setiobus(RB_UARTCONTROLLER5, 0);
}

static int uart5_clk_set_rate(struct clk *clk, unsigned long rate)
{
	if (clk->rate != rate) {
		tca_ckc_setperi(PERI_UART5, ENABLE, rate, PCDIRECTPLL1);
		clk->rate = rate;
	}

	return 0;
}

static unsigned long uart5_clk_get_rate(struct clk *clk)
{
	return (unsigned long)tca_ckc_getperi(PERI_UART5);
}

static struct clk clk_uart5 = {
	.name = "uart5",
	.parent = &clk_io,
	.flags = CLK_AUTO_OFF,
	.enable = uart5_clk_enable,
	.disable = uart5_clk_disable,
	.set_rate = uart5_clk_set_rate,
	.get_rate = uart5_clk_get_rate,
};

/*----------------------------- I2C 0 -----------------------------*/
static int i2c0_clk_enable(struct clk *clk)
{
	tca_ckc_setiobus(RB_I2CCONTROLLER0, ENABLE);
	tca_ckc_pclk_enable(PERI_I2C0, 1);
	return 0;
}

static void i2c0_clk_disable(struct clk *clk)
{
	tca_ckc_pclk_enable(PERI_I2C0, 0);
	tca_ckc_setiobus(RB_I2CCONTROLLER0, DISABLE);
}

static int i2c0_clk_set_rate(struct clk *clk, unsigned long rate)
{
	if (clk->rate != rate) {
		tca_ckc_setperi(PERI_I2C0, ENABLE, rate, PCDIRECTXIN);
		clk->rate = rate;
	}

	return 0;
}

static unsigned long i2c0_clk_get_rate(struct clk *clk)
{
	return (unsigned long)tca_ckc_getperi(PERI_I2C0);
}

static struct clk clk_i2c0 = {
	.name = "i2c0",
	.parent = &clk_io,
	.flags = CLK_AUTO_OFF,
	.enable = i2c0_clk_enable,
	.disable = i2c0_clk_disable,
	.set_rate = i2c0_clk_set_rate,
	.get_rate = i2c0_clk_get_rate,
};

/*----------------------------- I2C 1 -----------------------------*/
static int i2c1_clk_enable(struct clk *clk)
{
	tca_ckc_setiobus(RB_I2CCONTROLLER1, ENABLE);
	tca_ckc_pclk_enable(PERI_I2C1, 1);
	return 0;
}

static void i2c1_clk_disable(struct clk *clk)
{
	tca_ckc_pclk_enable(PERI_I2C1, 0);
	tca_ckc_setiobus(RB_I2CCONTROLLER1, DISABLE);
}

static int i2c1_clk_set_rate(struct clk *clk, unsigned long rate)
{
	if (clk->rate != rate) {
		tca_ckc_setperi(PERI_I2C1, ENABLE, rate, PCDIRECTXIN);
		clk->rate = rate;
	}

	return 0;
}

static unsigned long i2c1_clk_get_rate(struct clk *clk)
{
	return (unsigned long)tca_ckc_getperi(PERI_I2C1);
}

static struct clk clk_i2c1 = {
	.name = "i2c1",
	.parent = &clk_io,
	.flags = CLK_AUTO_OFF,
	.enable = i2c1_clk_enable,
	.disable = i2c1_clk_disable,
	.set_rate = i2c1_clk_set_rate,
	.get_rate = i2c1_clk_get_rate,
};

/*----------------------------- I2C 2 -----------------------------*/
static int i2c2_clk_enable(struct clk *clk)
{
	tca_ckc_setiobus(RB_I2CCONTROLLER2, ENABLE);
	tca_ckc_pclk_enable(PERI_I2C2, 1);
	return 0;
}

static void i2c2_clk_disable(struct clk *clk)
{
	tca_ckc_pclk_enable(PERI_I2C2, 0);
	tca_ckc_setiobus(RB_I2CCONTROLLER2, DISABLE);
}

static int i2c2_clk_set_rate(struct clk *clk, unsigned long rate)
{
	if (clk->rate != rate) {
		tca_ckc_setperi(PERI_I2C2, ENABLE, rate, PCDIRECTXIN);
		clk->rate = rate;
	}

	return 0;
}

static unsigned long i2c2_clk_get_rate(struct clk *clk)
{
	return (unsigned long)tca_ckc_getperi(PERI_I2C2);
}

static struct clk clk_i2c2 = {
	.name = "i2c2",
	.parent = &clk_io,
	.flags = CLK_AUTO_OFF,
	.enable = i2c2_clk_enable,
	.disable = i2c2_clk_disable,
	.set_rate = i2c2_clk_set_rate,
	.get_rate = i2c2_clk_get_rate,
};

/*---------------------------- GPSB 0 -----------------------------*/
static int gpsb0_clk_enable(struct clk *clk)
{
	tca_ckc_setiobus(RB_GPSBCONTROLLER0, 1);
	tca_ckc_pclk_enable(PERI_GPSB0, 1);
	return 0;
}

static void gpsb0_clk_disable(struct clk *clk)
{
	tca_ckc_pclk_enable(PERI_GPSB0, 0);
	tca_ckc_setiobus(RB_GPSBCONTROLLER0, 0);
}

static int gpsb0_clk_set_rate(struct clk *clk, unsigned long rate)
{
	if (clk->rate != rate) {
		tca_ckc_setperi(PERI_GPSB0, ENABLE, rate, PCDIRECTPLL1);
		clk->rate = rate;
	}

	return 0;
}

static unsigned long gpsb0_clk_get_rate(struct clk *clk)
{
	return (unsigned long)tca_ckc_getperi(PERI_GPSB0);
}

static struct clk clk_gpsb0 = {
	.name = "gpsb0",
	.parent = &clk_io,
	.flags = CLK_AUTO_OFF,
	.enable = gpsb0_clk_enable,
	.disable = gpsb0_clk_disable,
	.set_rate = gpsb0_clk_set_rate,
	.get_rate = gpsb0_clk_get_rate,
};

/*---------------------------- GPSB 1 -----------------------------*/
static int gpsb1_clk_enable(struct clk *clk)
{
	tca_ckc_setiobus(RB_GPSBCONTROLLER1, 1);
	tca_ckc_pclk_enable(PERI_GPSB1, 1);
	return 0;
}

static void gpsb1_clk_disable(struct clk *clk)
{
	tca_ckc_pclk_enable(PERI_GPSB1, 0);
	tca_ckc_setiobus(RB_GPSBCONTROLLER1, 0);
}

static int gpsb1_clk_set_rate(struct clk *clk, unsigned long rate)
{
	if (clk->rate != rate) {
		tca_ckc_setperi(PERI_GPSB1, ENABLE, rate, PCDIRECTPLL1);
		clk->rate = rate;
	}

	return 0;
}

static unsigned long gpsb1_clk_get_rate(struct clk *clk)
{
	return (unsigned long)tca_ckc_getperi(PERI_GPSB1);
}

static struct clk clk_gpsb1 = {
	.name = "gpsb1",
	.parent = &clk_io,
	.flags = CLK_AUTO_OFF,
	.enable = gpsb1_clk_enable,
	.disable = gpsb1_clk_disable,
	.set_rate = gpsb1_clk_set_rate,
	.get_rate = gpsb1_clk_get_rate,
};

/*---------------------------- GPSB 2 -----------------------------*/
static int gpsb2_clk_enable(struct clk *clk)
{
	tca_ckc_setiobus(RB_GPSBCONTROLLER2, 1);
	tca_ckc_pclk_enable(PERI_GPSB2, 1);
	return 0;
}

static void gpsb2_clk_disable(struct clk *clk)
{
	tca_ckc_pclk_enable(PERI_GPSB2, 0);
	tca_ckc_setiobus(RB_GPSBCONTROLLER2, 0);
}

static int gpsb2_clk_set_rate(struct clk *clk, unsigned long rate)
{
	if (clk->rate != rate) {
		tca_ckc_setperi(PERI_GPSB2, ENABLE, rate, PCDIRECTPLL1);
		clk->rate = rate;
	}

	return 0;
}

static unsigned long gpsb2_clk_get_rate(struct clk *clk)
{
	return (unsigned long)tca_ckc_getperi(PERI_GPSB2);
}

static struct clk clk_gpsb2 = {
	.name = "gpsb2",
	.parent = &clk_io,
	.flags = CLK_AUTO_OFF,
	.enable = gpsb2_clk_enable,
	.disable = gpsb2_clk_disable,
	.set_rate = gpsb2_clk_set_rate,
	.get_rate = gpsb2_clk_get_rate,
};

/*---------------------------- GPSB 3 -----------------------------*/
static int gpsb3_clk_enable(struct clk *clk)
{
	tca_ckc_setiobus(RB_GPSBCONTROLLER3, 1);
	tca_ckc_pclk_enable(PERI_GPSB3, 1);
	return 0;
}

static void gpsb3_clk_disable(struct clk *clk)
{
	tca_ckc_pclk_enable(PERI_GPSB3, 0);
	tca_ckc_setiobus(RB_GPSBCONTROLLER3, 0);
}

static int gpsb3_clk_set_rate(struct clk *clk, unsigned long rate)
{
	if (clk->rate != rate) {
		tca_ckc_setperi(PERI_GPSB3, ENABLE, rate, PCDIRECTPLL1);
		clk->rate = rate;
	}

	return 0;
}

static unsigned long gpsb3_clk_get_rate(struct clk *clk)
{
	return (unsigned long)tca_ckc_getperi(PERI_GPSB3);
}

static struct clk clk_gpsb3 = {
	.name = "gpsb3",
	.parent = &clk_io,
	.flags = CLK_AUTO_OFF,
	.enable = gpsb3_clk_enable,
	.disable = gpsb3_clk_disable,
	.set_rate = gpsb3_clk_set_rate,
	.get_rate = gpsb3_clk_get_rate,
};

/*---------------------------- GPSB 4 -----------------------------*/
static int gpsb4_clk_enable(struct clk *clk)
{
	tca_ckc_setiobus(RB_GPSBCONTROLLER4, 1);
	tca_ckc_pclk_enable(PERI_GPSB4, 1);
	return 0;
}

static void gpsb4_clk_disable(struct clk *clk)
{
	tca_ckc_pclk_enable(PERI_GPSB4, 0);
	tca_ckc_setiobus(RB_GPSBCONTROLLER4, 0);
}

static int gpsb4_clk_set_rate(struct clk *clk, unsigned long rate)
{
	if (clk->rate != rate) {
		tca_ckc_setperi(PERI_GPSB4, ENABLE, rate, PCDIRECTPLL1);
		clk->rate = rate;
	}

	return 0;
}

static unsigned long gpsb4_clk_get_rate(struct clk *clk)
{
	return (unsigned long)tca_ckc_getperi(PERI_GPSB4);
}

static struct clk clk_gpsb4 = {
	.name = "gpsb4",
	.parent = &clk_io,
	.flags = CLK_AUTO_OFF,
	.enable = gpsb4_clk_enable,
	.disable = gpsb4_clk_disable,
	.set_rate = gpsb4_clk_set_rate,
	.get_rate = gpsb4_clk_get_rate,
};

/*---------------------------- GPSB 5 -----------------------------*/
static int gpsb5_clk_enable(struct clk *clk)
{
	tca_ckc_setiobus(RB_GPSBCONTROLLER5, 1);
	tca_ckc_pclk_enable(PERI_GPSB5, 1);
	return 0;
}

static void gpsb5_clk_disable(struct clk *clk)
{
	tca_ckc_pclk_enable(PERI_GPSB5, 0);
	tca_ckc_setiobus(RB_GPSBCONTROLLER5, 0);
}

static int gpsb5_clk_set_rate(struct clk *clk, unsigned long rate)
{
	if (clk->rate != rate) {
		tca_ckc_setperi(PERI_GPSB5, ENABLE, rate, PCDIRECTPLL1);
		clk->rate = rate;
	}

	return 0;
}

static unsigned long gpsb5_clk_get_rate(struct clk *clk)
{
	return (unsigned long)tca_ckc_getperi(PERI_GPSB5);
}

static struct clk clk_gpsb5 = {
	.name = "gpsb5",
	.parent = &clk_io,
	.flags = CLK_AUTO_OFF,
	.enable = gpsb5_clk_enable,
	.disable = gpsb5_clk_disable,
	.set_rate = gpsb5_clk_set_rate,
	.get_rate = gpsb5_clk_get_rate,
};

/*----------------------------- TSADC -----------------------------*/
static int adc_clk_enable(struct clk *clk)
{
	tca_ckc_setiobus(RB_TSADCCONTROLLER, 1);
	tca_ckc_pclk_enable(PERI_ADC, 1);
	return 0;
}

static void adc_clk_disable(struct clk *clk)
{
	tca_ckc_pclk_enable(PERI_ADC, 0);
	tca_ckc_setiobus(RB_TSADCCONTROLLER, 0);
}

static int adc_clk_set_rate(struct clk *clk, unsigned long rate)
{
	if (clk->rate != rate) {
		tca_ckc_setperi(PERI_ADC, ENABLE, rate, PCDIRECTXIN);
		clk->rate = rate;
	}

	return 0;
}

static unsigned long adc_clk_get_rate(struct clk *clk)
{
	return (unsigned long)tca_ckc_getperi(PERI_ADC);
}

static struct clk clk_adc = {
	.name = "adc",
	.parent = &clk_io,
	.flags = CLK_AUTO_OFF,
	.enable = adc_clk_enable,
	.disable = adc_clk_disable,
	.set_rate = adc_clk_set_rate,
	.get_rate = adc_clk_get_rate,
};

/*----------------------------- GPIO ------------------------------*/
static int gpio_clk_enable(struct clk *clk)
{
	tca_ckc_setiobus(RB_GPIOCONTROLLER, 1);
	return 0;
}

static void gpio_clk_disable(struct clk *clk)
{
	tca_ckc_setiobus(RB_GPIOCONTROLLER, 0);
}

static struct clk clk_gpio = {
	.name = "gpio",
	.parent = &clk_io,
	.flags = CLK_ALWAYS_ENABLED,
	.enable = gpio_clk_enable,
	.disable = gpio_clk_disable,
};

/*---------------------------- SPDIF 0 ----------------------------*/
static int spdif0_clk_enable(struct clk *clk)
{
	tca_ckc_setiobus(RB_SPDIF0CONTROLLER, 1);
	tca_ckc_pclk_enable(PERI_SPDIF, 1);
	return 0;
}

static void spdif0_clk_disable(struct clk *clk)
{
	tca_ckc_pclk_enable(PERI_SPDIF, 0);
	tca_ckc_setiobus(RB_SPDIF0CONTROLLER, 0);
}

static int spdif0_clk_set_rate(struct clk *clk, unsigned long rate)
{
	if (clk->rate != rate) {
		tca_ckc_setperi(PERI_SPDIF, ENABLE, rate, PCDIRECTPLL1);
		clk->rate = rate;
	}

	return 0;
}

static unsigned long spdif0_clk_get_rate(struct clk *clk)
{
	return (unsigned long)tca_ckc_getperi(PERI_SPDIF);
}

static struct clk clk_spdif0 = {
	.name = "spdif0",
	.parent = &clk_io,
	.flags = CLK_AUTO_OFF,
	.enable = spdif0_clk_enable,
	.disable = spdif0_clk_disable,
	.set_rate = spdif0_clk_set_rate,
	.get_rate = spdif0_clk_get_rate,
};

/*---------------------------- PROTECT ----------------------------*/
static int protect_clk_enable(struct clk *clk)
{
	tca_ckc_setiobus(RB_PROTECTCONTROLLER, 1);
	return 0;
}

static void protect_clk_disable(struct clk *clk)
{
	tca_ckc_setiobus(RB_PROTECTCONTROLLER, 0);
}

static struct clk clk_protect = {
	.name = "protect",
	.parent = &clk_io,
	.flags = CLK_AUTO_OFF,
	.enable = protect_clk_enable,
	.disable = protect_clk_disable,
};

/*---------------------------- SPDIF 1 ----------------------------*/
static int spdif1_clk_enable(struct clk *clk)
{
	tca_ckc_setiobus(RB_SPDIF1CONTROLLER, 1);
	tca_ckc_pclk_enable(PERI_SPDIF1, 1);
	return 0;
}

static void spdif1_clk_disable(struct clk *clk)
{
	tca_ckc_pclk_enable(PERI_SPDIF1, 0);
	tca_ckc_setiobus(RB_SPDIF1CONTROLLER, 0);
}

static int spdif1_clk_set_rate(struct clk *clk, unsigned long rate)
{
	if (clk->rate != rate) {
		tca_ckc_setperi(PERI_SPDIF1, ENABLE, rate, PCDIRECTPLL1);
		clk->rate = rate;
	}

	return 0;
}

static unsigned long spdif1_clk_get_rate(struct clk *clk)
{
	return (unsigned long)tca_ckc_getperi(PERI_SPDIF1);
}

static struct clk clk_spdif1 = {
	.name = "spdif1",
	.parent = &clk_io,
	.flags = CLK_AUTO_OFF,
	.enable = spdif1_clk_enable,
	.disable = spdif1_clk_disable,
	.set_rate = spdif1_clk_set_rate,
	.get_rate = spdif1_clk_get_rate,
};


/*===========================================================================

                              Video Bus Clock

===========================================================================*/

/*--------------------------- Video Bus ---------------------------*/
static int vpubus_clk_set_rate(struct clk *clk, unsigned long rate)
{
	int i;
	struct clk *pclk;

	list_for_each_entry(pclk, &clock_list, list) {
		if (pclk->parent == clk && pclk->rate > rate) {
				rate = pclk->rate;
		}
	}

	for(i = 0; i < NUM_FREQS ; i++) {
		if(rate <= clk_table[i].freq) {
			rate = clk_table[i].freq;
			break;
		}
	}

	if (clk->rate != rate) {
		tca_ckc_setfbusctrl(CLKCTRL5, NOCHANGE, NORMAL_MD, clk_table[i].freq, clk_table[i].clk_src); /*FBUS_VBUS*/
		clk->rate = rate;
	}

	return 0;
}

static int vbus_clk_enable(struct clk *clk)
{
#if defined(USE_CLOCK_PMU_CTRL)
	if(tca_ckc_getpmupwroff(PMU_VIDEOBUS) == 1) {
		tca_ckc_setpmupwroff(PMU_VIDEOBUSPRE, 0);
		tca_ckc_setpmupwroff(PMU_VIDEOBUS, 0);
		tca_ckc_setpmupwroff(PMU_VIDEOBUSISOLATION, 1);
		ndelay(100); // min 50ns
		tca_ckc_setswreset(RESET_VIDEOBUS, 0);
		tca_ckc_setswreset(RESET_VIDEOCORE, 0);
	}
#endif

	tca_ckc_enable(CLKCTRL5, 1);

	return 0;
}

static void vbus_clk_disable(struct clk *clk)
{
	tca_ckc_enable(CLKCTRL5, 0);

#if defined(USE_CLOCK_PMU_CTRL)
	tca_ckc_setswreset(RESET_VIDEOCORE, 1);
	tca_ckc_setswreset(RESET_VIDEOBUS, 1);
	ndelay(100); // min 50ns
	tca_ckc_setpmupwroff(PMU_VIDEOBUSISOLATION, 0);
	tca_ckc_setpmupwroff(PMU_VIDEOBUSPRE, 1);
	tca_ckc_setpmupwroff(PMU_VIDEOBUS, 1);
#endif
}

static struct clk clk_vbus = {
	.name = "vbus",
	.parent = &clk_pll5,
	.flags = CLK_AUTO_OFF,
	.enable = vbus_clk_enable,
	.disable = vbus_clk_disable,
	.set_rate = vpubus_clk_set_rate,
};

/*------------------- Video Cache in Video Bus --------------------*/
 static int vpu_vcache_clk_set_rate(struct clk *clk, unsigned long rate)
{
	clk->rate = rate;
	if(rate == 0)
		return 0;

	return _clk_set_rate(clk->parent, rate);
}

static int vcache_clk_enable(struct clk *clk)
{
    tca_ckc_setvideobuscfgpwdn(VIDEOBUSCFG_PWDNVIDEOCACHE, 1);
	return 0;
}

static void vcache_clk_disable(struct clk *clk)
{
	tca_ckc_setvideobuscfgpwdn(VIDEOBUSCFG_PWDNVIDEOCACHE, 0);
}

static struct clk clk_vcache = {
	.name = "vcache",
    .parent = &clk_vbus,
    .flags = CLK_AUTO_OFF,
    .enable = vcache_clk_enable,
    .disable = vcache_clk_disable,
    .set_rate = vpu_vcache_clk_set_rate,
};

/*------------------- Video Codec in Video Bus --------------------*/
 static int vpu_vcodec_clk_set_rate(struct clk *clk, unsigned long rate)
{
	clk->rate = rate;
	if(rate == 0)
		return 0;

	return _clk_set_rate(clk->parent, rate);
}

static int vcodec_clk_enable(struct clk *clk)
{
    tca_ckc_setvideobuscfgpwdn(VIDEOBUSCFG_PWDNVIDEOCODEC, 1);
	return 0;
}

static void vcodec_clk_disable(struct clk *clk)
{
    tca_ckc_setvideobuscfgpwdn(VIDEOBUSCFG_PWDNVIDEOCODEC, 0);
}

static struct clk clk_vcodec = {
	.name = "vcodec",
    .parent = &clk_vbus,
    .flags = CLK_AUTO_OFF,
    .enable = vcodec_clk_enable,
    .disable = vcodec_clk_disable,
	.set_rate = vpu_vcodec_clk_set_rate,
};

/*------------------- Jpeg Encoder in Video Bus -------------------*/
static int vpu_jpege_clk_set_rate(struct clk *clk, unsigned long rate)
{
	clk->rate = rate;
	if(rate == 0)
		return 0;

	return _clk_set_rate(clk->parent, rate);
}

static int jpege_clk_enable(struct clk *clk)
{
    tca_ckc_setvideobuscfgpwdn(VIDEOBUSCFG_PWDNJPEGENC, 1);
	return 0;
}

static void jpege_clk_disable(struct clk *clk)
{
    tca_ckc_setvideobuscfgpwdn(VIDEOBUSCFG_PWDNJPEGENC, 0);
}

static struct clk clk_jpege = {
	.name = "jpege",
    .parent = &clk_vbus,
    .flags = CLK_AUTO_OFF,
    .enable = jpege_clk_enable,
    .disable = jpege_clk_disable,
	.set_rate = vpu_jpege_clk_set_rate,
};

/*-------------------------- Video Core ---------------------------*/
static int vcorebus_clk_set_rate(struct clk *clk, unsigned long rate)
{
	int i;

	for(i = 0; i < NUM_FREQS ; i++) {
		if(rate <= clk_table[i].freq) {
			rate = clk_table[i].freq;
			break;
		}
	}

	if (clk->rate != rate) {
		tca_ckc_setfbusctrl(CLKCTRL6, NOCHANGE, NORMAL_MD, clk_table[i].freq, clk_table[i].clk_src); /*FBUS_VCore*/
		clk->rate = rate;
	}
	return 0;
}

static int vcore_clk_enable(struct clk *clk)
{
	tca_ckc_enable(CLKCTRL6, 1);
	return 0;
}

static void vcore_clk_disable(struct clk *clk)
{
	tca_ckc_enable(CLKCTRL6, 0);
}

static struct clk clk_vcore = {
	.name = "vcore",
	.parent = &clk_pll4,
	.flags = CLK_AUTO_OFF,
	.enable = vcore_clk_enable,
	.disable = vcore_clk_disable,
	.set_rate = vcorebus_clk_set_rate,
};

/*--------------------------- Video DAC ---------------------------*/
static int videodac_enable(struct clk *clk)
{
	tca_ckc_setpmupwroff(PMU_VIDEODAC, 1);
	return 0;
}

static void videodac_disable(struct clk *clk)
{
	tca_ckc_setpmupwroff(PMU_VIDEODAC, 0);
}

static struct clk clk_videodac = {
	.name = "videodac",
	.flags = CLK_AUTO_OFF,
	.enable = videodac_enable,
	.disable = videodac_disable,
};


/*===========================================================================

                        High Speed I/O Bus Clock

===========================================================================*/

/*------------------------ HSIO Bus Clock -------------------------*/
static int hsio_clk_enable(struct clk *clk)
{
#if defined(USE_CLOCK_PMU_CTRL)
	if(tca_ckc_getpmupwroff(PMU_HSBUS) == 1) {
		tca_ckc_setpmupwroff(PMU_HSBUSPRE, 0);
		tca_ckc_setpmupwroff(PMU_HSBUS, 0);
		tca_ckc_setpmupwroff(PMU_HSBUSISOLATION, 1);
		ndelay(100); // min 50ns
		tca_ckc_setswreset(RESET_HSBUS, 0);
	}
#endif

	tca_ckc_enable(CLKCTRL8, 1);

	return 0;
}

static void hsio_clk_disable(struct clk *clk)
{
	tca_ckc_enable(CLKCTRL8, 0);

#if defined(USE_CLOCK_PMU_CTRL)
	tca_ckc_setswreset(RESET_HSBUS, 1);
	ndelay(100); // min 50ns
	tca_ckc_setpmupwroff(PMU_HSBUSISOLATION, 0);
	tca_ckc_setpmupwroff(PMU_HSBUSPRE, 1);
	tca_ckc_setpmupwroff(PMU_HSBUS, 1);
#endif
}

static int hsiobus_clk_set_rate(struct clk *clk, unsigned long rate)
{
	int i;
	struct clk *pclk;

	list_for_each_entry(pclk, &clock_list, list) {
		if (pclk->parent == clk && pclk->rate > rate) {
                rate = pclk->rate;
		}
	}

	for(i = 0; i < NUM_FREQS ; i++) {
		if(rate <= clk_table[i].freq) {
			rate = clk_table[i].freq;
			break;
		}
	}

	if (clk->rate != rate) {
		tca_ckc_setfbusctrl(CLKCTRL8, NOCHANGE, NORMAL_MD, clk_table[i].freq, clk_table[i].clk_src); /*FBUS_HSIO*/
		clk->rate = rate;
	}

	return 0;
}

static struct clk clk_hsio = {
	.name = "hsio",
	.parent = &clk_pll1,
	.flags = CLK_AUTO_OFF,
	.enable = hsio_clk_enable,
	.disable = hsio_clk_disable,
	.set_rate = hsiobus_clk_set_rate,
};

/*--------------------------- SATA COMMON----------------------------*/
static int sata_clk_enable(struct clk *clk)
{
	tca_ckc_pclk_enable(PERI_SATA, 1);
	tca_ckc_pclk_enable(PERI_SATA_REF, 1);
	return 0;
}

static void sata_clk_disable(struct clk *clk)
{
	tca_ckc_pclk_enable(PERI_SATA_REF, 0);
	tca_ckc_pclk_enable(PERI_SATA, 0);
}

static struct clk clk_sata = {
	.name = "sata_common",
	.parent = &clk_hsio,
	.flags = CLK_AUTO_OFF,	
	.enable = sata_clk_enable,
	.disable = sata_clk_disable,
};

/*------------------------------- PWB -------------------------------*/
static int hs_pwb_clk_enable(struct clk *clk)
{
	tca_ckc_sethsiobus(HSIO_PWB, 1);
	return 0;
}

static void hs_pwb_clk_disable(struct clk *clk)
{
	tca_ckc_sethsiobus(HSIO_PWB, 0);
}

static struct clk clk_hs_pwb = {
	.name = "hs_pwb",
	.parent = &clk_hsio,
	.flags = CLK_AUTO_OFF,
	.enable = hs_pwb_clk_enable,
	.disable = hs_pwb_clk_disable,
};

/*------------------------------ SATAD ------------------------------*/
static int satad_clk_enable(struct clk *clk)
{
	tca_ckc_sethsiobus(HSIO_SATAD, 1);
	return 0;
}

static void satad_clk_disable(struct clk *clk)
{
	tca_ckc_sethsiobus(HSIO_SATAD, 0);
}

static struct clk clk_satad = {
	.name = "satad",
	.flags = CLK_AUTO_OFF,	
	.parent = &clk_sata,
	.enable = satad_clk_enable,
	.disable = satad_clk_disable,
};

/*------------------------------ GMAC -------------------------------*/
static int gmac_clk_enable(struct clk *clk)
{
	tca_ckc_sethsiobus(HSIO_GMAC, 1);
	tca_ckc_pclk_enable(PERI_GMAC, 1);
	return 0;
}

static void gmac_clk_disable(struct clk *clk)
{
	tca_ckc_pclk_enable(PERI_GMAC, 0);
	tca_ckc_sethsiobus(HSIO_GMAC, 0);
}

static struct clk clk_gmac = {
	.name = "gmac",
	.flags = CLK_AUTO_OFF,	
	.parent = &clk_hsio,
	.enable = gmac_clk_enable,
	.disable = gmac_clk_disable,
};

/*----------------------------- USBOTG ------------------------------*/
static int hs_usb_otg_clk_enable(struct clk *clk)
{
	tca_ckc_sethsiobus(HSIO_USBOTG, 1);
	return 0;
}

static void hs_usb_otg_clk_disable(struct clk *clk)
{
	tca_ckc_sethsiobus(HSIO_USBOTG, 0);
}

static struct clk clk_hs_usb_otg = {
	.name = "hs_usb_otg",
	.flags = CLK_AUTO_OFF,	
	.enable = hs_usb_otg_clk_enable,
	.disable = hs_usb_otg_clk_disable,
};

/*------------------------------ GDMA -------------------------------*/
static int hs_gdma_clk_enable(struct clk *clk)
{
	tca_ckc_sethsiobus(HSIO_GDMA, 1);
	return 0;
}

static void hs_gdma_clk_disable(struct clk *clk)
{
	tca_ckc_sethsiobus(HSIO_GDMA, 0);
}

static struct clk clk_hs_gdma = {
	.name = "hs_gdma",
	.parent = &clk_hsio,
	.flags = CLK_AUTO_OFF,
	.enable = hs_gdma_clk_enable,
	.disable = hs_gdma_clk_disable,
};

/*----------------------------- MSTICK ------------------------------*/
static int mstick_clk_enable(struct clk *clk)
{
	tca_ckc_sethsiobus(HSIO_MSTICK, 1);
	tca_ckc_pclk_enable(PERI_MSTICK, 1);
	return 0;
}

static void mstick_clk_disable(struct clk *clk)
{
	tca_ckc_pclk_enable(PERI_MSTICK, 0);
	tca_ckc_sethsiobus(HSIO_MSTICK, 0);
}

static int mstick_clk_set_rate(struct clk *clk, unsigned long rate)
{
	if (clk->rate != rate) {
		tca_ckc_setperi(PERI_MSTICK, ENABLE, rate, PCDIRECTPLL1);
		clk->rate = rate;
	}

	return 0;
}

static unsigned long mstick_clk_get_rate(struct clk *clk)
{
	return (unsigned long)tca_ckc_getperi(PERI_MSTICK);
}

static struct clk clk_mstick = {
	.name = "mstick",
	.flags = CLK_AUTO_OFF,	
	.enable = mstick_clk_enable,
	.disable = mstick_clk_disable,
	.set_rate = mstick_clk_set_rate,
	.get_rate = mstick_clk_get_rate,
};

/*----------------------------- CIPHER ------------------------------*/
static int cipher_clk_enable(struct clk *clk)
{
	tca_ckc_sethsiobus(HSIO_CIPHER, 1);
	return 0;
}

static void cipher_clk_disable(struct clk *clk)
{
	tca_ckc_sethsiobus(HSIO_CIPHER, 0);
}

static struct clk clk_cipher = {
	.name = "cipher",
	.parent = &clk_hsio,
	.flags = CLK_AUTO_OFF,	
	.enable = cipher_clk_enable,
	.disable = cipher_clk_disable,
};

/*------------------------------ SATAH ------------------------------*/
static int satah_clk_enable(struct clk *clk)
{
	tca_ckc_sethsiobus(HSIO_SATAH, 1);
	return 0;
}

static void satah_clk_disable(struct clk *clk)
{
	tca_ckc_sethsiobus(HSIO_SATAH, 0);
}

static struct clk clk_satah = {
	.name = "satah",
	.parent = &clk_sata,
	.flags = CLK_AUTO_OFF,	
	.enable = satah_clk_enable,
	.disable = satah_clk_disable,
};


/*===========================================================================

                           Camera Bus Clock

===========================================================================*/

/*----------------------- Camera Bus Clock ------------------------*/
static int cam_clk_enable(struct clk *clk)
{
#if defined(USE_CLOCK_PMU_CTRL)
	if(tca_ckc_getpmupwroff(PMU_CAMBUS) == 1) {
		tca_ckc_setpmupwroff(PMU_CAMBUSPRE, 0);
		tca_ckc_setpmupwroff(PMU_CAMBUS, 0);
		tca_ckc_setpmupwroff(PMU_CAMBUSISOLATION, 1);
		ndelay(100); // min 50ns
		tca_ckc_setswreset(RESET_CAMBUS, 0);
	}
#endif

	tca_ckc_enable(CLKCTRL9, 1);

	return 0;
}

static void cam_clk_disable(struct clk *clk)
{
	tca_ckc_enable(CLKCTRL9, 0);

#if defined(USE_CLOCK_PMU_CTRL)
	tca_ckc_setswreset(RESET_CAMBUS, 1);
	ndelay(100); // min 50ns
	tca_ckc_setpmupwroff(PMU_CAMBUSISOLATION, 0);
	tca_ckc_setpmupwroff(PMU_CAMBUSPRE, 1);
	tca_ckc_setpmupwroff(PMU_CAMBUS, 1);
#endif
}

static int cambus_clk_set_rate(struct clk *clk, unsigned long rate)
{
	int i;
	struct clk *pclk;

	list_for_each_entry(pclk, &clock_list, list) {
		if (pclk->parent == clk && pclk->rate > rate) {
                rate = pclk->rate;
		}
	}

	for(i = 0; i < NUM_FREQS ; i++) {
		if(rate <= clk_table[i].freq) {
			rate = clk_table[i].freq;
			break;
		}
	}

	if (clk->rate != rate) {
		tca_ckc_setfbusctrl(CLKCTRL9, NOCHANGE, NORMAL_MD, clk_table[i].freq, clk_table[i].clk_src); /*FBUS_CAM*/
		clk->rate = rate;
	}

	return 0;
}

static struct clk clk_cam = {
	.name = "cam",
	.parent = &clk_pll1,
	.flags = CLK_AUTO_OFF,
	.enable = cam_clk_enable,
	.disable = cam_clk_disable,
	.set_rate = cambus_clk_set_rate,
};

/*--------------------------- CIF Clock ---------------------------*/
static int cif_clk_enable(struct clk *clk)
{
	tca_ckc_setcambuspwdn(CAMBUS_CIF, 1);
	return 0;
}

static void cif_clk_disable(struct clk *clk)
{
	tca_ckc_setcambuspwdn(CAMBUS_CIF, 0);
}

static struct clk clk_cif = {
	.name = "cif",
	.parent = &clk_cam,
	.flags = CLK_AUTO_OFF,
	.enable = cif_clk_enable,
	.disable = cif_clk_disable,
};

/*--------------------------- ISP Clock ---------------------------*/
static int isp_clk_enable(struct clk *clk)
{
	tca_ckc_setcambuspwdn(CAMBUS_ISP, 1);
	return 0;
}

static void isp_clk_disable(struct clk *clk)
{
	tca_ckc_setcambuspwdn(CAMBUS_ISP, 0);
}

static struct clk clk_isp = {
	.name = "isp",
	.parent = &clk_cam,
	.flags = CLK_AUTO_OFF,
	.enable = isp_clk_enable,
	.disable = isp_clk_disable,
};

/*------------------------- CIF MC clock --------------------------*/
static int cifmc_clk_enable(struct clk *clk)
{
    tca_ckc_pclk_enable(PERI_CIFMC, 1);
	return 0;
}

static void cifmc_clk_disable(struct clk *clk)
{
    tca_ckc_pclk_enable(PERI_CIFMC, 0);
}

static int cifmc_clk_set_rate(struct clk *clk, unsigned long rate)
{
	if (clk->rate != rate) {
		tca_ckc_setperi(PERI_CIFMC, ENABLE, rate, PCDIRECTPLL1);
		clk->rate = rate;
	}

	return 0;
}

static unsigned long cifmc_clk_get_rate(struct clk *clk)
{
	return (unsigned long)tca_ckc_getperi(PERI_CIFMC);
}

static struct clk clk_cifmc = {
	.name = "cifmc",
	.parent = &clk_cif,
    .flags = CLK_AUTO_OFF,
    .enable = cifmc_clk_enable,
    .disable = cifmc_clk_disable,
    .set_rate = cifmc_clk_set_rate,
    .get_rate = cifmc_clk_get_rate,
};

/*----------------------- CIF Scaler clock ------------------------*/
static int cifsc_clk_enable(struct clk *clk)
{
    tca_ckc_pclk_enable(PERI_CIFSC, 1);
	return 0;
}

static void cifsc_clk_disable(struct clk *clk)
{
    tca_ckc_pclk_enable(PERI_CIFSC, 0);
}

static int cifsc_clk_set_rate(struct clk *clk, unsigned long rate)
{
	if (clk->rate != rate) {
		tca_ckc_setperi(PERI_CIFSC, ENABLE, rate, PCDIRECTPLL1);
		clk->rate = rate;
	}

	return 0;
}

static unsigned long cifsc_clk_get_rate(struct clk *clk)
{
	return (unsigned long)tca_ckc_getperi(PERI_CIFSC);
}

static struct clk clk_cifsc = {
	.name = "cifsc",
	.parent = &clk_cif,
    .flags = CLK_AUTO_OFF,
    .enable = cifsc_clk_enable,
    .disable = cifsc_clk_disable,
};

/*----------------------- ISP System clock ------------------------*/
static int isps_clk_enable(struct clk *clk)
{
	tca_ckc_pclk_enable(PERI_ISPS, 1);
	return 0;
}

static void isps_clk_disable(struct clk *clk)
{
	tca_ckc_pclk_enable(PERI_ISPS, 0);
}

static struct clk clk_isps = {
	.name = "isps",
	.parent = &clk_isp,
	.flags = CLK_AUTO_OFF,
	.enable = isps_clk_enable,
	.disable = isps_clk_disable,		
};

/*----------------------- ISP JPEG clock ------------------------*/
static int ispj_clk_enable(struct clk *clk)
{
	tca_ckc_pclk_enable(PERI_ISPJ, 1);
	return 0;
}

static void ispj_clk_disable(struct clk *clk)
{
	tca_ckc_pclk_enable(PERI_ISPJ, 0);
}

static struct clk clk_ispj = {
	.name = "ispj",
	.parent = &clk_isp,
	.flags = CLK_AUTO_OFF,
	.enable = ispj_clk_enable,
	.disable = ispj_clk_disable,		
};


/*===========================================================================

                           DDI Sub Bus Clock

===========================================================================*/

/*----------------------- DDI Sub Bus Clock -----------------------*/
static int ddisub_clk_enable(struct clk *clk)
{
#if defined(USE_CLOCK_PMU_CTRL)
	if(tca_ckc_getpmupwroff(PMU_DDISUBBUS) == 1) {
		tca_ckc_setpmupwroff(PMU_DDISUBBUSPRE, 0);
		tca_ckc_setpmupwroff(PMU_DDISUBBUS, 0);
		tca_ckc_setpmupwroff(PMU_DDISUBBUSISOLATION, 1);
		ndelay(100); // min 50ns
		tca_ckc_setswreset(RESET_DDISUBBUS, 0);
	}
#endif

	tca_ckc_enable(CLKCTRL10, 1);

	return 0;
}

static void ddisub_clk_disable(struct clk *clk)
{
	tca_ckc_enable(CLKCTRL10, 0);

#if defined(USE_CLOCK_PMU_CTRL)
	tca_ckc_setswreset(RESET_DDISUBBUS, 1);
	ndelay(100); // min 50ns
	tca_ckc_setpmupwroff(PMU_DDISUBBUSISOLATION, 0);
	tca_ckc_setpmupwroff(PMU_DDISUBBUSPRE, 1);
	tca_ckc_setpmupwroff(PMU_DDISUBBUS, 1);
#endif
}

static int ddisubbus_clk_set_rate(struct clk *clk, unsigned long rate)
{
	int i;
	struct clk *pclk;
	
	list_for_each_entry(pclk, &clock_list, list) {
		if (pclk->parent == clk && pclk->rate > rate) {
			rate = pclk->rate;
		}
	}

	for(i = 0; i < NUM_FREQS ; i++) {
		if(rate <= clk_table[i].freq) {
			rate = clk_table[i].freq;
			break;
		}
	}

	if (clk->rate != rate) {
		tca_ckc_setfbusctrl(CLKCTRL10, NOCHANGE, NORMAL_MD, clk_table[i].freq, clk_table[i].clk_src); /*FBUS_DDISUB*/
		clk->rate = rate;
	}

	return 0;
}

static struct clk clk_ddisub = {
	.name = "ddisub",
	.parent = &clk_pll2,
	.flags = CLK_AUTO_OFF,	
	.enable = ddisub_clk_enable,
	.disable = ddisub_clk_disable,
	.set_rate = ddisubbus_clk_set_rate,
};

/*-------------------------- MIPI Clock ---------------------------*/
static int mipi_clk_enable(struct clk *clk)
{
	tca_ckc_setddisubbus(DDISUBBUS_HCLK_MIPIDSI, 1);
	return 0;
}

static void mipi_clk_disable(struct clk *clk)
{
	tca_ckc_setddisubbus(DDISUBBUS_HCLK_MIPIDSI, 0);
}

static struct clk clk_mipi = {
	.name = "mipi",
	.parent = &clk_ddisub,
	.flags = CLK_AUTO_OFF,
	.enable = mipi_clk_enable,
	.disable = mipi_clk_disable,		
};

/*------------------------ MIPI EHI Clock -------------------------*/
static int mipi_ehi_clk_enable(struct clk *clk)
{
	tca_ckc_setddisubbus(DDISUBBUS_HCLK_EHI, 1);
	return 0;
}

static void mipi_ehi_clk_disable(struct clk *clk)
{
	tca_ckc_setddisubbus(DDISUBBUS_HCLK_EHI, 0);
}

static struct clk clk_mipi_ehi = {
	.name = "mipi_ehi",
	.parent = &clk_ddisub,
	.flags = CLK_AUTO_OFF,
	.enable = mipi_ehi_clk_enable,
	.disable = mipi_ehi_clk_disable,		
};












/*===========================================================================

                          Register Clock Driver

===========================================================================*/

static struct clk *onchip_clocks[] = {
	// clock source
	&clk_xin,
//	&clk_xtin,
	&clk_pll0,
	&clk_pll1,
	&clk_pll2,
	&clk_pll3,
	&clk_pll4,
	&clk_pll5,

	// System clocks
	&clk_cpu,
	&clk_membus,
	&clk_smu,

	// Display Bus
	&clk_ddi,
	&clk_viqe,			// bit 1
	&clk_lcdc0,			// bit 2
	&clk_lcdc1,			// bit 3
	&clk_lcdsi0,		// bit 4
	&clk_m2m0,			// bit 5
	&clk_m2m1,			// bit 6
	&clk_ddi_cache,		// bit 7
	&clk_hdmi,			// bit 8
	&clk_lcdsi1,		// bit 9

	// Graphic Bus
	&clk_mali,			// bit 0
	&clk_gpu,

	// IO Bus
//	&clk_pwb,			// bit 0
	&clk_ehi0,			// bit 1
	&clk_ehi1,			// bit 2
	&clk_usb_otg,		// bit 3
//	&clk_nfc,			// bit 4
	&clk_sdhc,			// bit 5
	&clk_sdhc0,			// bit 6
	&clk_sdhc1,			// bit 7
	&clk_sdhc2,			// bit 8
//	&clk_gdma0,			// bit 9
//	&clk_gdma1,			// bit 10
//	&clk_gdma2,			// bit 11
//	&clk_pwm,			// bit 12
	&clk_overlay,		// bit 13
	&clk_sdhc3,			// bit 14
//	&clk_remote,		// bit 15
//	&clk_tsif0,			// bit 16
//	&clk_tsif1,			// bit 17
	&clk_adma0,			// bit 18
	&clk_dai0,			// bit 19
//	&clk_adma1,			// bit 20
//	&clk_dai1,			// bit 21
	&clk_uart0,			// bit 22
	&clk_uart1,			// bit 23
	&clk_uart2,			// bit 24
	&clk_uart3,			// bit 25
	&clk_uart4,			// bit 26
	&clk_uart5,			// bit 27
	&clk_i2c0,			// bit 28
	&clk_i2c1,			// bit 29
	&clk_i2c2,			// bit 30
	&clk_gpsb0,			// bit 31
	&clk_gpsb1,			// bit 32
	&clk_gpsb2,			// bit 33
	&clk_gpsb3,			// bit 34
	&clk_gpsb4,			// bit 35
	&clk_gpsb5,			// bit 36
	&clk_adc,			// bit 37
//	&clk_gpio,			// bit 38
	&clk_spdif0,		// bit 39
//	&clk_protect,		// bit 40
	&clk_spdif1,		// bit 41
	&clk_io,

	// HSIO Bus
	&clk_sata,
//	&clk_hs_pwb,		// bit 0
	&clk_satad,			// bit 1
	&clk_gmac,			// bit 2
	&clk_hs_usb_otg,	// bit 3
	&clk_hs_gdma,		// bit 4
	&clk_mstick,		// bit 5
	&clk_cipher,		// bit 6
	&clk_satah,			// bit 7
	&clk_hsio,


	// Video Bus
	&clk_jpege,			// bit 0
	&clk_vcodec,		// bit 2
	&clk_vcache,		// bit 3
	&clk_videodac,
	&clk_vcore,
	&clk_vbus,

	// Camera Bus
	&clk_cif,			// bit 0
	&clk_isp,			// bit 1
	&clk_cifmc,
	&clk_cifsc,
	&clk_isps,
	&clk_ispj,
	&clk_cam,

	// Display Sub Bus
	&clk_mipi_ehi,		// bit 0
	&clk_mipi,			// bit 1
	&clk_ddisub,

#if 0
	// peripheral clocks
	&clk_tcx,		// 0x080
	&clk_tct,
	&clk_tcz,
//	&clk_lcdc0,
//	&clk_lcdc1,		// 0x090
//	&clk_lcdsi0,
//	&clk_cifmc,
//	&clk_cifsc,
	&clk_out0,		// 0xA0
	&clk_out1
//	&clk_hdmi,
//	&clk_sdhc2,
//	&clk_sdhc0,	// 0xB0
//	&clk_mstick,
//	&clk_i2c0,
//	&clk_uart0,
//	&clk_uart1,		// 0x0C0
//	&clk_uart2,
//	&clk_uart3,
//	&clk_uart4,
//	&clk_uart5,		// 0x0D0
//	&clk_gpsb0,
//	&clk_gpsb1,
//	&clk_gpsb2,
//	&clk_gpsb3,		// 0x0E0
//	&clk_gpsb4,
//	&clk_gpsb5,
//	&clk_adc,
//	&clk_spdif0,	// 0x0F0
//	&clk_ehi0,
//	&clk_ehi1,
//	&clk_dai0,
//	&clk_pdm,		// 0x100
//	&clk_lcdsi1,
//	&clk_sdhc1,
//	&clk_i2c1,
//	&clk_dai1,		// 0x110
//	&clk_i2c2,
//	&clk_isps,
//	&clk_ispj,
//	&clk_nfc,		// 0x120
	&clk_gmac,
//	&clk_sata,
//	&clk_sdhc3,
//	&clk_spdif1,	// 0x130
//	&clk_sata_ref,
#endif
};

struct clk *clk_get(struct device *dev, const char *id)
{
	struct clk *clk;

	mutex_lock(&clock_mutex);
	list_for_each_entry(clk, &clock_list, list) {
		if (!strcmp(id, clk->name) && clk->dev == dev)
			goto found;
		if (!strcmp(id, clk->name) && clk->dev == NULL)
			goto found;
	}
	clk = ERR_PTR(-ENOENT);
found:
	mutex_unlock(&clock_mutex);
	return clk;
}
EXPORT_SYMBOL(clk_get);

void clk_put(struct clk *clk)
{
	/* do nothing */
}
EXPORT_SYMBOL(clk_put);

static int _clk_enable(struct clk *clk)
{
	int ret = 0;

	clk->usecount++;
	if (clk->usecount == 1) {
		if (clk->parent)
			ret = _clk_enable(clk->parent);

		if (clk->enable)
			ret = clk->enable(clk);
		else
			pr_warning("clock: %s: unimplemented function 'clk_enable' called\n", clk->name);
	}
	return ret;
}

int clk_enable(struct clk *clk)
{
	unsigned long flags;
	int ret = 0;

	BUG_ON(clk == 0);

	spin_lock_irqsave(&clock_lock, flags);
	ret = _clk_enable(clk);
	spin_unlock_irqrestore(&clock_lock, flags);
	return ret;
}
EXPORT_SYMBOL(clk_enable);

static void _clk_disable(struct clk *clk)
{
	if (clk->flags & CLK_ALWAYS_ENABLED)
		return;

	clk->usecount--;
	if (clk->usecount == 0) {
		if (clk->disable)
			(*clk->disable)(clk);
		else
			pr_warning("clock: %s: unimplemented function 'clk_disable' called\n", clk->name);
		if (clk->parent)
			_clk_disable(clk->parent);
	}
}

void clk_disable(struct clk *clk)
{
	unsigned long flags;

	BUG_ON(clk->usecount == 0);

	spin_lock_irqsave(&clock_lock, flags);
	_clk_disable(clk);
	spin_unlock_irqrestore(&clock_lock, flags);
}
EXPORT_SYMBOL(clk_disable);

unsigned long clk_get_rate(struct clk *clk)
{
	unsigned long rate;

	mutex_lock(&clock_mutex);
	if (clk->get_rate)
		rate = (*clk->get_rate)(clk);
	else
		rate = clk->rate;
	mutex_unlock(&clock_mutex);
	return rate;
}
EXPORT_SYMBOL(clk_get_rate);

/*
 * Set the clock rate for a clock source
 */
static int _clk_set_rate(struct clk *clk, unsigned long rate)
{
	int ret = 0;
	unsigned long flags;

	if (clk->flags & CLK_FIXED_RATE)
		return 0;

	spin_lock_irqsave(&clock_lock, flags);
	if (clk->set_rate) {
		ret = (*clk->set_rate)(clk, rate);
	} else {
		pr_warning("clock: unimplemented function 'clk_set_rate' called\n");
	}
	spin_unlock_irqrestore(&clock_lock, flags);
	return ret;
}

int clk_set_rate(struct clk *clk, unsigned long rate)
{
	int ret;

	ret = _clk_set_rate(clk, rate);

	if(ret == 0){
		clk->rate = rate;
	}

	return ret;
}
EXPORT_SYMBOL(clk_set_rate);

/*
 * Adjust a rate to the exact rate a clock can provide
 */
long clk_round_rate(struct clk *clk, unsigned long rate)
{
	//TODO
	return rate;
}
EXPORT_SYMBOL(clk_round_rate);

/*
 * Set the parent clock source for the clock
 */
int clk_set_parent(struct clk *clk, struct clk *parent)
{
	unsigned long flags;
	spin_lock_irqsave(&clock_lock, flags);
	clk->parent = parent;
	spin_unlock_irqrestore(&clock_lock, flags);
	return 0;
}
EXPORT_SYMBOL(clk_set_parent);

/*
 * Get the parent clock source for the clock
 */
struct clk *clk_get_parent(struct clk *clk)
{
	BUG_ON(clk == 0);

	return clk->parent;
}
EXPORT_SYMBOL(clk_get_parent);

int __init clk_init(void)
{
	struct clk **clk;

	spin_lock_init(&clock_lock);
	mutex_lock(&clock_mutex);

	tca_ckc_init();

	for (clk = onchip_clocks; clk < onchip_clocks + ARRAY_SIZE(onchip_clocks); clk++) {
		list_add_tail(&(*clk)->list, &clock_list);
	}
	mutex_unlock(&clock_mutex);

	pr_info("TCC clock driver initialized\n");

	return 0;
}

arch_initcall(clk_init);

void clk_disable_unused(void)
{
	struct clk *clk;
	int count = 0;

	list_for_each_entry(clk, &clock_list, list) {
		if (clk->flags & CLK_AUTO_OFF && !clk->usecount && clk->disable) {
			clk->disable(clk);
			count++;
			pr_debug("clock: clock '%s' disabled\n", clk->name);
		}
	}
	pr_debug("clock: disabled %d unused clocks\n", count);
}
EXPORT_SYMBOL(clk_disable_unused);

static int __init clock_late_init(void)
{
	clk_disable_unused();
	return 0;
}

late_initcall(clock_late_init);

#include <linux/proc_fs.h>
#include <linux/seq_file.h>

static void *tcc93xx_ck_start(struct seq_file *m, loff_t *pos)
{
	return *pos < 1 ? (void *)1 : NULL;
}

static void *tcc93xx_ck_next(struct seq_file *m, void *v, loff_t *pos)
{
	++*pos;
	return NULL;
}

static void tcc93xx_ck_stop(struct seq_file *m, void *v)
{
}

static int tcc93xx_ck_show(struct seq_file *m, void *v)
{
	struct clk *cp;
	
	list_for_each_entry(cp, &clock_list, list)
		if(!strcmp(cp->name,"cpu") || !strcmp(cp->name,"membus"))
			seq_printf(m,"%d ",cp->rate);
		else if(!strcmp(cp->name,"ddi") || !strcmp(cp->name,"gpu") || !strcmp(cp->name,"iobus") || 
				!strcmp(cp->name,"vbus") || !strcmp(cp->name,"vcore") || !strcmp(cp->name,"smu"))
			seq_printf(m,"%d ",cp->rate*100);

	seq_printf(m,"\n");

	return 0;
}

static const struct seq_operations tcc93xx_ck_op = {
	.start	= tcc93xx_ck_start,
	.next	= tcc93xx_ck_next,
	.stop	= tcc93xx_ck_stop,
	.show	= tcc93xx_ck_show
};

static int tcc93xx_ck_open(struct inode *inode, struct file *file)
{
	return seq_open(file, &tcc93xx_ck_op);
}

static const struct file_operations proc_tcc93xx_ck_operations = {
	.open		= tcc93xx_ck_open,
	.read		= seq_read,
	.llseek		= seq_lseek,
	.release	= seq_release,
};

static int __init tcc93xx_ck_proc_init(void)
{
	proc_create("clocks", 0, NULL, &proc_tcc93xx_ck_operations);
	return 0;

}
__initcall(tcc93xx_ck_proc_init);
