/*
 * linux/drivers/char/tcc_intr.c 
 *
 * Author:  <linux@telechips.com>
 * Created: September 30, 2008
 * Description: User-level Interrupt Driver for TCC79X.
 *
 * Copyright (C) 2008-2009 Telechips 
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 *
 * ChangeLog:
 *
 *	Oct 30, 2008	:	Added Mem-to-Mem Scaler & 2D Graphic Engine interrupts
 *						using device minor number
 *	Sep 30, 2008	:	Added Sub-Core interrupt (MAILBOX interrupt)
 *
 */

#include <linux/init.h>
#include <linux/kernel.h>
#include <linux/module.h>
#include <linux/moduleparam.h>
#include <linux/init.h>
#include <linux/fs.h>
#include <linux/slab.h>
#include <linux/platform_device.h>
#include <linux/errno.h>
#include <linux/types.h>
#include <linux/fcntl.h>
#include <linux/delay.h>

#include <asm/uaccess.h>
#include <asm/io.h>

#include <linux/interrupt.h>
#include <mach/irqs.h>
#include <mach/tcc79x.h>
#include <linux/tcc_intr.h>

#include <linux/time.h>
#include <linux/timer.h>
#include <linux/poll.h>

#define TCC_INTR_VERSION	"0.5"
#define TCC_INTR_DEV_NAME	"intr"

typedef struct _tcc_intr_data_t {
	wait_queue_head_t wq;
	spinlock_t lock;
	unsigned int count;
	unsigned int irq;
	unsigned int irq_bit;
} tcc_intr_data_t;


/*
 *	Subcore Mailbox Interface.
 */
static void init_mb_interrupt(tcc_intr_data_t *tcc_intr)
{
	BITCLR(HwMODE, tcc_intr->irq_bit);	// edge-triggered
	BITCLR(HwMODEA, tcc_intr->irq_bit);	// single-edge
	BITSET(HwPOL, tcc_intr->irq_bit);	// active-low
}

static irqreturn_t mb_handler(int irq, void *dev_id)
{
	tcc_intr_data_t *tcc_intr = dev_id;

	spin_lock_irq(&(tcc_intr->lock));
	tcc_intr->count++;
	spin_unlock_irq(&(tcc_intr->lock));

	wake_up_interruptible(&(tcc_intr->wq));

	return IRQ_HANDLED;
}

/*
 *	Mem to Mem Scaler Interface.
 */
static void init_sc_interrupt(tcc_intr_data_t *tcc_intr)
{
	BITCLR(HwMODE, tcc_intr->irq_bit);	// edge-triggered
	BITCLR(HwMODEA, tcc_intr->irq_bit);	// single-edge
	BITSET(HwPOL, tcc_intr->irq_bit);	// active-low
}

static irqreturn_t sc_handler(int irq, void *dev_id)
{
	tcc_intr_data_t *tcc_intr = dev_id;

	spin_lock_irq(&(tcc_intr->lock));
	tcc_intr->count++;
	spin_unlock_irq(&(tcc_intr->lock));

	wake_up_interruptible(&(tcc_intr->wq));

	return IRQ_HANDLED;
}

///////////////////////////////////////////////////////////////////////////
//
static long io_interrupt(tcc_intr_data_t *tcc_intr, long flag)
{
	long ret = 0;
	if (flag == 0) {
		BITSET(HwCLR, tcc_intr->irq_bit);			// clear intr
		BITCLR(HwINTMSK, tcc_intr->irq_bit);		// disable intr
	} else if (flag == 1) {
		BITSET(HwCLR, tcc_intr->irq_bit);
		BITSET(HwINTMSK, tcc_intr->irq_bit);		// enable intr
	} else if (flag == 2) {
		ret = (HwINTMSK & tcc_intr->irq_bit)?1:0;	// get int-mask status
	} else {
		ret = -1;
	}
	return ret;
}

static unsigned int intr_poll(struct file *filp, poll_table *wait)
{
	tcc_intr_data_t *tcc_intr = (tcc_intr_data_t *)filp->private_data;

	if (tcc_intr == NULL)
		return -EFAULT;

	if (tcc_intr->count > 0) {
		spin_lock_irq(&(tcc_intr->lock));
		tcc_intr->count--;
		spin_unlock_irq(&(tcc_intr->lock));
		return (POLLIN | POLLRDNORM);
	}
	
	poll_wait(filp, &(tcc_intr->wq), wait);

	if (tcc_intr->count > 0) {
		spin_lock_irq(&(tcc_intr->lock));
		tcc_intr->count--;
		spin_unlock_irq(&(tcc_intr->lock));
		return (POLLIN | POLLRDNORM);
	} else {
		return 0;
	}
}

static int intr_ioctl(struct inode *inode, struct file *filp, 
							unsigned int cmd, unsigned long arg)
{
	unsigned long data;
	tcc_intr_data_t *tcc_intr = (tcc_intr_data_t *)filp->private_data;

	switch (cmd) {
		case IOCTL_INTR_SET:
			if (copy_from_user((void *)&data, (const void *)arg, sizeof(data)))
				return -EFAULT;
			if (data == 0 || data == 1) io_interrupt(tcc_intr, data);
			else return -EFAULT;
			break;
		case IOCTL_INTR_GET:
			data = io_interrupt(tcc_intr, 2);
			if (copy_to_user((void *)arg, (const void *)&data, sizeof(data)))
				return -EFAULT;
			break;
		default:
			printk("tcc_intr(%d): unrecognized ioctl (0x%x)\n", MINOR(inode->i_rdev), cmd);
			return -EINVAL;
			break;
	}

	return 0;
}

static int intr_test_ioctl(struct inode *inode, struct file *filp, 
							unsigned int cmd, unsigned long arg)
{
	unsigned long data;

	switch (cmd) {
		case IOCTL_INTR_TEST:
			if (copy_from_user((void *)&data, (const void *)arg, sizeof(data))) {
				return -EFAULT;
			} else {
				if (data == 1) {
					BITSET(HwTIG, HwINT_SCORE);
				} else if (data == 2) {
					BITSET(HwTIG, HwINT_SC);
				} else {
					return -EFAULT;
				}
			}
			break;
		default:
			printk("tcc_intr(%d): unrecognized ioctl (0x%x)\n", MINOR(inode->i_rdev), cmd);
			return -EINVAL;
			break;
	}

	return 0;
}

static int intr_release(struct inode *inode, struct file *filp)
{
	tcc_intr_data_t *tcc_intr = (tcc_intr_data_t *)filp->private_data;

	free_irq(tcc_intr->irq, tcc_intr);
	kfree(tcc_intr);

	return 0;
}

///////////////////////////////////////////////////////////////////////////
//
static int intr_sc_open(struct inode *inode, struct file *filp)
{
	int ret = 0;
	tcc_intr_data_t *sc_data = NULL;

	sc_data = (tcc_intr_data_t *)kmalloc(sizeof(tcc_intr_data_t), GFP_KERNEL);
	if (sc_data == NULL) {
		ret = -ENOMEM;
		goto error;
	}
	memset(sc_data, 0, sizeof(tcc_intr_data_t));

	spin_lock_init(&(sc_data->lock));
	init_waitqueue_head(&(sc_data->wq));
	sc_data->irq = INT_SC;
	sc_data->irq_bit = HwINT_SC;

	init_sc_interrupt(sc_data);
	ret = request_irq(sc_data->irq, sc_handler, IRQF_DISABLED, "scaler", sc_data);
	if (ret) {
		ret = -EFAULT;
		goto error;
	}

	filp->private_data = (void *)sc_data;

	return 0;
error:
	if (sc_data) 
		kfree(sc_data);
	return ret;
}

static int intr_mb_open(struct inode *inode, struct file *filp)
{
	int ret = 0;
	tcc_intr_data_t *mb_data = NULL;

	mb_data = (tcc_intr_data_t *)kmalloc(sizeof(tcc_intr_data_t), GFP_KERNEL);
	if (mb_data == NULL) {
		ret = -ENOMEM;
		goto error;
	}
	memset(mb_data, 0, sizeof(tcc_intr_data_t));

	spin_lock_init(&(mb_data->lock));
	init_waitqueue_head(&(mb_data->wq));
	mb_data->irq = INT_SCORE;
	mb_data->irq_bit = HwINT_SCORE;

	init_mb_interrupt(mb_data);
	ret = request_irq(mb_data->irq, mb_handler, IRQF_DISABLED, "mailbox", mb_data);
	if (ret) {
		ret = -EFAULT;
		goto error;
	}

	filp->private_data = (void *)mb_data;

	return 0;
error:
	if (mb_data) 
		kfree(mb_data);
	return ret;
}

struct file_operations intr_mb_fops =
{
	.owner		= THIS_MODULE,
	.poll		= intr_poll,
	.ioctl		= intr_ioctl,
	.open		= intr_mb_open,
	.release	= intr_release,
};

struct file_operations intr_sc_fops =
{
	.owner		= THIS_MODULE,
	.poll		= intr_poll,
	.ioctl		= intr_ioctl,
	.open		= intr_sc_open,
	.release	= intr_release,
};

struct file_operations intr_test_fops =
{
	.owner		= THIS_MODULE,
	.ioctl		= intr_test_ioctl,
};

static int tcc_intr_open(struct inode *inode, struct file *filp)
{
	switch (MINOR(inode->i_rdev)) {
		case 1: filp->f_op = &intr_mb_fops; break;
		case 2: filp->f_op = &intr_sc_fops; break;
		case 3: filp->f_op = &intr_test_fops; break;
		default : return -ENXIO;
	}

	if (filp->f_op && filp->f_op->open)
		return filp->f_op->open(inode, filp);

	return 0;
}

struct file_operations tcc_intr_fops =
{
	.owner		= THIS_MODULE,
	.open		= tcc_intr_open,
};

static struct class *intr_class;

static int __init tcc_intr_init(void)
{
	int res;

	res = register_chrdev(TCC_INTR_DEV_MAJOR, TCC_INTR_DEV_NAME, &tcc_intr_fops);
	if (res < 0)
		return res;

	intr_class = class_create(THIS_MODULE, TCC_INTR_DEV_NAME);
	device_create(intr_class, NULL, MKDEV(TCC_INTR_DEV_MAJOR, 1), NULL, TCC_INTR_DEV_NAME);

	printk("tcc_intr: init (ver %s)\n", TCC_INTR_VERSION);
	return 0;
}

static void __exit tcc_intr_exit(void)
{
	unregister_chrdev(TCC_INTR_DEV_MAJOR, TCC_INTR_DEV_NAME);
	printk("tcc_intr: exit\n");
}


module_init(tcc_intr_init);
module_exit(tcc_intr_exit);

MODULE_AUTHOR("Telechips Inc. SYS4-3 linux@telechips.com");
MODULE_DESCRIPTION("Telechips user level interrut driver");
MODULE_LICENSE("GPL");
