/* 
 * linux/drivers/i2c/busses/i2c-tcc92xx.c
 *
 * Author:  <linux@telechips.com>
 * Created: 10th Jun, 2008 
 * Description: Telechips I2C Controller
 *
 * Copyright (c) Telechips, Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * ----------------------------
 * for TCC DIBCOM DVB-T module 
 * [M] i2c-core.c, i2c-dev.c
 * [M] include/linux/i2c-dev.h, include/linux/i2c.h
 *
*/

#include <linux/kernel.h>
#include <linux/module.h>
#include <linux/delay.h>
#include <linux/i2c.h>
#include <linux/i2c-id.h>
#include <linux/err.h>
#include <linux/interrupt.h>
#include <linux/completion.h>
#include <linux/platform_device.h>
#include <linux/clk.h>
#include <asm/io.h>
#include <mach/bsp.h>
#include "tca_i2c.h"


enum tcc_i2c_state {
	STATE_IDLE,
	STATE_START,
	STATE_READ,
	STATE_WRITE,
	STATE_STOP
};

struct tcc_i2c {
	spinlock_t lock;
	wait_queue_head_t wait;
	volatile struct tcc_i2c_regs *regs;
    volatile unsigned long IRQSTR;
	int ch;
	unsigned int clk;
	struct clk *pclk;

	struct i2c_msg *msg;
	unsigned int msg_num;
	unsigned int msg_idx;
	unsigned int msg_ptr;

	enum tcc_i2c_state state;

	struct device *dev;
	struct i2c_adapter adap;
};

/* tcc_i2c_message_start
 *
 * put the start of a message onto the bus 
*/
static void tcc_i2c_message_start(struct tcc_i2c *i2c, struct i2c_msg *msg)
{
	unsigned int addr = (msg->addr & 0x7f) << 1;

	if (msg->flags & I2C_M_RD)
		addr |= 1;
	if (msg->flags & I2C_M_REV_DIR_ADDR)
		addr ^= 1;

	i2c->regs->TXR = addr;
	i2c->regs->CMD = Hw7 | Hw4;
}

static int wait_intr(struct tcc_i2c *i2c)
{
	unsigned long cnt = 0;

	if (i2c->ch == 0 || i2c->ch == 1) {
	while (!(tcc_readl(i2c->IRQSTR) & (i2c->ch?Hw1:Hw0))) {
		cnt++;
		if (cnt > 100000) {
			printk("i2c-tcc: time out!\n");
			return -1;
		}
	}
	} else {
		/* SMU_I2C wait */
	while (1) {
			cnt = i2c->regs->SR;
			if (!(cnt & Hw1)) break;
		}
		for (cnt = 0; cnt <15; cnt++);
		}

	return 0;
}

static int recv_i2c(struct tcc_i2c *i2c)
{
	int ret, i;
	dev_dbg(&i2c->adap.dev, "READ [%x][%d]\n", i2c->msg->addr, i2c->msg->len);		

	tcc_i2c_message_start(i2c, i2c->msg);

	ret = wait_intr(i2c);
	if (ret != 0)
		return ret;
	BITSET(i2c->regs->CMD, Hw0);	/* Clear a pending interrupt */

	for (i = 0; i < i2c->msg->len; i++) {
			if (i == (i2c->msg->len - 1)) 
				i2c->regs->CMD = Hw5 | Hw3;
			else
				i2c->regs->CMD = Hw5;

		ret = wait_intr(i2c);
		if (ret != 0)
			return ret;
		BITSET(i2c->regs->CMD, Hw0);

		i2c->msg->buf[i] = (unsigned char) i2c->regs->RXR;
	}

	i2c->regs->CMD = Hw6;

	ret = wait_intr(i2c);
	if (ret != 0)
		return ret;
	BITSET(i2c->regs->CMD, Hw0);

	return 1;
}

static int send_i2c(struct tcc_i2c *i2c)
{
	int ret, i, no_stop = 0;
	dev_dbg(&i2c->adap.dev, "SEND [%x][%d]", i2c->msg->addr, i2c->msg->len);		

	tcc_i2c_message_start(i2c, i2c->msg);

	ret = wait_intr(i2c);
	if (ret != 0)
		return ret;
	BITSET(i2c->regs->CMD, Hw0);	/* Clear a pending interrupt */

	for (i = 0; i < i2c->msg->len; i++) {
		i2c->regs->TXR = i2c->msg->buf[i];
		i2c->regs->CMD = Hw4;
		ret = wait_intr(i2c);
		if (ret != 0)
			return ret;
		BITSET(i2c->regs->CMD, Hw0);

		//i2c->msg->buf[i] = (unsigned char) i2c->regs->RXR;	//XXX
	}

	/*
	 *	Check no-stop operation condition (write -> read operation)
	 *	1. DIBCOM 
	 *  2. msg_num == 2
	 */
	if (i2c->msg->flags & I2C_M_DIBCOM_WR_RD)
		no_stop = 1;
	if (i2c->msg_num > 1)
		no_stop = 1;
	
	//if ( (!(i2c->msg->flags & I2C_M_DIBCOM_WR_RD)) || (i2c->msg_num == 1) ) {
	if (no_stop == 0) {
		i2c->regs->CMD = Hw6;
		ret = wait_intr(i2c);
		if (ret != 0)
			return ret;
	}
	BITSET(i2c->regs->CMD, Hw0);	/* Clear a pending interrupt */

	return 1;
}

/* tcc_i2c_doxfer
 *
 * this starts an i2c transfer
*/
static int tcc_i2c_doxfer(struct tcc_i2c *i2c, struct i2c_msg *msgs, int num)
{
	int ret, i;

	for (i = 0; i < num; i++) {
		spin_lock_irq(&i2c->lock);
		i2c->msg 		= &msgs[i];
		i2c->msg->flags = msgs[i].flags;
		i2c->msg_num 	= num;
		i2c->msg_ptr 	= 0;
		i2c->msg_idx 	= 0;
		i2c->state = STATE_START;
		spin_unlock_irq(&i2c->lock);

		if (i2c->msg->flags & I2C_M_RD) {
			ret = recv_i2c(i2c);
			if (ret != 1)
				printk("recv_i2c failed\n");
		} else {
			ret = send_i2c(i2c);
			if (ret != 1)
				printk("send_i2c failed\n");
		}
	}
	
	return ret;
}

/* tcc_i2c_xfer
 *
 * first port of call from the i2c bus code when an message needs
 * transferring across the i2c bus.
*/
static int tcc_i2c_xfer(struct i2c_adapter *adap, struct i2c_msg *msgs, int num)
{
	struct tcc_i2c *i2c = (struct tcc_i2c *)adap->algo_data;
	int retry;
	int ret;

	for (retry = 0; retry < adap->retries; retry++) {
		
		ret = tcc_i2c_doxfer(i2c, msgs, num);

		if (ret == 1)
			return num;

		dev_dbg(&i2c->adap.dev, "Retrying transmission (%d)\n", retry);

		udelay(100);
	}

	return -EREMOTEIO;
}

static u32 tcc_i2c_func(struct i2c_adapter *adap)
{
	return I2C_FUNC_I2C | I2C_FUNC_SMBUS_EMUL;
}

/* i2c bus registration info */
static const struct i2c_algorithm tcc_i2c_algorithm = {
	.master_xfer	= tcc_i2c_xfer,
	.functionality	= tcc_i2c_func,
};

/* tcc_i2c_init
 *
 * initialize the I2C controller, set the IO lines and frequency
 */
static void tcc_i2c_init(struct tcc_i2c *i2c)
{
	if (i2c->ch == 0 || i2c->ch == 1) {
	/* I2C GPIO setting */
	tca_i2c_setgpio(i2c->ch);

	i2c->pclk = clk_get(NULL, "i2c");
	if (!i2c->pclk) {
		dev_err(&i2c->dev, "can't get i2c clock\n");
		return;
	}
	clk_enable(i2c->pclk);

	/* I2C clock setting */
	tca_i2c_setclock(i2c->ch, i2c->regs, i2c->clk);
	} else {
		/* SMU_I2C clock setting */
		tcc_writel(0x80000000, i2c->IRQSTR);

		i2c->regs->CTRL = 0;
		i2c->regs->PRES = 4;
		BITSET(i2c->regs->CTRL, Hw7|Hw6);
	}
}

/* tcc83xx_i2c_probe
 *
 * called by the bus driver when a suitable device is found
*/
static int tcc_i2c_probe(struct platform_device *pdev)
{
	struct tcc_i2c *i2c;
	struct resource *resource_mem, *resource_clk;
	struct resource *resource_mem_smu_i2c, *resource_clk_smu_i2c;
	int i, ret;

	resource_mem = platform_get_resource(pdev, IORESOURCE_MEM, 0);
	if (!resource_mem) {
		dev_err(&pdev->dev, "no mem resource?\n");
		return -ENODEV;
	}
	resource_clk = platform_get_resource(pdev, IORESOURCE_MEM, 1);
	if (!resource_clk) {
		dev_err(&pdev->dev, "no clk resource?\n");
		return -ENODEV;
	}

	/* SMU_I2C */
	resource_mem_smu_i2c = platform_get_resource(pdev, IORESOURCE_MEM, 2);
	if (!resource_mem_smu_i2c) {
		dev_err(&pdev->dev, "no mem resource? (smu_i2c)\n");
		return -ENODEV;
	}
	resource_clk_smu_i2c = platform_get_resource(pdev, IORESOURCE_MEM, 3);
	if (!resource_clk_smu_i2c) {
		dev_err(&pdev->dev, "no clk resource? (smu_i2c)\n");
		return -ENODEV;
	}

	i2c = kzalloc(sizeof(struct tcc_i2c) * I2C_NUM_OF_CH, GFP_KERNEL);
	if (!i2c) {
		ret = -ENOMEM;
		goto err_nockl;
	}

	for (i = 0; i < I2C_NUM_OF_CH; i++) {
		i2c[i].adap.owner = THIS_MODULE;
		i2c[i].adap.algo = &tcc_i2c_algorithm;
		i2c[i].adap.retries = 2;
		sprintf(i2c[i].adap.name, "tcc-i2c-%d", i);

		spin_lock_init(&i2c[i].lock);
		init_waitqueue_head(&i2c[i].wait);

		if (i == 0) {
			i2c[i].ch = 0;
			i2c[i].clk = (unsigned int)resource_clk->start;
			i2c[i].regs = (volatile struct tcc_i2c_regs *)resource_mem->start;
			i2c[i].IRQSTR = resource_mem->start + I2C_IRQSTR_OFFSET;
		} else if (i == 1) {
			i2c[i].ch = 1;
			i2c[i].clk = (unsigned int)resource_clk->end;
			i2c[i].regs = (volatile struct tcc_i2c_regs *)resource_mem->end;
			i2c[i].IRQSTR = resource_mem->start + I2C_IRQSTR_OFFSET;
		} else if (i == 2) {
			i2c[i].ch = 2;
			i2c[i].clk = (unsigned int)resource_clk_smu_i2c->start;
			i2c[i].regs = (volatile struct tcc_i2c_regs *)resource_mem_smu_i2c->start;
			i2c[i].IRQSTR = resource_mem_smu_i2c->start + I2C_ICLK_OFFSET;
		} else if (i == 3) {
			i2c[i].ch = 3;
			i2c[i].clk = (unsigned int)resource_clk_smu_i2c->end;
			i2c[i].regs = (volatile struct tcc_i2c_regs *)resource_mem_smu_i2c->end;
			i2c[i].IRQSTR = resource_mem_smu_i2c->start + I2C_ICLK_OFFSET;
		} else {
			dev_err(&pdev->dev, "no i2c channel?\n");
			return -ENODEV;
		}
	}

	/* I2C reset */
	tca_i2c_reset();

	for (i = 0; i < I2C_NUM_OF_CH; i++) {
		tcc_i2c_init(i2c + i);

		i2c[i].adap.algo_data = i2c + i;
		i2c[i].adap.dev.parent = &pdev->dev;
		i2c[i].adap.class = I2C_CLASS_ALL;

		ret = i2c_add_adapter(&i2c[i].adap);
		if (ret < 0) {
			printk("tcc-i2c-%d: Failed to add bus\n", i);
			for (i--; i >= 0; i--)
				i2c_del_adapter(&i2c[i].adap);
			goto err_clk;
		}
	}

	platform_set_drvdata(pdev, i2c);
	dev_info(&pdev->dev, "%s: I2C adapter\n", i2c->adap.dev.bus_id);

	return 0;

err_clk:
	//TODO: I2C peri bus disable

err_nockl:
	return ret;
}

static int tcc_i2c_remove(struct platform_device *pdev)
{
	int i;
	struct tcc_i2c *i2c = platform_get_drvdata(pdev);

	platform_set_drvdata(pdev, NULL);

	for (i = 0; i < I2C_NUM_OF_CH; i++) {
		i2c_del_adapter(&i2c[i].adap);

		/* I2C bus & clock disable */
		clk_disable(i2c->pclk);
		clk_put(i2c->pclk);
	}

	kfree(i2c);

	return 0;
}

#ifdef CONFIG_PM
static int tcc_i2c_suspend(struct platform_device *dev, pm_message_t state)
{
	return 0;
}

static int tcc_i2c_resume(struct platform_device *dev)
{
	return 0;
}
#else
#define tcc_i2c_suspend	NULL
#define tcc_i2c_resume	NULL
#endif

/* device driver for platform bus bits */

static struct platform_driver tcc_i2c_driver = {
	.probe		= tcc_i2c_probe,
	.remove		= tcc_i2c_remove,
	.suspend	= tcc_i2c_suspend,
	.resume		= tcc_i2c_resume,
	.driver		= {
		.owner	= THIS_MODULE,
		.name	= "tcc-i2c",
	},
};

static int __init tcc_i2c_adap_init(void)
{
	return platform_driver_register(&tcc_i2c_driver);
}

static void __exit tcc_i2c_adap_exit(void)
{
	platform_driver_unregister(&tcc_i2c_driver);
}

module_init(tcc_i2c_adap_init);
module_exit(tcc_i2c_adap_exit);

MODULE_AUTHOR("Telechips Inc. SYS4-3 linux@telechips.com");
MODULE_DESCRIPTION("Telechips H/W I2C driver");
MODULE_LICENSE("GPL");
