/*
 * linux/drivers/serial/tcc_ts_cypress.c
 *
 * Author: <linux@telechips.com>
 * Created: June 10, 2008
 * Description: Touchscreen driver for TSC2003 on Telechips TCC Series
 *
 * Copyright (C) 2008-2009 Telechips 
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, see the file COPYING, or write
 * to the Free Software Foundation, Inc.,
 * 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */
#include <linux/kernel.h>
#include <linux/device.h>
#include <linux/hrtimer.h>
#include <linux/delay.h>
#include <linux/platform_device.h>
#include <linux/init.h>
#include <linux/input.h>
#include <linux/interrupt.h>
#include <linux/module.h>
#include <linux/clk.h>
#include <linux/irq.h>
#include <linux/spinlock.h>
#include <linux/i2c.h>
#ifdef CONFIG_HAS_EARLYSUSPEND
#include <linux/earlysuspend.h>
#endif
#if defined(CONFIG_ARCH_TCC92X)
#include <mach/bsp.h>
#endif

#define TSC_SUPPORT_MULTI_TOUCH
//#define TSC_USE_HRTIMER

#define dbg(msg...)	if (0) { printk( "TCC_TSC: " msg); }

#define TSC_EINT_PIN			Hw14
#define TSC_EINT_PORT_EN		HwGPIOD->GPEN
#define TSC_EINT_PORT_PD		HwGPIOD->GPPD0
#define TSC_EINT_PORT_PD_MASK	(Hw28|Hw29)
#define TSC_EINT_PORT_PD_UP		(Hw28)
#define TSC_EINT_PORT_PD_DOWN	(Hw29)
#define TSC_EINT_PORT_DAT		HwGPIOD->GPDAT
#define TSC_EINT_SEL			HwEINTSEL2_EINT11(SEL_GPIOD14)

#define TSC_EINT_SEL_MASK		HwEINTSEL2_EINT11_MASK
#define TSC_EINT_SEL_ID			HwINT0_EI11

#define TS_WAKEUP_PORT_EN		HwGPIOD->GPEN
#define TS_WAKEUP_PORT_DAT		HwGPIOD->GPDAT
#define TS_WAKEUP_PIN			Hw12

#ifdef TSC_USE_HRTIMER
#define CYPRESS_TS_TIMER_PERIOD	(10*1000*1000/2)	/* touch polling period : 10 ms. */
#else
#define CYPRESS_TS_TIMER_PERIOD	10	/* touch polling period : 10 ms. */
#endif

//==============================================================
#define DEVICE_NAME		"cypress_ts"
#define PEN_RELEASE		0x0
#define PEN_DOWN		0x1

#define READ_PENIRQ()	((HwGPIOD->GPDAT & Hw14) == 0)
#define X_AXIS_MIN		0
#define X_AXIS_MAX		480
#define Y_AXIS_MIN		0
#define Y_AXIS_MAX		320
#ifdef TSC_SUPPORT_MULTI_TOUCH
#define DISTANCE_MAX	20
#endif

struct cypress_ts {
	struct input_dev *input;
	int irq;
#ifdef TSC_USE_HRTIMER	
	struct hrtimer timer;
#else
	struct timer_list timer;
#endif
	struct work_struct work;
	struct clk *clk;
#ifdef CONFIG_HAS_EARLYSUSPEND
    struct early_suspend early_suspend;
	int	in_suspend, running;
#endif
};

static struct workqueue_struct *cypress_ts_wq;

static unsigned short probe[] = { 1, 0x4d, I2C_CLIENT_END };
static unsigned short dummy[] = {I2C_CLIENT_END};

static struct i2c_client_address_data addr_data = {
	.normal_i2c = dummy,
	.probe      = probe,
	.ignore     = dummy,
};

static struct i2c_driver cypress_i2c_driver;
static struct i2c_client *cypress_i2c_client;

#ifdef CONFIG_HAS_EARLYSUSPEND
static void tcc92xx_tsc_early_suspend(struct early_suspend *h);
static void tcc92xx_tsc_late_resume(struct early_suspend *h);
#endif

static void TOUCH_SEND_CMD(unsigned char reg, unsigned char val)
{
	unsigned char cmd[2];
	cmd[0] = reg;
	cmd[1] = val;	
	i2c_master_send(cypress_i2c_client, cmd, 2);
}

static int TOUCH_READ_MULTI(unsigned int reg, unsigned char *buf, int size)
{
	unsigned char result = 0;
	unsigned char reg_buf;
	reg_buf = (unsigned char)reg;
	result = i2c_master_send(cypress_i2c_client, &reg_buf, 1);
	if (result != 1)
		return result;
	result = i2c_master_recv(cypress_i2c_client, buf, size);
	if (result != size)
		return result;
	return 0;
}

static void cypress_ts_irq_init(void)
{
	// Interrupt Pin
	BITCSET(HwGPIOD->GPFN1, 0x0F000000, 0x00000000);
	HwGPIOD->GPEN	&= ~Hw14;
	HwGPIOD->GPDAT	&= ~Hw14;

	BITCLR(TSC_EINT_PORT_EN, TSC_EINT_PIN);						/* PENIRQ Port direction set (input) */
	BITCSET(TSC_EINT_PORT_PD, TSC_EINT_PORT_PD_MASK, TSC_EINT_PORT_PD_UP);
	BITCSET(HwEINTSEL->EINTSEL2, TSC_EINT_SEL_MASK, TSC_EINT_SEL);

	HwPIC->INTMSK0 |= TSC_EINT_SEL_ID;	   						/* interrupt mask enabke */

	HwPIC->IEN0		&= ~TSC_EINT_SEL_ID;						/* disable Isr */
	HwPIC->MODEA0	|= TSC_EINT_SEL_ID; 						/* both edge */
	HwPIC->MODE0	&= ~TSC_EINT_SEL_ID;						/* set edge trigger mode */
	HwPIC->POL0		|= TSC_EINT_SEL_ID;							/* active-low */
	HwPIC->CLR0		|= TSC_EINT_SEL_ID;							/* clear pending status */
	HwPIC->SEL0		|= TSC_EINT_SEL_ID;							/* IRQ Select */
	HwPIC->IEN0		|= TSC_EINT_SEL_ID;							/* Enable Interrupt */
}

static void cypress_ts_irq_enable(void)
{
	HwPIC->INTMSK0 |= TSC_EINT_SEL_ID;	   
	HwPIC->CLR0    |= TSC_EINT_SEL_ID;
	HwPIC->IEN0    |= TSC_EINT_SEL_ID;
}

static void cypress_ts_irq_disable(void)
{
	HwPIC->INTMSK0 &= ~TSC_EINT_SEL_ID;
	HwPIC->IEN0    &= ~TSC_EINT_SEL_ID; 
}

static void cypress_io_init(void)
{
	BITCSET(TS_WAKEUP_PORT_EN, TS_WAKEUP_PIN, TS_WAKEUP_PIN);  //output mode
	BITCSET(TS_WAKEUP_PORT_DAT, TS_WAKEUP_PIN, 0);//HIGH to enter sleep mode
	mdelay(100);
	BITCSET(TS_WAKEUP_PORT_DAT, TS_WAKEUP_PIN, TS_WAKEUP_PIN);//HIGH to enter sleep mode
	udelay(100);
	BITCSET(TS_WAKEUP_PORT_DAT, TS_WAKEUP_PIN, 0);//set low to wakeup goodix device
	mdelay(100);

#ifdef TSC_SUPPORT_MULTI_TOUCH
	TOUCH_SEND_CMD(0x06, 0x02);
#else
	TOUCH_SEND_CMD(0x06, 0x00);
#endif
}

static int cypress_i2c_probe(struct i2c_adapter *adap, int addr, int kind)
{
	struct i2c_client *i2c;
	int ret;

	i2c = kzalloc(sizeof(*i2c), GFP_KERNEL);
	if (!i2c)
		return -ENOMEM;

	strcpy(i2c->name, "cypress");
	i2c->flags = I2C_M_DIBCOM_MODE | I2C_M_DIBCOM_WR_RD;;   //MCC //for multi read data.
	i2c->addr = addr;
	i2c->adapter = adap;
	i2c->driver = &cypress_i2c_driver;
	cypress_i2c_client = i2c;

	/* attach i2c client device */
	ret = i2c_attach_client(i2c);
	if (ret < 0) {
		printk("%s: failed to attach codec at addr %x\n", __func__, addr);
		goto err;
	}

	return ret;
err:
	kfree(i2c);
	return ret;
}

static int cypress_i2c_detach(struct i2c_client *client)
{
	i2c_detach_client(client);
	kfree(cypress_i2c_client);
	return 0;
}

static int cypress_i2c_attach(struct i2c_adapter *adap)
{
	return i2c_probe(adap, &addr_data, cypress_i2c_probe);
}

/* cypress i2c codec control layer */
static struct i2c_driver cypress_i2c_driver = {
	.driver = {
		.name   = "tcc-cypress-ts",
		.owner  = THIS_MODULE,
	},
//	.id             = I2C_DRIVERID_CYPRESS,
	.attach_adapter = cypress_i2c_attach,
	.detach_client  = cypress_i2c_detach,
	.command        = NULL,
};

void cypress_openi2c(void)
{
	int ret;
	ret = i2c_add_driver(&cypress_i2c_driver);
	if (ret != 0)
		printk("%s: can't add i2c driver", __func__);
}

unsigned int cypress_getirqnum(void)
{
	return INT_EI11;
}

static void cypress_ts_work_func(struct work_struct *work)
{
	struct cypress_ts* ts = container_of(work, struct cypress_ts, work);
	struct input_dev *dev = ts->input;
	unsigned char readData[6];
	unsigned int fingers, x_pos, y_pos, dist;
	int res, retry_cnt;

    ts->running = 1;
	cypress_ts_irq_disable();

	HwGPIOD->GPDAT |= Hw10;

	/* read x,y posision */
	retry_cnt = 3;
	do {
		res = TOUCH_READ_MULTI(0x00, readData, 6);
		retry_cnt--;
	} while (retry_cnt && res);

	HwGPIOD->GPDAT &= ~Hw10;

	x_pos = (((u16)readData[1]) <<8)|(u16)readData[2];
	y_pos = (((u16)readData[3]) <<8)|(u16)readData[4];
	fingers = readData[0] & 0x3;
	dist = (readData[5] & 0x3F);
	if (dist > DISTANCE_MAX)
		dist = DISTANCE_MAX;

	/* send input events */
	if (fingers) {
		input_report_abs(dev, ABS_X, x_pos);
		input_report_abs(dev, ABS_Y, y_pos);
	}
    input_report_key(dev, BTN_TOUCH, fingers);
    input_report_key(dev, ABS_PRESSURE, fingers);
#ifdef TSC_SUPPORT_MULTI_TOUCH
	if (fingers > 1)	/* two fingers touch */
		input_report_key(dev, ABS_DISTANCE, dist);
#endif
    input_sync(dev);

	dbg("%s: finger:%d, x:%d, y:%d, dist:%d\n", __func__, fingers, x_pos, y_pos, dist);

	if (fingers) {
#ifdef TSC_USE_HRTIMER
		hrtimer_start(&ts->timer, ktime_set(0, CYPRESS_TS_TIMER_PERIOD), HRTIMER_MODE_REL);
#else
        ts->timer.expires = jiffies + msecs_to_jiffies(CYPRESS_TS_TIMER_PERIOD);
        add_timer(&ts->timer);
#endif
	}
	else {
		ts->running = 0;
		cypress_ts_irq_enable();
	}
}

static irqreturn_t cypress_ts_irq_handler(int irq, void *dev_id)
{
	struct cypress_ts *ts = (struct cypress_ts *)dev_id;
	//dbg("\n %s\n",__func__);
	if ((ts->in_suspend == 0) && (ts->running == 0)) {
        ts->running = 1;
		queue_work(cypress_ts_wq, &ts->work);
	}
	return IRQ_HANDLED;
}

#ifdef TSC_USE_HRTIMER
static enum hrtimer_restart cypress_ts_timer_func(struct hrtimer *timer)
{
	struct cypress_ts *ts = container_of(timer, struct cypress_ts, timer);
	if (ts->in_suspend)
		ts->running = 0;
	else
		queue_work(cypress_ts_wq, &ts->work);
	return HRTIMER_NORESTART;
}
#else
static void cypress_ts_timer_func(unsigned long data)
{
	struct cypress_ts *ts = (struct cypress_ts *)data;
	if (ts->in_suspend)
		ts->running = 0;
	else
		queue_work(cypress_ts_wq, &ts->work);
}
#endif

static int __devinit cypress_ts_probe(struct platform_device *pdev)
{
	struct input_dev *input_dev;
	struct cypress_ts *ts;
	struct clk *clk;
	int err = -ENOMEM;

	printk("%s\n", __func__);

	/* INPUT DEVICE */
	ts = kzalloc(sizeof(struct cypress_ts), GFP_KERNEL);

	input_dev = input_allocate_device();

	if (!ts || !input_dev) {
		printk("ERROR 1\n");
		goto fail1;
	}

	platform_set_drvdata(pdev, ts);

	clk = clk_get(NULL, "i2c");
	if (clk)
		clk_enable(clk);
	else
		pr_err("%s: can't get i2c clock\n", DEVICE_NAME);

	ts->clk = clk;
	ts->running = 0;
	ts->input = input_dev;

	cypress_openi2c();

	input_dev->name = "tcc-cypress-ts";
	input_dev->phys = "cypress/input0";
	input_dev->id.version = 0x0001;
	input_dev->id.vendor = 0x00;   //MCC test
	input_dev->id.product = 0x00;   //MCC test
	input_dev->dev.parent = &pdev->dev;

	input_dev->evbit[0] = BIT_MASK(EV_KEY) | BIT_MASK(EV_ABS);
	input_dev->keybit[BIT_WORD(BTN_TOUCH)] = BIT_MASK(BTN_TOUCH);
	input_set_abs_params(input_dev, ABS_X, X_AXIS_MIN, X_AXIS_MAX, 0, 0);
	input_set_abs_params(input_dev, ABS_Y, Y_AXIS_MIN, Y_AXIS_MAX, 0, 0);
	/* TODO */
	input_set_abs_params(input_dev, ABS_PRESSURE, 0, 1, 0, 0);
#ifdef TSC_SUPPORT_MULTI_TOUCH
	input_set_abs_params(input_dev, ABS_DISTANCE, 0, DISTANCE_MAX, 0, 0);
#endif

	INIT_WORK(&(ts->work), cypress_ts_work_func);

#ifdef TSC_USE_HRTIMER
	hrtimer_init(&ts->timer, CLOCK_MONOTONIC, HRTIMER_MODE_REL);
	ts->timer.function = cypress_ts_timer_func;
#else
    init_timer(&ts->timer);
    ts->timer.data = (unsigned long)ts;
    ts->timer.function = cypress_ts_timer_func;
#endif

	cypress_ts_irq_init();

	ts->irq = cypress_getirqnum();
	/* Enable touch interrupt */
	if (request_irq(ts->irq, cypress_ts_irq_handler, IRQF_DISABLED, DEVICE_NAME,ts)) {
		err = -EBUSY;
		goto fail1;
	}

	cypress_io_init();	

	err = input_register_device(ts->input);
	if (err)
		goto fail2;

#ifdef CONFIG_HAS_EARLYSUSPEND
	ts->early_suspend.suspend 	= tcc92xx_tsc_early_suspend;
	ts->early_suspend.resume 	= tcc92xx_tsc_late_resume;
	ts->early_suspend.level 	= EARLY_SUSPEND_LEVEL_BLANK_SCREEN;
	register_early_suspend(&ts->early_suspend);
	ts->in_suspend = 0;
#endif

	printk("cypress driver is successfully loaded\n");
	return 0;

fail2:
	free_irq(ts->irq, ts);
fail1:
	input_free_device(input_dev);
	kfree(ts);
	return err;
}

static int __devexit cypress_ts_remove(struct platform_device *pdev)
{
	struct cypress_ts *ts = platform_get_drvdata(pdev);
	unregister_early_suspend(&ts->early_suspend);
	free_irq(ts->irq, ts);
#ifdef TSC_USE_HRTIMER
	hrtimer_cancel(&ts->timer);
#else
	del_timer(&ts->timer);
#endif
	input_unregister_device(ts->input);
	i2c_del_driver(&cypress_i2c_driver);
	kfree(cypress_i2c_client);
	clk_disable(ts->clk);
	clk_put(ts->clk);
	kfree(ts);
	return 0;
}

#ifdef CONFIG_HAS_EARLYSUSPEND
static void tcc92xx_tsc_early_suspend(struct early_suspend *h)
{
	struct cypress_ts *ts = container_of(h, struct cypress_ts, early_suspend);
	ts->in_suspend = 1;
	dbg("%s\n", __func__);
	cypress_ts_irq_disable();
//	TOUCH_SEND_CMD(0x06, 0x01);
//	BITCSET(TS_WAKEUP_PORT_DAT, TS_WAKEUP_PIN, TS_WAKEUP_PIN);//HIGH to enter sleep mode
}
static void tcc92xx_tsc_late_resume(struct early_suspend *h)
{
	char temp[6];
	struct cypress_ts *ts = container_of(h, struct cypress_ts, early_suspend);
	dbg("%s\n", __func__);
	ts->in_suspend = 0;
	TOUCH_READ_MULTI(0x00, temp, 6);
	cypress_ts_irq_enable();
}
#else
static int cypress_ts_suspend(struct platform_device *dev, pm_message_t state)
{
//	TOUCH_SEND_CMD(0x06, 0x01);
//	BITCSET(TS_WAKEUP_PORT_DAT, TS_WAKEUP_PIN, TS_WAKEUP_PIN);
	return 0;
}

static int cypress_ts_resume(struct platform_device *dev)
{
	cypress_io_init();
	return 0;
}
#endif

static struct platform_driver cypress_driver = {
	.driver     = {
		.name   = "tcc-cypress-ts",
	},
	.probe      = cypress_ts_probe,
	.remove     = cypress_ts_remove,
#ifndef CONFIG_HAS_EARLYSUSPEND
	.suspend    = cypress_ts_suspend,
	.resume     = cypress_ts_resume,
#endif
};

static int __devinit cypress_ts_init(void)
{
	int res;
	printk("Telechips CYPRESS Touch Driver\n");

	cypress_ts_wq = create_singlethread_workqueue("cypress_ts_wq");
	if (!cypress_ts_wq)
		return -ENOMEM;

	res =  platform_driver_register(&cypress_driver);
	if(res) {
		printk("fail : platform driver %s (%d) \n", cypress_driver.driver.name, res);
		return res;
	}

	return 0;
}

void __exit cypress_ts_exit(void)
{
	platform_driver_unregister(&cypress_driver);

	if (cypress_ts_wq)
		destroy_workqueue(cypress_ts_wq);
}

module_init(cypress_ts_init);
module_exit(cypress_ts_exit);

MODULE_AUTHOR("linux@telechips.com");
MODULE_DESCRIPTION("Telechips cypress touch driver");
MODULE_LICENSE("GPL");
