/*
 * drivers/mfd/act8810.c
 *
 * ACT8810 PMIC driver
 *
 * Copyright (C) 2009 Telechips, Inc.
 *
 * This software is licensed under the terms of the GNU General Public
 * License version 2, as published by the Free Software Foundation, and
 * may be copied, distributed, and modified under those terms.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	 See the
 * GNU General Public License for more details.
 */

#include <linux/kernel.h>
#include <linux/i2c.h>
#include <linux/mfd/core.h>
#include <linux/mfd/act8810.h>
#include <linux/regulator/act8810.h>
#include <linux/clk.h>

#define ACT8810_DRIVER_NAME "act8810-regulator"

#define ACT8810_I2C_ADDR		0x5A

static unsigned short dummy[] = { I2C_CLIENT_END };

static unsigned short probe[] = { 0, ACT8810_I2C_ADDR, I2C_CLIENT_END };

static struct i2c_client_address_data addr_data = {
	.normal_i2c = dummy,
	.probe	    = probe,
	.ignore	    = dummy,
};

static struct clk *clk;

static int act8810_attach_adapter(struct i2c_adapter *adapter);
static int act8810_detach_client(struct i2c_client *client);

static int act8810_read(struct act8810 *act8810, u8 reg, int count, u8 *data)
{
	struct i2c_client *i2c = act8810->client;
	struct i2c_msg xfer[2];
	int ret;

	/* write register */
	xfer[0].addr = i2c->addr;
	xfer[0].flags = 0;
	xfer[0].len =  1;
	xfer[0].buf = &reg;

	/* read data */
	xfer[1].addr = i2c->addr;
	xfer[1].flags = I2C_M_RD;
	xfer[1].len = count * sizeof(u8);
	xfer[1].buf = data;

	ret = i2c_transfer(i2c->adapter, xfer, 2);
	pr_debug("%s: returns %d\n", __func__, ret);
	if (ret < 0)
		return -EINVAL;
	return 0;
}

int act8810_write(struct act8810 *act8810, u8 reg, int count, u8 *data)
{
	struct i2c_client *i2c = act8810->client;
	u8 *msg;
	int ret;

	msg = kmalloc(count * sizeof(u8) + 1, GFP_KERNEL);
	if (msg == NULL)
		return -ENOMEM;

	msg[0] = reg;
	memcpy(&msg[1], data, count * sizeof(u8));

	ret = i2c_master_send(i2c, msg, count * sizeof(u8) + 1);
	pr_debug("%s: returns %d\n", __func__, ret);
	kfree(msg);

	if (ret < 0)
		return -EINVAL;
	return 0;
}

int act8810_reg_read(struct act8810 *act8810, u8 reg)
{
	u8 val;

	mutex_lock(&act8810->lock);
	if (act8810_read(act8810, reg, 1, &val) < 0)
		val = -EINVAL;
	mutex_unlock(&act8810->lock);

	return val;
}
EXPORT_SYMBOL(act8810_reg_read);

int act8810_reg_write(struct act8810 *act8810, u8 reg, u8 val)
{
	int ret;

	mutex_lock(&act8810->lock);
	ret = act8810_write(act8810, reg, 1, &val);
	mutex_unlock(&act8810->lock);

	return ret;
}
EXPORT_SYMBOL(act8810_reg_write);

int act8810_reg_set_bit_mask(struct act8810 *act8810, u8 reg, u8 mask, u8 val)
{
	int ret;
	u8 tmp;

	mutex_lock(&act8810->lock);
	ret = act8810_read(act8810, reg, 1, &tmp);
	if (ret < 0)
		goto out;
	tmp &= ~mask;
	tmp |= mask & val;
	ret = act8810_write(act8810, reg, 1, &tmp);
out:
	mutex_unlock(&act8810->lock);

	return ret;	
}
EXPORT_SYMBOL(act8810_reg_set_bit_mask);

static int act8810_init(struct act8810 *act8810)
{
	int i;

	/* set REG1 voltage */
	act8810_reg_set_bit_mask(act8810, 0x10, 0x1F, 0x1B);	/* 1.30V */
//	act8810_reg_set_bit_mask(act8810, 0x11, 0x1F, 0x10);	/* 1.80V */
//	act8810_reg_set_bit_mask(act8810, 0x12, 0x1F, 0x10);	/* 1.80V */
	act8810_reg_set_bit_mask(act8810, 0x13, 0x3F, 0x1B);	/* 1.80V */

	for (i = 0; i <= 0xf; i++) {
		pr_debug("%s: reg=0x%x, val=0x%x\n", __func__, i, act8810_reg_read(act8810, i));
	}

	return 0;
}

static struct i2c_driver act8810_i2c_driver = {
	.driver		= {
		.name	= ACT8810_DRIVER_NAME,
	},
//	.id		= I2C_DRIVERID_ACT8810,
	.attach_adapter	= act8810_attach_adapter,
	.detach_client	= act8810_detach_client,
};

static int act8810_probe(struct i2c_adapter *adapter, int addr, int kind)
{
	struct i2c_client *client;
	struct act8810 *act8810;
	int ret;

	client = kzalloc(sizeof(struct i2c_client), GFP_KERNEL);
	if (client == NULL) {
		ret = -ENOMEM;
		goto err_mem;
	}

	client->addr = addr;
	client->adapter = adapter;
	client->driver = &act8810_i2c_driver;
	strlcpy(client->name, ACT8810_DRIVER_NAME, I2C_NAME_SIZE);

	ret = i2c_attach_client(client);
	if (ret < 0) {
		pr_err("act8810: can't attach i2c client: %d\n", ret);
		goto err;
	}

	act8810 = kzalloc(sizeof(struct act8810), GFP_KERNEL);
	if (act8810 == NULL) {
		ret = -ENOMEM;
		goto err;
	}

	mutex_init(&act8810->lock);

	i2c_set_clientdata(client, act8810);

	act8810->client = client;

#if defined(CONFIG_REGULATOR_ACT8810)
	act8810_register_regulator(&client->dev, ACT8810_OUT1);
	act8810_register_regulator(&client->dev, ACT8810_OUT2);
	act8810_register_regulator(&client->dev, ACT8810_OUT3);
#endif 

	ret = act8810_init(act8810);
	if (ret != 0)
		goto err;

	return 0;
err:
	kfree(client);
err_mem:
	return ret;
}

static int act8810_attach_adapter(struct i2c_adapter *adapter)
{
	return i2c_probe(adapter, &addr_data, act8810_probe);
}

static int act8810_detach_client(struct i2c_client *client)
{
	return i2c_detach_client(client);
}

static int __init act8810_module_init(void)
{
	int ret;

	pr_info("ACT8810 regulator driver init\n");

	clk = clk_get(NULL, "i2c");
	if (clk) {
		clk_enable(clk);
	} else {
		pr_err("act8810: can't get i2c clock\n");
		return -ENOMEM;
	}

	ret = i2c_add_driver(&act8810_i2c_driver);
	if (ret != 0) {
		pr_err("act8810: failed to register driver: %d\n", ret);
	}
	return ret;
}
module_init(act8810_module_init);

static void __exit act8810_module_exit(void)
{
	i2c_del_driver(&act8810_i2c_driver);
	clk_disable(clk);
	clk_put(clk);
}
module_exit(act8810_module_exit);

MODULE_AUTHOR("Telechips, Inc.");
MODULE_LICENSE("GPL");
