/*
 * drivers/regulator/lp8725-regulator.c
 *
 * LP8725 PMIC driver
 *
 * Copyright (C) 2009 Telechips, Inc.
 *
 * This software is licensed under the terms of the GNU General Public
 * License version 2, as published by the Free Software Foundation, and
 * may be copied, distributed, and modified under those terms.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 */

#include <linux/kernel.h>
#include <linux/err.h>
#include <linux/platform_device.h>
#include <linux/mfd/lp8725.h>
#include <linux/regulator/driver.h>
#include <linux/regulator/machine.h>
#include <linux/regulator/lp8725.h>

struct lp8725_voltage {
	int uV;
	int val;
};

static int current_voltages[LP8725_NUM_REGULATORS];

static int find_voltage_from_value(struct lp8725_voltage *voltages, int val)
{
	int i;

	for (i = 0; i < 32; i++) {
		if (voltages[i].val == val)
			return voltages[i].uV;
	}
	return 0;
}

static int find_value_from_voltage(struct lp8725_voltage *voltages, int uV)
{
	int i;

	for (i = 0; i < 32; i++) {
		if (voltages[i].uV == uV)
			return voltages[i].val;
	}
	return 0;
}

static int lp8725_ldo_is_enabled(struct regulator_dev *dev)
{
	struct lp8725 *lp8725 = rdev_get_drvdata(dev);
	int id = rdev_get_id(dev);
	int enable;
	int reg;

	reg = lp8725_reg_read(lp8725, LP8725_REG_LDO_CONTROL);
	if (reg < 0)
		return -EINVAL;

	switch (id) {
	case LP8725_LDO1:
		enable = reg & LP8725_LDO1_EN;
		break;

	case LP8725_LDO2:
		enable = reg & LP8725_LDO1_EN;
		break;

	case LP8725_LDO3:
		enable = reg & LP8725_LDO1_EN;
		break;

	case LP8725_LDO4:
		enable = reg & LP8725_LDO1_EN;
		break;

	case LP8725_LDO5:
		enable = reg & LP8725_LDO1_EN;
		break;

	default:
		return -EINVAL;
	}
	return enable;
}

static int lp8725_ldo_enable(struct regulator_dev *dev)
{
	struct lp8725 *lp8725 = rdev_get_drvdata(dev);
	int id = rdev_get_id(dev);
	int mask;
	int ret;

	switch (id) {
	case LP8725_LDO1:
		mask = LP8725_LDO1_EN;
		break;

	case LP8725_LDO2:
		mask = LP8725_LDO2_EN;
		break;

	case LP8725_LDO3:
		mask = LP8725_LDO3_EN;
		break;

	case LP8725_LDO4:
		mask = LP8725_LDO4_EN;
		break;

	case LP8725_LDO5:
		mask = LP8725_LDO5_EN;
		break;

	default:
		return -EINVAL;
	}
	ret = lp8725_reg_set_bit_mask(lp8725, LP8725_REG_LDO_CONTROL, mask, 0xff);
	return ret;
}

static int lp8725_ldo_disable(struct regulator_dev *dev)
{
	struct lp8725 *lp8725 = rdev_get_drvdata(dev);
	int id = rdev_get_id(dev);
	int mask;
	int ret;

	switch (id) {
	case LP8725_LDO1:
		mask = LP8725_LDO1_EN;
		break;

	case LP8725_LDO2:
		mask = LP8725_LDO2_EN;
		break;

	case LP8725_LDO3:
		mask = LP8725_LDO3_EN;
		break;

	case LP8725_LDO4:
		mask = LP8725_LDO4_EN;
		break;

	case LP8725_LDO5:
		mask = LP8725_LDO5_EN;
		break;

	default:
		return -EINVAL;
	}
	ret = lp8725_reg_set_bit_mask(lp8725, LP8725_REG_LDO_CONTROL, mask, 0);
	return ret;
}

static struct lp8725_voltage ldo_voltages[] = {
	{ 1200000, LP8725_LDO_1_20V },
	{ 1250000, LP8725_LDO_1_25V },
	{ 1300000, LP8725_LDO_1_30V },
	{ 1350000, LP8725_LDO_1_35V },
	{ 1400000, LP8725_LDO_1_40V },
	{ 1450000, LP8725_LDO_1_45V },
	{ 1500000, LP8725_LDO_1_50V },
	{ 1550000, LP8725_LDO_1_55V },
	{ 1600000, LP8725_LDO_1_60V },
	{ 1650000, LP8725_LDO_1_65V },
	{ 1700000, LP8725_LDO_1_70V },
	{ 1750000, LP8725_LDO_1_75V },
	{ 1800000, LP8725_LDO_1_80V },
	{ 1850000, LP8725_LDO_1_85V },
	{ 1900000, LP8725_LDO_1_90V },
	{ 2000000, LP8725_LDO_2_00V },
	{ 2100000, LP8725_LDO_2_10V },
	{ 2200000, LP8725_LDO_2_20V },
	{ 2300000, LP8725_LDO_2_30V },
	{ 2400000, LP8725_LDO_2_40V },
	{ 2500000, LP8725_LDO_2_50V },
	{ 2600000, LP8725_LDO_2_60V },
	{ 2650000, LP8725_LDO_2_65V },
	{ 2700000, LP8725_LDO_2_70V },
	{ 2750000, LP8725_LDO_2_75V },
	{ 2800000, LP8725_LDO_2_80V },
	{ 2850000, LP8725_LDO_2_85V },
	{ 2900000, LP8725_LDO_2_90V },
	{ 2950000, LP8725_LDO_2_95V },
	{ 3000000, LP8725_LDO_3_00V },
	{ 3100000, LP8725_LDO_3_10V },
	{ 3300000, LP8725_LDO_3_30V }
};

static int lp8725_ldo_get_voltage(struct regulator_dev *dev)
{
	struct lp8725 *lp8725 = rdev_get_drvdata(dev);
	int id = rdev_get_id(dev);
	int reg;
	int volt;

	switch (id) {
	case LP8725_LDO1:
		reg = lp8725_reg_read(lp8725, LP8725_REG_LDO1);
		if (reg < 0)
			return -EINVAL;
		break;

	case LP8725_LDO2:
		reg = lp8725_reg_read(lp8725, LP8725_REG_LDO2);
		if (reg < 0)
			return -EINVAL;
		break;

	case LP8725_LDO3:
		reg = lp8725_reg_read(lp8725, LP8725_REG_LDO3);
		if (reg < 0)
			return -EINVAL;
		break;

	case LP8725_LDO4:
		reg = lp8725_reg_read(lp8725, LP8725_REG_LDO4);
		if (reg < 0)
			return -EINVAL;
		break;

	case LP8725_LDO5:
		reg = lp8725_reg_read(lp8725, LP8725_REG_LDO5);
		if (reg < 0)
			return -EINVAL;
		break;

	default:
		return -EINVAL;
	}
	reg &= 0x1F;
	volt = find_voltage_from_value(ldo_voltages, reg);

	return volt;
}

static int lp8725_ldo_set_voltage(struct regulator_dev *dev,
				  int min_uV, int max_uV)
{
	struct lp8725 *lp8725 = rdev_get_drvdata(dev);
	int id = rdev_get_id(dev);
	int reg;
	int ret;

	if (current_voltages[id] == min_uV)
		return 0;

	pr_debug("lp8725-regulator: set '%s' to %dmV\n", lp8725->regulators[id].name, min_uV / 1000);

	reg = find_value_from_voltage(ldo_voltages, min_uV);

	switch (id) {
	case LP8725_LDO1:
		ret = lp8725_reg_set_bit_mask(lp8725, LP8725_REG_LDO1, 0x1F, reg);
		break;

	case LP8725_LDO2:
		ret = lp8725_reg_set_bit_mask(lp8725, LP8725_REG_LDO2, 0x1F, reg);
		break;

	case LP8725_LDO3:
		ret = lp8725_reg_set_bit_mask(lp8725, LP8725_REG_LDO3, 0x1F, reg);
		break;

	case LP8725_LDO4:
		ret = lp8725_reg_set_bit_mask(lp8725, LP8725_REG_LDO4, 0x1F, reg);
		break;

	case LP8725_LDO5:
		ret = lp8725_reg_set_bit_mask(lp8725, LP8725_REG_LDO5, 0x1F, reg);
		break;

	default:
		return -EINVAL;
	}
	current_voltages[id] = min_uV;

	return ret;
}

static int lp8725_ldo_set_suspend_enable(struct regulator_dev *dev)
{
	return 0;
}

static int lp8725_ldo_set_suspend_disable(struct regulator_dev *dev)
{
	return 0;
}

static int lp8725_ldo_set_suspend_voltage(struct regulator_dev *dev, int uV)
{
	return 0;
}

static struct regulator_ops lp8725_ldo_ops = {
	.is_enabled = lp8725_ldo_is_enabled,
	.enable = lp8725_ldo_enable,
	.disable = lp8725_ldo_disable,
	.get_voltage = lp8725_ldo_get_voltage,
	.set_voltage = lp8725_ldo_set_voltage,
	.set_suspend_enable = lp8725_ldo_set_suspend_enable,
	.set_suspend_disable = lp8725_ldo_set_suspend_disable,
	.set_suspend_voltage = lp8725_ldo_set_suspend_voltage,
};

static int lp8725_lilo_is_enabled(struct regulator_dev *dev)
{
	struct lp8725 *lp8725 = rdev_get_drvdata(dev);
	int id = rdev_get_id(dev);
	int enable;
	int reg;

	reg = lp8725_reg_read(lp8725, LP8725_REG_LDO_CONTROL);
	if (reg < 0)
		return -EINVAL;

	switch (id) {
	case LP8725_LILO1:
		enable = reg & LP8725_LILO1_EN;
		break;

	case LP8725_LILO2:
		enable = reg & LP8725_LILO2_EN;
		break;

	default:
		return -EINVAL;
	}
	return enable;
}

static int lp8725_lilo_enable(struct regulator_dev *dev)
{
	struct lp8725 *lp8725 = rdev_get_drvdata(dev);
	int id = rdev_get_id(dev);
	int mask;
	int ret;

	switch (id) {
	case LP8725_LILO1:
		mask = LP8725_LILO1_EN;
		break;

	case LP8725_LILO2:
		mask = LP8725_LILO2_EN;
		break;

	default:
		return -EINVAL;
	}
	ret = lp8725_reg_set_bit_mask(lp8725, LP8725_REG_LDO_CONTROL, mask, 0xff);
	return ret;
}

static int lp8725_lilo_disable(struct regulator_dev *dev)
{
	struct lp8725 *lp8725 = rdev_get_drvdata(dev);
	int id = rdev_get_id(dev);
	int mask;
	int ret;

	switch (id) {
	case LP8725_LILO1:
		mask = LP8725_LILO1_EN;
		break;

	case LP8725_LILO2:
		mask = LP8725_LILO2_EN;
		break;

	default:
		return -EINVAL;
	}
	ret = lp8725_reg_set_bit_mask(lp8725, LP8725_REG_LDO_CONTROL, mask, 0);
	return ret;
}

static int lp8725_lilo_set_suspend_enable(struct regulator_dev *dev)
{
	return 0;
}

static int lp8725_lilo_set_suspend_disable(struct regulator_dev *dev)
{
	return 0;
}

static int lp8725_lilo_set_suspend_voltage(struct regulator_dev *dev, int uV)
{
	return 0;
}

static struct regulator_ops lp8725_lilo_ops = {
	.is_enabled = lp8725_lilo_is_enabled,
	.enable = lp8725_lilo_enable,
	.disable = lp8725_lilo_disable,
	.set_suspend_enable = lp8725_lilo_set_suspend_enable,
	.set_suspend_disable = lp8725_lilo_set_suspend_disable,
	.set_suspend_voltage = lp8725_lilo_set_suspend_voltage,
};

static struct lp8725_voltage dcdc_voltages[] = {
	{  800000, LP8725_DCDC_0_80V },
	{  850000, LP8725_DCDC_0_85V },
	{  900000, LP8725_DCDC_0_90V },
	{  950000, LP8725_DCDC_0_95V },
	{ 1000000, LP8725_DCDC_1_00V },
	{ 1050000, LP8725_DCDC_1_05V },
	{ 1100000, LP8725_DCDC_1_10V },
	{ 1150000, LP8725_DCDC_1_15V },
	{ 1200000, LP8725_DCDC_1_20V },
	{ 1250000, LP8725_DCDC_1_25V },
	{ 1300000, LP8725_DCDC_1_30V },
	{ 1350000, LP8725_DCDC_1_35V },
	{ 1400000, LP8725_DCDC_1_40V },
	{ 1500000, LP8725_DCDC_1_50V },
	{ 1600000, LP8725_DCDC_1_60V },
	{ 1700000, LP8725_DCDC_1_70V },
	{ 1750000, LP8725_DCDC_1_75V },
	{ 1800000, LP8725_DCDC_1_80V },
	{ 1850000, LP8725_DCDC_1_85V },
	{ 1900000, LP8725_DCDC_1_90V },
	{ 2000000, LP8725_DCDC_2_00V },
	{ 2100000, LP8725_DCDC_2_10V },
	{ 2200000, LP8725_DCDC_2_20V },
	{ 2300000, LP8725_DCDC_2_30V },
	{ 2400000, LP8725_DCDC_2_40V },
	{ 2500000, LP8725_DCDC_2_50V },
	{ 2600000, LP8725_DCDC_2_60V },
	{ 2700000, LP8725_DCDC_2_70V },
	{ 2800000, LP8725_DCDC_2_80V },
	{ 2850000, LP8725_DCDC_2_85V },
	{ 2900000, LP8725_DCDC_2_90V },
	{ 3000000, LP8725_DCDC_3_00V }
};

static int lp8725_dcdc_get_voltage(struct regulator_dev *dev)
{
	struct lp8725 *lp8725 = rdev_get_drvdata(dev);
	int id = rdev_get_id(dev);
	int reg;
	int volt;

	switch (id) {
	case LP8725_DCDC1:
		reg = lp8725_reg_read(lp8725, LP8725_REG_BUCK1_V1);
		if (reg < 0)
			return -EINVAL;
		break;

	case LP8725_DCDC2:
		reg = lp8725_reg_read(lp8725, LP8725_REG_BUCK2_V1);
		if (reg < 0)
			return -EINVAL;
		break;

	default:
		return -EINVAL;
	}
	reg &= 0x1F;
	volt = find_voltage_from_value(dcdc_voltages, reg);

	return volt;
}

static int lp8725_dcdc_set_voltage(struct regulator_dev *dev,
				   int min_uV, int max_uV)
{
	struct lp8725 *lp8725 = rdev_get_drvdata(dev);
	int id = rdev_get_id(dev);
	int reg;
	int ret;

	if (current_voltages[id] == min_uV)
		return 0;

	pr_debug("lp8725-regulator: set '%s' to %dmV\n", lp8725->regulators[id].name, min_uV / 1000);

	reg = find_value_from_voltage(dcdc_voltages, min_uV);

	switch (id) {
	case LP8725_DCDC1:
		ret = lp8725_reg_set_bit_mask(lp8725, LP8725_REG_BUCK1_V1, 0x1F, reg);
		if (ret < 0)
			break;
		ret = lp8725_reg_set_bit_mask(lp8725, LP8725_REG_BUCK1_V2, 0x1F, reg);
		break;

	case LP8725_DCDC2:
		ret = lp8725_reg_set_bit_mask(lp8725, LP8725_REG_BUCK2_V1, 0x1F, reg);
		if (ret < 0)
			break;
		ret = lp8725_reg_set_bit_mask(lp8725, LP8725_REG_BUCK2_V2, 0x1F, reg);
		break;

	default:
		return -EINVAL;
	}
	current_voltages[id] = min_uV;

	return ret;
}

static int lp8725_dcdc_set_suspend_enable(struct regulator_dev *dev)
{
	return 0;
}

static int lp8725_dcdc_set_suspend_disable(struct regulator_dev *dev)
{
	return 0;
}

static int lp8725_dcdc_set_suspend_voltage(struct regulator_dev *dev, int uV)
{
	return lp8725_dcdc_set_voltage(dev, uV, uV);
}

static struct regulator_ops lp8725_dcdc_ops = {
	.get_voltage = lp8725_dcdc_get_voltage,
	.set_voltage = lp8725_dcdc_set_voltage,
	.set_suspend_enable = lp8725_dcdc_set_suspend_enable,
	.set_suspend_disable = lp8725_dcdc_set_suspend_disable,
	.set_suspend_voltage = lp8725_dcdc_set_suspend_voltage,
};

static struct regulator_consumer_supply vdd_usb33_consumers[] = {
	{
		.supply = "vdd_usb33",
	}
};

static struct regulator_consumer_supply vdd_usb12_consumers[] = {
	{
		.supply = "vdd_usb12",
	}
};

static struct regulator_consumer_supply vdd_core_consumers[] = {
	{
		.supply = "vdd_core",
	},
};

static struct regulator_init_data regulator_initdata[LP8725_NUM_REGULATORS] = {
	[LP8725_LDO1] = {
		.constraints = {
			.name = "vdd_io28",
			.min_uV = 1200000,
			.max_uV = 3300000,
			.valid_modes_mask = REGULATOR_MODE_NORMAL,
			.valid_ops_mask = REGULATOR_CHANGE_VOLTAGE,
			.always_on = 1,
			.boot_on = 1,
		},
	},
	[LP8725_LDO2] = {
		.constraints = {
			.name = "LDO2",
			.min_uV = 1200000,
			.max_uV = 3300000,
			.valid_modes_mask = REGULATOR_MODE_NORMAL,
			.valid_ops_mask = REGULATOR_CHANGE_VOLTAGE,
		},
	},
	[LP8725_LDO3] = {
		.constraints = {
			.name = "vdd_usb33",
			.min_uV = 1200000,
			.max_uV = 3300000,
			.valid_modes_mask = REGULATOR_MODE_NORMAL,
			.valid_ops_mask = REGULATOR_CHANGE_VOLTAGE,
			.boot_on = 1,
		},
		.num_consumer_supplies = ARRAY_SIZE(vdd_usb33_consumers),
		.consumer_supplies = vdd_usb33_consumers,
	},
	[LP8725_LDO4] = {
		.constraints = {
			.min_uV = 1200000,
			.max_uV = 3300000,
			.valid_modes_mask = REGULATOR_MODE_NORMAL,
			.valid_ops_mask = REGULATOR_CHANGE_VOLTAGE,
		},
	},
	[LP8725_LDO5] = {
		.constraints = {
			.min_uV = 1200000,
			.max_uV = 3300000,
			.valid_modes_mask = REGULATOR_MODE_NORMAL,
			.valid_ops_mask = REGULATOR_CHANGE_VOLTAGE,
		},
	},
	[LP8725_LILO1] = {
		.constraints = {
			.name = "vdd_usb12",
			.min_uV = 800000,
			.max_uV = 3300000,
			.valid_modes_mask = REGULATOR_MODE_NORMAL,
			.valid_ops_mask = REGULATOR_CHANGE_VOLTAGE,
			.boot_on = 1,
		},
		.num_consumer_supplies = ARRAY_SIZE(vdd_usb12_consumers),
		.consumer_supplies = vdd_usb12_consumers,
	},
	[LP8725_LILO2] = {
		.constraints = {
			.name = "vdd_io18",
			.min_uV = 800000,
			.max_uV = 3300000,
			.valid_modes_mask = REGULATOR_MODE_NORMAL,
			.valid_ops_mask = REGULATOR_CHANGE_VOLTAGE,
			.always_on = 1,
			.boot_on = 1,
		},
	},
	[LP8725_DCDC1] = {
		.constraints = {
			.name = "vdd_core",
			.min_uV = 1000000,
			.max_uV = 1500000,
			.valid_modes_mask = REGULATOR_MODE_NORMAL,
			.valid_ops_mask = REGULATOR_CHANGE_VOLTAGE,
			.state_mem = {
				.uV = 1050000,
				.mode = REGULATOR_MODE_NORMAL,
				.enabled = 1,
			},
			.always_on = 1,
			.boot_on = 1,
		},
		.num_consumer_supplies = ARRAY_SIZE(vdd_core_consumers),
		.consumer_supplies = vdd_core_consumers,
	},
	[LP8725_DCDC2] = {
		.constraints = {
			.min_uV = 800000,
			.max_uV = 3000000,
			.valid_modes_mask = REGULATOR_MODE_NORMAL,
			.valid_ops_mask = REGULATOR_CHANGE_VOLTAGE,
		},
	},
};

static struct regulator_desc regulators[LP8725_NUM_REGULATORS] = {
	{
		.name = "vdd_io28",
		.id = LP8725_LDO1,
		.ops = &lp8725_ldo_ops,
		.type = REGULATOR_VOLTAGE,
		.owner = THIS_MODULE,
	},
	{
		.name = "LDO2",
		.id = LP8725_LDO2,
		.ops = &lp8725_ldo_ops,
		.type = REGULATOR_VOLTAGE,
		.owner = THIS_MODULE,
	},
	{
		.name = "vdd_usb33",
		.id = LP8725_LDO3,
		.ops = &lp8725_ldo_ops,
		.type = REGULATOR_VOLTAGE,
		.owner = THIS_MODULE,
	},
	{
		.name = "LDO4",
		.id = LP8725_LDO4,
		.ops = &lp8725_ldo_ops,
		.type = REGULATOR_VOLTAGE,
		.owner = THIS_MODULE,
	},
	{
		.name = "LDO5",
		.id = LP8725_LDO5,
		.ops = &lp8725_ldo_ops,
		.type = REGULATOR_VOLTAGE,
		.owner = THIS_MODULE,
	},
	{
		.name = "vdd_usb12",
		.id = LP8725_LILO1,
		.ops = &lp8725_lilo_ops,
		.type = REGULATOR_VOLTAGE,
		.owner = THIS_MODULE,
	},
	{
		.name = "vdd_io18",
		.id = LP8725_LILO2,
		.ops = &lp8725_lilo_ops,
		.type = REGULATOR_VOLTAGE,
		.owner = THIS_MODULE,
	},
	{
		.name = "vdd_core",
		.id = LP8725_DCDC1,
		.ops = &lp8725_dcdc_ops,
		.type = REGULATOR_VOLTAGE,
		.owner = THIS_MODULE,
	},
	{
		.name = "DCDC2",
		.id = LP8725_DCDC2,
		.ops = &lp8725_dcdc_ops,
		.type = REGULATOR_VOLTAGE,
		.owner = THIS_MODULE,
	},
};

static int __devinit lp8725_regulator_probe(struct platform_device *pdev)
{
	struct regulator_dev *rdev;

	rdev = regulator_register(&regulators[pdev->id], &pdev->dev, platform_get_drvdata(pdev));
	if (IS_ERR(rdev))
		return PTR_ERR(rdev);
	return 0;
}

static int __devexit lp8725_regulator_remove(struct platform_device *pdev)
{
	struct regulator_dev *rdev = platform_get_drvdata(pdev);

	regulator_unregister(rdev);

	return 0;
}

int lp8725_register_regulator(struct device *dev, int reg)
{
	struct lp8725 *lp8725 = dev_get_drvdata(dev);
	struct regulator_init_data *initdata;

	if (lp8725->regulators[reg].name)
		return -EBUSY;

	initdata = &regulator_initdata[reg];
	initdata->driver_data = lp8725;

	lp8725->regulators[reg].name = "lp8725-regulator";
	lp8725->regulators[reg].id = reg;
	lp8725->regulators[reg].dev.parent = dev;
	dev_set_drvdata(&lp8725->regulators[reg].dev, lp8725);
	lp8725->regulators[reg].dev.platform_data = initdata;

	return platform_device_register(&lp8725->regulators[reg]);
}
EXPORT_SYMBOL(lp8725_register_regulator);

static struct platform_driver lp8725_regulator_driver = {
	.driver = {
		.name = "lp8725-regulator",
	},
	.probe = lp8725_regulator_probe,
	.remove = lp8725_regulator_remove,
};

static int __init lp8725_regulator_init(void)
{
	return platform_driver_register(&lp8725_regulator_driver);
}
module_init(lp8725_regulator_init);

static void __exit lp8725_regulator_exit(void)
{
	platform_driver_unregister(&lp8725_regulator_driver);
}
module_exit(lp8725_regulator_exit);

MODULE_AUTHOR("Telechips, Inc.");
MODULE_DESCRIPTION("LP8725 regulator driver");
MODULE_LICENSE("GPL");
