/*
 * linux/drivers/serial/tcc_serial.c
 *
 * Based on: drivers/serial/s3c2410.c and driver/serial/bfin_5xx.c
 * Author:  <linux@telechips.com>
 * Created: June 10, 2008
 * Description: Driver for onboard UARTs on the Telechips TCC Series
 *
 * Copyright (C) 2008-2009 Telechips 
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, see the file COPYING, or write
 * to the Free Software Foundation, Inc.,
 * 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */


#include <linux/module.h>
#include <linux/ioport.h>
#include <linux/platform_device.h>
#include <linux/init.h>
#include <linux/sysrq.h>
#include <linux/console.h>
#include <linux/tty.h>
#include <linux/tty_flip.h>
#include <linux/serial_core.h>
#include <linux/serial.h>
#include <linux/delay.h>
#include <linux/clk.h>

#include <asm/io.h>
#include <asm/irq.h>

#include <mach/hardware.h>
#include <asm/plat-tcc/tcc_ckc.h>

#ifdef CONFIG_ARCH_TCC83X
#include <mach/tcc83x_virt.h>
#define DRV_NAME "tcc8300-uart"
#define CONSOLE_PORT 3
#define	CONSOLE_BASE	*(volatile unsigned long *)0xF100A000
#elif CONFIG_ARCH_TCC79X
#include <mach/tcc79x.h>
#define DRV_NAME "tcc7901-uart"
#define CONSOLE_PORT 0
#define	CONSOLE_BASE	*(volatile unsigned long *)0xF0055000	
#endif

#define BAUDRATE  115200

#include "tca_tccserial.h"

#ifdef CONFIG_SERIAL_TCC_DMA
#include <linux/dma-mapping.h>

#define DMA_PORT	1
#define DMA_NUM		6
#define DMA_BUF_SIZE	FIFOSIZE
#endif

#define TIMEOUT     jiffies_to_msecs(1000)
/* UARTx_LCR Masks                                              */
#define WLS(x)      (((x)-5) & 0x03)    /* Word Length Select */
#define STB         0x04                /* Stop Bits            */
#define PEN         0x08                /* Parity Enable        */
#define EPS         0x10                /* Even Parity Select   */
#define SP          0x20                /* Stick Parity         */
#define SB          0x40                /* Set Break            */
#define DLAB        0x80                /* Divisor Latch Access */

/* UARTx_MCR Mask                                       */
#define AFE         0x20    /* Auto Flow Control Enable */
#define LOOP        0x10    /* Loopback Mode Enable     */
#define RTS         0x02    /* Request To Sent          */

/* UARTx_LSR Masks                                      */
#define DR          0x01    /* Data Ready               */
#define OE          0x02    /* Overrun Error            */
#define PE          0x04    /* Parity Error             */
#define FE          0x08    /* Framing Error            */
#define BI          0x10    /* Break Interrupt          */
#define THRE        0x20    /* THR Empty                */
#define TEMT        0x40    /* TSR and UART_THR Empty   */

/* UARTx_IER Masks */
#define ERXI    0x01
#define ETXI    0x02
#define ELSI    0x04

#define OFFSET_THR    0x00	/* Transmit Holding register            */
#define OFFSET_RBR    0x00	/* Receive Buffer register              */
#define OFFSET_DLL    0x00	/* Divisor Latch (Low-Byte)             */
#define OFFSET_IER    0x04	/* Interrupt Enable Register            */
#define OFFSET_DLM    0x04	/* Divisor Latch (High-Byte)            */
#define OFFSET_IIR    0x08	/* Interrupt Identification Register    */
#define OFFSET_FCR    0x08	/* FIFO Control Register                */
#define OFFSET_LCR    0x0C	/* Line Control Register                */
#define OFFSET_MCR    0x10	/* Modem Control Register               */
#define OFFSET_LSR    0x14	/* Line Status Register                 */
#define OFFSET_MSR    0x18	/* Modem Status Register                */
#define OFFSET_SCR    0x1C	/* SCR Scratch Register                 */
#define OFFSET_AFT    0x20	/* AFC Trigger Level Register           */
#define OFFSET_UCR    0x24	/* UART Control Register                */

#define portaddr(port, reg) ((port)->membase + (reg))

#define rd_regb(port, reg) (__raw_readb(portaddr(port, reg)))
#define rd_regl(port, reg) (__raw_readl(portaddr(port, reg)))

#define wr_regl(port, reg, val) \
    do { __raw_writel(val, portaddr(port, reg)); } while(0)
#define wr_regb(port, reg, val) \
    do { __raw_writeb(val, portaddr(port, reg)); } while(0)



/* structures */

struct tcc_uart_clksrc {
	const char	    *name;
	unsigned int	divisor;
	unsigned int	min_baud;
	unsigned int	max_baud;
};

struct tcc_uart_info {
	char			*name;
	unsigned int	type;
	unsigned int	fifosize;
};

struct tcc_reg_info {
	unsigned long	bDLL;
	unsigned long	bIER;
	unsigned long	bDLM;
	unsigned long	bLCR;
	unsigned long	bMCR;
	unsigned long	bSCR;
	unsigned long	bFCR;
	unsigned long	bMSR;
	unsigned long	bAFT;
	unsigned long	bUCR;
};

#ifdef CONFIG_SERIAL_TCC_DMA
typedef struct _tcc_dma_buf_t {
    char *addr;
    dma_addr_t dma_addr;
    int  buf_size; // total size of DMA
} tcc_dma_buf_t;
#endif
struct tcc_uart_port {
	unsigned char			rx_claimed;
	unsigned char			tx_claimed;

	struct tcc_uart_info	*info;
	struct tcc_uart_clksrc	*clksrc;
	struct clk				*clk;
	struct clk				*baudclk;
	struct uart_port		port;
    wait_queue_head_t wait_q;
    int    fifosize;
    char *name;
#ifdef CONFIG_SERIAL_TCC_DMA
    wait_queue_head_t wait_dma_q;
	tcc_dma_buf_t	dma_buffer;
	tcc_dma_buf_t	dma_buffer1;
#endif
	struct tcc_reg_info		reg;	// for suspend/resume
};

/* configuration defines */

#if 0
//#define dbg(x...) printk(KERN_DEBUG "tcc uart: ");
#define dbg printk
#else /* no debug */
#define dbg(x...) do {} while(0)
#endif

/* UART name and device definitions */

#define TCC_SERIAL_NAME	"ttySAC"
#define TCC_SERIAL_MAJOR	204
#define TCC_SERIAL_MINOR	64

/* conversion functions */

#define tcc_dev_to_port(__dev) (struct uart_port *)dev_get_drvdata(__dev)

/* Number of serial ports */
#define NR_PORTS (4)

#define FIFOSIZE (16) /* defend on FCR */

#define tx_enabled(port) ((port)->unused[0])
#define rx_enabled(port) ((port)->unused[1])
#define port_used(port)		((port)->unused1)

static void tcc_serial_console_putchar(struct uart_port *port, int ch);
extern void tcc_serial_uart_putchar(struct uart_port *port, int ch);

int tcc_serial_init_ports(struct tcc_uart_port *ourport, struct tcc_uart_info *info, struct platform_device *pdev);

static struct uart_ops tcc_serial_ops;

static struct tcc_uart_port tcc_serial_ports[NR_PORTS] = {
	[0] = {
		.port = {
			.lock		= __SPIN_LOCK_UNLOCKED(tcc_uart_port[0].port.lock),
			.iotype		= UPIO_MEM,
			.irq		= INT_UT0,
			.uartclk	= 0,
			.fifosize	= FIFOSIZE,
			.ops		= &tcc_serial_ops,
			.flags		= UPF_BOOT_AUTOCONF,
			.line		= 0,
		},
        .name = "uart0"
	},
	[1] = {
		.port = {
			.lock		= __SPIN_LOCK_UNLOCKED(tcc_uart_port[1].port.lock),
			.iotype		= UPIO_MEM,
			.uartclk	= 0,
			.irq		= INT_UT1,
			.fifosize	= FIFOSIZE,
			.ops		= &tcc_serial_ops,
			.flags		= UPF_BOOT_AUTOCONF,
			.line		= 1,
		},
        .name = "uart1"
	},
	[2] = {
		.port = {
			.lock		= __SPIN_LOCK_UNLOCKED(tcc_uart_port[2].port.lock),
			.iotype		= UPIO_MEM,
			.irq		= INT_UT2,
			.uartclk	= 0,
			.fifosize	= FIFOSIZE,
			.ops		= &tcc_serial_ops,
			.flags		= UPF_BOOT_AUTOCONF,
			.line		= 2,
		},
        .name = "uart2"
	},
	[3] = {
		.port = {
			.lock		= __SPIN_LOCK_UNLOCKED(tcc_uart_port[3].port.lock),
			.iotype		= UPIO_MEM,
			.irq		= INT_UT3,
			.uartclk	= 0,
			.fifosize	= FIFOSIZE,
			.ops		= &tcc_serial_ops,
			.flags		= UPF_BOOT_AUTOCONF,
			.line		= 3,
		},
        .name = "uart3"
	},

};

static inline struct tcc_uart_port *to_ourport(struct uart_port *port)
{
	return container_of(port, struct tcc_uart_port, port);
}

/* translate a port to the device name */

static inline const char *tcc_serial_portname(struct uart_port *port)
{
	return to_platform_device(port->dev)->name;
}


static void tcc_serial_stop_tx(struct uart_port *port)
{
    while(!(rd_regl(port, OFFSET_LSR)) & TEMT)
        continue;

    if(tx_enabled(port)){
        disable_irq(port->irq);
        tx_enabled(port) = 0;
    }
}

static int get_writable_fifo_size(struct uart_port *port)
{
    unsigned int ier; 
#ifdef CONFIG_SERIAL_TCC_DMA
	struct circ_buf *xmit = &port->info->xmit;
    int tmp;
#endif
    ier = rd_regl(port, OFFSET_IER);

    wr_regl(port, OFFSET_IER, ier | Hw1);

#if 0
    if (interruptible_sleep_on_timeout(&(tcc_serial_ports[port->line].wait_q), TIMEOUT) != 0) {
#ifdef CONFIG_SERIAL_TCC_DMA
        tmp = xmit->head - xmit->tail;
        if(port->line== DMA_PORT)
        {
            if (tmp > 0)
                return tmp;
            else 
                return UART_XMIT_SIZE + tmp;
        }
        else
#endif
        return FIFOSIZE;
    }
#endif


#ifdef CONFIG_SERIAL_TCC_DMA
    if (interruptible_sleep_on_timeout(&(tcc_serial_ports[port->line].wait_q), TIMEOUT) != 0) {
        if(port->line== DMA_PORT) {
            int f_size = 0;
            tmp = xmit->head - xmit->tail;
            f_size = ((tmp > 0) ? tmp : UART_XMIT_SIZE + tmp);
            if (f_size > FIFOSIZE) {
                f_size = FIFOSIZE;
            }
            return f_size;
        } else {
            return FIFOSIZE;
        }
    }

    if(port->line== DMA_PORT) {
        tmp = xmit->head - xmit->tail;
        return ((tmp > 0) ? tmp : UART_XMIT_SIZE + tmp);
    } else if (interruptible_sleep_on_timeout(&(tcc_serial_ports[port->line].wait_q), TIMEOUT) != 0) {
        return FIFOSIZE;
    }
#else
    if (interruptible_sleep_on_timeout(&(tcc_serial_ports[port->line].wait_q), TIMEOUT) != 0) {
        return FIFOSIZE;
    }
#endif
    return 0;
}

static void update_fifo_size(struct uart_port *port)
{
#ifdef CONFIG_SERIAL_TCC_DMA
    if(port->line == DMA_PORT)
        tcc_serial_ports[port->line].fifosize = get_writable_fifo_size(port);
    else if (tcc_serial_ports[port->line].fifosize <= 0) 
        tcc_serial_ports[port->line].fifosize = get_writable_fifo_size(port);
#else 
    if (tcc_serial_ports[port->line].fifosize <= 0) {
        tcc_serial_ports[port->line].fifosize = get_writable_fifo_size(port);
    }
#endif
}

static void tcc_serial_start_tx(struct uart_port *port)
{
	struct circ_buf *xmit = &port->info->xmit;
	unsigned long flags;
#ifdef CONFIG_SERIAL_TCC_DMA
    struct tcc_uart_port *ourport;
	static int buf_select = 0;
#endif

    if(!tx_enabled(port)) {
        enable_irq(port->irq);
        tx_enabled(port) = 1;
    }

	local_irq_save(flags);
	if (port->x_char) {
        update_fifo_size(port);
        if (tcc_serial_ports[port->line].fifosize > 0) {
            tcc_serial_uart_putchar(port, port->x_char);
            port->icount.tx++;
            port->x_char = 0;
            tcc_serial_ports[port->line].fifosize--;
        }
	}

     if (uart_circ_empty(xmit) || uart_tx_stopped(port)) {
         tcc_serial_stop_tx(port);
         goto out;
     }
	
     while(!(uart_circ_empty(xmit))) {
        update_fifo_size(port);
        if (tcc_serial_ports[port->line].fifosize > 0) {
#ifdef CONFIG_SERIAL_TCC_DMA
            
            if(port->line==DMA_PORT)
            {
                ourport = &tcc_serial_ports[port->line];

                 if(buf_select > 100)
                     buf_select = 0;

                 if(tcc_serial_ports[DMA_PORT].fifosize > 0)
                 {
                     wr_regl(port, OFFSET_UCR, rd_regl(port, OFFSET_UCR) & ~Hw0);
                     if(buf_select%2)
                     {
                         memcpy(tcc_serial_ports[DMA_PORT].dma_buffer.addr, (xmit->buf+xmit->tail),tcc_serial_ports[DMA_PORT].fifosize );

                         //DMA Setting
                         tca_dma_setconfig(DMA_NUM,
                                           (void *)tcc_serial_ports[DMA_PORT].dma_buffer.dma_addr, 	0x1, /* src Param */
                                           portaddr(port, 0x0), 0x0,
                                           HwCHCTRL_SYNC_EN		|
                                           HwCHCTRL_TYPE_SL		|
                                           HwCHCTRL_BSIZE_1		|
                                           HwCHCTRL_WSIZE_8		|
                                           HwCHCTRL_IEN_ON			|
                                           HwCHCTRL_FLAG			,	
                                           tcc_serial_ports[DMA_PORT].fifosize			,
                                           DMA_PORT, 0);
                     } else {
                         memcpy(tcc_serial_ports[DMA_PORT].dma_buffer1.addr, xmit->buf+xmit->tail, tcc_serial_ports[DMA_PORT].fifosize);

                         //DMA Setting
                         tca_dma_setconfig(DMA_NUM,
                                           (void *)tcc_serial_ports[DMA_PORT].dma_buffer1.dma_addr, 	0x1, /* src*/
                                           portaddr(port, 0x0), 0x0,
                                           HwCHCTRL_SYNC_EN		|
                                           HwCHCTRL_TYPE_SL		|
                                           HwCHCTRL_BSIZE_1		|
                                           HwCHCTRL_WSIZE_8		|
                                           HwCHCTRL_IEN_ON			|
                                           HwCHCTRL_FLAG			,	
                                           tcc_serial_ports[DMA_PORT].fifosize			,
                                           DMA_PORT, 0);
                     }

                     wr_regl(port, OFFSET_UCR, rd_regl(port, OFFSET_UCR) |Hw0);
                     interruptible_sleep_on_timeout(&(tcc_serial_ports[port->line].wait_dma_q), TIMEOUT);

                     xmit->tail = (xmit->tail + tcc_serial_ports[DMA_PORT].fifosize) & (UART_XMIT_SIZE - 1);
                     
                     port->icount.tx+= tcc_serial_ports[port->line].fifosize;
                     //tcc_serial_ports[port->line].fifosize-=tcc_serial_ports[DMA_PORT].fifosize;
                     tcc_serial_ports[port->line].fifosize = 0;

                     buf_select++;
                 }

             } else {
                 tcc_serial_uart_putchar(port, xmit->buf[xmit->tail]);
            xmit->tail = (xmit->tail + 1) & (UART_XMIT_SIZE - 1);
            port->icount.tx++;
            tcc_serial_ports[port->line].fifosize--;
        }
#else
         tcc_serial_uart_putchar(port, xmit->buf[xmit->tail]);
         xmit->tail = (xmit->tail + 1) & (UART_XMIT_SIZE - 1);
         port->icount.tx++;
         tcc_serial_ports[port->line].fifosize--;
#endif
         }// if (tcc_serial_ports[port->line].fifosize > 0) 
     }// while(!(uart_circ_empty(xmit))) 

    if (uart_circ_chars_pending(xmit) < WAKEUP_CHARS)
        uart_write_wakeup(port);

out:
	local_irq_restore(flags);
}


static void tcc_serial_stop_rx(struct uart_port *port)
{
	dbg("%s\n", __func__);
    if (rx_enabled(port)) {
        disable_irq(port->irq);
        rx_enabled(port) = 0;
    }

}

static void tcc_serial_enable_ms(struct uart_port *port)
{
	dbg("%s\n", __func__);
}

static inline struct tcc_uart_info *tcc_port_to_info(struct uart_port *port)
{
	return to_ourport(port)->info;
}

#ifdef CONFIG_SERIAL_TCC_DMA
static irqreturn_t tcc_serial_interrupt_dma(int irq, void *id)
{
	struct uart_port *port = id;

    spin_lock(port->lock);
    if (port->line == DMA_PORT) {
        wake_up(&(tcc_serial_ports[port->line].wait_dma_q));
        tca_serial_dmaclrinterrupt(DMA_NUM);
    }
    spin_unlock(port->lock);
	return IRQ_HANDLED;
}
#endif
static irqreturn_t tcc_serial_interrupt(int irq, void *id)
{
	unsigned int ch = 0;
	struct uart_port *port = id;
	unsigned uerstat, flag;
	struct tty_struct *tty = port->info->port.tty;
    int rx_flag = 0;
    unsigned int iir_data = 0;
    unsigned int t_ier;
    
    uerstat = 0;
	
    spin_lock(port->lock);

    iir_data = rd_regl(port, OFFSET_IIR);
    iir_data = (iir_data & 0x0E) >> 1;
    if (iir_data & 0x02 || iir_data & 0x06) {
        ch = rd_regb(port, OFFSET_RBR);
        rx_flag = 1;
    } else if (iir_data & 0x01) {
        t_ier = rd_regl(port, OFFSET_IER);
        wr_regl(port, OFFSET_IER, (t_ier & ~Hw1));

        wake_up(&(tcc_serial_ports[port->line].wait_q));
    }

    if (rx_flag) {
        flag = TTY_NORMAL;
        port->icount.rx++;

        if (uerstat & HwUART_LSR_BI_ON) {
            dbg("break!\n");
            port->icount.brk++;
            goto out;
        }

        if (uerstat & HwUART_LSR_FE_ON)
            port->icount.frame++;
        if (uerstat & HwUART_LSR_OE_ON)
            port->icount.overrun++;

        uerstat &= port->read_status_mask;

        if (uerstat & HwUART_LSR_BI_ON)
            flag = TTY_BREAK;
        else if (uerstat & HwUART_LSR_PE_ON)
            flag = TTY_PARITY;
        else if (uerstat & ( HwUART_LSR_FE_ON | HwUART_LSR_OE_ON))
            flag = TTY_FRAME;

        if (uart_handle_sysrq_char(port, ch))
            goto out;
        /* put the received char into UART buffer */
        uart_insert_char(port, uerstat, HwUART_LSR_OE_ON, ch, flag);

        tty_flip_buffer_push(tty);
    }

out:
    spin_unlock(port->lock);
	return IRQ_HANDLED;
}

static unsigned int tcc_serial_tx_empty(struct uart_port *port)
{
	unsigned short lsr;

	lsr = rd_regl(port, OFFSET_LSR);
	if (lsr & TEMT)
		return TIOCSER_TEMT;
	else
	return 0;
}

static unsigned int tcc_serial_get_mctrl(struct uart_port *port)
{
	dbg("%s\n", __func__);
    return TIOCM_CTS | TIOCM_DSR | TIOCM_CAR;
}

static void tcc_serial_set_mctrl(struct uart_port *port, unsigned int mctrl)
{
	dbg("%s\n", __func__);
	/* todo - possibly remove AFC and do manual CTS */
}

static void tcc_serial_break_ctl(struct uart_port *port, int break_state)
{
	dbg("%s\n", __func__);
}

static void tcc_serial_shutdown(struct uart_port *port)
{
    //interruptible_sleep_on_timeout(&(tcc_serial_ports[port->line].wait_q), 1 * 100);
    wr_regl(port, OFFSET_IER, 0x0);

    free_irq(port->irq, port);
#ifdef CONFIG_SERIAL_TCC_DMA
	if(port->line == DMA_PORT)
    	free_irq(INT_DMA2_CH0, port);
#endif
	port_used(port) = 0;	// for suspend/resume
}

/* while application opening the console device, this function will invoked
 * This function will initialize the interrupt handling 
 */
static int tcc_serial_startup(struct uart_port *port)
{
	int retval=0;
	unsigned int lcr;
	
	dbg("%s:%d\n", __func__, port->line);

    tx_enabled(port) = 1;
    rx_enabled(port) = 1;
	port_used(port) = 1;	// for suspend/resume

    tca_serial_portinit(port->line);

    wr_regl(port, OFFSET_IER, 0x0);
    
    lcr = rd_regl(port, OFFSET_LCR);
    wr_regl(port, OFFSET_LCR, (lcr | Hw7));
    
    wr_regl(port, OFFSET_FCR, 0x07);
    tcc_serial_ports[port->line].fifosize = FIFOSIZE;
    tcc_serial_ports[port->line].reg.bFCR = 0x07;		// for resume restore

    wr_regl(port, OFFSET_LCR, (lcr & (~Hw7)));

    rd_regl(port, OFFSET_IIR);
    
    retval = request_irq(port->irq, tcc_serial_interrupt, IRQF_SHARED, tcc_serial_ports[port->line].name , port);

#ifdef CONFIG_SERIAL_TCC_DMA
    if(port->line == DMA_PORT)
    	retval = request_irq(INT_DMA2_CH0, tcc_serial_interrupt_dma, IRQF_SHARED, "uart_dma" , port);
#endif
    dbg("request serial irq:%d,retval:%d\n", port->irq,retval);
	
    //wr_regl(port, OFFSET_IER, ERXI);
    wr_regl(port, OFFSET_IER, ERXI | ETXI);
    

    tca_serial_intrdone();

	return retval;
}

/* power power management control */

static void tcc_serial_pm(struct uart_port *port, unsigned int level, unsigned int old)
{
}


static inline int tcc_serial_setsource(struct uart_port *port, struct tcc_uart_clksrc *c)
{
	return 0;
}

static void tcc_serial_set_termios(struct uart_port *port,
				       struct ktermios *termios,
				       struct ktermios *old)
{
	unsigned long flags;
	unsigned int baud, quot;
	unsigned int ulcon;
	//unsigned int umcon;
	unsigned int umcon, lsr;
    int uart_clk = 0;

	/*
	 * We don't support modem control lines.
	 */
	termios->c_cflag &= ~(HUPCL | CMSPAR);
	termios->c_cflag |= CLOCAL;

	/*
	 * Ask the core to calculate the divisor for us.
	 */

	baud = uart_get_baud_rate(port, termios, old, 0, 3500000);

    if(port->line == 0){
        uart_clk = tca_ckc_getperi(PERI_UART0);
    } else if(port->line == 1){
        uart_clk = tca_ckc_getperi(PERI_UART1);
    } else if(port->line == 2){
        uart_clk = tca_ckc_getperi(PERI_UART2);
    } else if(port->line == 3){
        uart_clk = tca_ckc_getperi(PERI_UART3);
    }

	quot = (uart_clk*100)/(16*baud); 

    /*
     * set byte size
     */
	switch (termios->c_cflag & CSIZE) {
	case CS5:
		dbg("config: 5bits/char\n");
		ulcon = 0;
		break;
	case CS6:
		dbg("config: 6bits/char\n");
		ulcon = 1;
		break;
	case CS7:
		dbg("config: 7bits/char\n");
		ulcon = 2;
		break;
	case CS8:
	default:
		dbg("config: 8bits/char\n");
		ulcon = 3;
		break;
	}

	/* preserve original lcon IR settings */
	
	if (termios->c_cflag & CSTOPB)
		ulcon |= (1<<2); /* HwUART_LCR_STB_ONE */

	umcon = (termios->c_cflag & CRTSCTS) ? (Hw5|Hw1) : 0; /* HwUART_MCR_RTS_ON */

	if (termios->c_cflag & PARENB) {
		if (termios->c_cflag & PARODD)
			ulcon |= ((0<<4)|(1<<3));
		else
			ulcon |= ((1<<4)|(1<<3));
	} else {
		ulcon |= (0<<3);
	}

	spin_lock_irqsave(&port->lock, flags);

	do {
		lsr = rd_regl(port, OFFSET_LSR);
	} while (!(lsr & TEMT));

	dbg("setting ulcon to %08x, brddiv to %d\n", ulcon, quot);

    wr_regl(port, OFFSET_MCR, umcon);
    wr_regl(port, OFFSET_LCR, (ulcon | (1<<7)));
    if(quot > 0xFF) {
        wr_regl(port, OFFSET_DLL, 0x00FF);
        wr_regl(port, OFFSET_DLM, quot >> 8);
    }else if (quot > 0) {
    wr_regl(port, OFFSET_DLL, quot);
        wr_regl(port, OFFSET_DLM, 0x0);
    }
    wr_regl(port, OFFSET_LCR, (ulcon & (~(1<<7))));

	/*
	 * Update the per-port timeout.
	 */
	uart_update_timeout(port, termios->c_cflag, baud);

	/*
	 * Which character status flags are we interested in?
	 */
	port->read_status_mask = 0;
	if (termios->c_iflag & INPCK)
		port->read_status_mask |= 0;

	/*
	 * Which character status flags should we ignore?
	 */
	port->ignore_status_mask = 0;
	if (termios->c_iflag & IGNPAR)
		port->ignore_status_mask |= 0;
	if (termios->c_iflag & IGNBRK && termios->c_iflag & IGNPAR)
		port->ignore_status_mask |= 0;

	/*
	 * Ignore all characters if CREAD is not set.
	 */
	if ((termios->c_cflag & CREAD) == 0)
		port->ignore_status_mask |= 0;

	spin_unlock_irqrestore(&port->lock, flags);
}

static const char *tcc_serial_type(struct uart_port *port)
{
	return "TCC_UART";
}

#define MAP_SIZE (0x100)

static void tcc_serial_release_port(struct uart_port *port)
{
    /* TODO */
	//release_mem_region(port->mapbase, MAP_SIZE);
}

static int tcc_serial_request_port(struct uart_port *port)
{
    return 0;
    /*
	return request_mem_region(port->mapbase, MAP_SIZE, "tcc7901") ? 0 : -EBUSY;
    */
}

static void tcc_serial_config_port(struct uart_port *port, int flags)
{
	if (flags & UART_CONFIG_TYPE && tcc_serial_request_port(port) == 0)
        port->type = PORT_TCC;
}

/*
 * verify the new serial_struct (for TIOCSSERIAL).
 */
static int
tcc_serial_verify_port(struct uart_port *port, struct serial_struct *ser)
{
	struct tcc_uart_info *info = tcc_port_to_info(port);

	if (ser->type != PORT_UNKNOWN && ser->type != info->type)
		return -EINVAL;
	return 0;
}


#ifdef CONFIG_SERIAL_TCC_CONSOLE

static struct console tcc_serial_console;

#define TCC_SERIAL_CONSOLE &tcc_serial_console
#else
#define TCC_SERIAL_CONSOLE NULL
#endif

static struct uart_ops tcc_serial_ops = {
	.pm				= tcc_serial_pm,
	.tx_empty		= tcc_serial_tx_empty,
	.get_mctrl		= tcc_serial_get_mctrl,
	.set_mctrl		= tcc_serial_set_mctrl,
	.stop_tx		= tcc_serial_stop_tx,
	.start_tx		= tcc_serial_start_tx,
	.stop_rx		= tcc_serial_stop_rx,
	.enable_ms		= tcc_serial_enable_ms,
	.break_ctl		= tcc_serial_break_ctl,
	.startup		= tcc_serial_startup,
	.shutdown		= tcc_serial_shutdown,
	.set_termios	= tcc_serial_set_termios,
	.type			= tcc_serial_type,
	.release_port	= tcc_serial_release_port,
	.request_port	= tcc_serial_request_port,
	.config_port	= tcc_serial_config_port,
	.verify_port	= tcc_serial_verify_port,
};


static struct uart_driver tcc_uart_drv = {
	.owner		= THIS_MODULE,
	.dev_name	= "tcc-uart",
	.nr		    = NR_PORTS,
	.cons		= TCC_SERIAL_CONSOLE,
	.driver_name    = TCC_SERIAL_NAME,
	.major		= TCC_SERIAL_MAJOR,
	.minor		= TCC_SERIAL_MINOR,
};


/*  initialise  serial port information */
/* cpu specific variations on the serial port support */

#if 0
static int tcc_serial_getsource(struct uart_port *port,
				    struct tcc_uart_clksrc *clk)
{
	return 0;
}
#endif

static struct tcc_uart_info tcc_uart_inf = {
	.name		    = "Telechips UART",
	.type	    	= PORT_TCC,
	.fifosize   	= FIFOSIZE,
};

#ifdef CONFIG_SERIAL_TCC_DMA
static void *tcc_free_dma_buf(tcc_dma_buf_t *dma_buf)
{
	dbg("%s\n", __func__);
    if (dma_buf) {
        if (dma_buf->dma_addr != 0) {
            dma_free_writecombine(0, dma_buf->buf_size, dma_buf->addr, dma_buf->dma_addr);
        }
        memset(dma_buf, 0, sizeof(tcc_dma_buf_t));
    }
    return NULL;
}

static void *tcc_malloc_dma_buf(tcc_dma_buf_t *dma_buf, int buf_size)
{


	dbg("%s\n", __func__);
    if (dma_buf) {
        tcc_free_dma_buf(dma_buf);
        dma_buf->buf_size = buf_size;
        dma_buf->addr = dma_alloc_writecombine(0, dma_buf->buf_size, &dma_buf->dma_addr, GFP_KERNEL);
        dbg("Malloc DMA buffer @0x%X(Phy=0x%X), size:%d\n",
               (unsigned int)dma_buf->addr,
               (unsigned int)dma_buf->dma_addr,
               dma_buf->buf_size);
        return dma_buf->addr;
    }
    return NULL;
}
#endif
static int probe_index = 0;

static int tcc_uart_probe(struct platform_device *dev, struct tcc_uart_info *info)
{
	int ret;
    struct tcc_uart_port *ourport;
	
	dbg("%s\n", __func__);
    
    tca_serial_intrinit();

    ourport = &tcc_serial_ports[probe_index];

    probe_index++;

    ret = tcc_serial_init_ports(ourport, info, dev);
    init_waitqueue_head(&(ourport->wait_q));

#ifdef CONFIG_SERIAL_TCC_DMA
    init_waitqueue_head(&(ourport->wait_dma_q));
#endif
    if (ret < 0){
        dbg("tcc_serial_init_ports failure\n");
        goto probe_err;
    }

    ret = uart_add_one_port(&tcc_uart_drv, &ourport->port);

    if (ret){
        dbg("uart_add_one_port failure\n");
        return ret;
    }

    platform_set_drvdata(dev, &ourport->port);

#ifdef CONFIG_SERIAL_TCC_DMA
    if((&ourport->port)->line == DMA_PORT)
    {
        if (!tcc_malloc_dma_buf(&(tcc_serial_ports[DMA_PORT].dma_buffer), DMA_BUF_SIZE)) {
            dbg("Unable to attach UART TX DMA 1 channel\n");
                ret = -ENOMEM;
            goto probe_err;
            }
        if (!tcc_malloc_dma_buf(&(tcc_serial_ports[DMA_PORT].dma_buffer1), DMA_BUF_SIZE)) {
            dbg("Unable to attach UART TX DMA 2 channel\n");
                ret = -ENOMEM;
            goto probe_err;
            }
    }
#endif
	return ret;

probe_err:
    dbg("probe_err\n");
	return ret;
}


static int tcc_serial_remove(struct platform_device *dev)
{
	struct uart_port *port = tcc_dev_to_port(&dev->dev);

	if (port)
		uart_remove_one_port(&tcc_uart_drv, port);

#ifdef CONFIG_SERIAL_TCC_DMA
    if(port->line == DMA_PORT)
    {
        tcc_free_dma_buf(&(tcc_serial_ports[DMA_PORT].dma_buffer));
        tcc_free_dma_buf(&(tcc_serial_ports[DMA_PORT].dma_buffer1));
    }
#endif
	return 0;
}

/* UART power management code */

#ifdef CONFIG_PM
/*-------------------------------------------------
 * TODO: handling DMA_PORT suspend/resume (TCC79X)
 *       DMA stop and start ...
 *-------------------------------------------------*/
static int tcc_serial_suspend(struct platform_device *dev, pm_message_t state)
{
	struct uart_port *port = tcc_dev_to_port(&dev->dev);
	struct tcc_reg_info *reg = &tcc_serial_ports[port->line].reg;

	if (port && port_used(port)) {
		port->suspended = 1;

		// disable interrupt
		wr_regl(port, OFFSET_IER, (rd_regl(port, OFFSET_IER) & (~HwUART_IER_ELSI_EN)));

		reg->bLCR = rd_regl(port, OFFSET_LCR);

		// DLAB = 0
		wr_regl(port, OFFSET_LCR, (reg->bLCR & (~Hw7)));
		reg->bIER = rd_regl(port, OFFSET_IER);

		// DLAB = 1
		wr_regl(port, OFFSET_LCR, (reg->bLCR | Hw7));
//		reg->bFCR = 0x07;						// set in tcc_serial_startup()			
		reg->bDLL = rd_regl(port, OFFSET_DLL);
		reg->bDLM = rd_regl(port, OFFSET_DLM);

		reg->bMCR = rd_regl(port, OFFSET_MCR);
		reg->bAFT = rd_regl(port, OFFSET_AFT);
		reg->bUCR = rd_regl(port, OFFSET_UCR);
	}
	return 0;
}

static int tcc_serial_resume(struct platform_device *dev)
{
	struct uart_port *port = tcc_dev_to_port(&dev->dev);
	struct tcc_reg_info *reg = &tcc_serial_ports[port->line].reg;

	if (port && port_used(port)) {
		if (port->suspended) {
			// DLAB = 0
			wr_regl(port, OFFSET_LCR, (reg->bLCR & (~Hw7)));
			wr_regl(port, OFFSET_IER, reg->bIER);

			// DLAB = 1
			wr_regl(port, OFFSET_LCR, (reg->bLCR | Hw7));
			wr_regl(port, OFFSET_FCR, reg->bFCR);
			wr_regl(port, OFFSET_DLL, reg->bDLL);
			wr_regl(port, OFFSET_DLM, reg->bDLM);

			wr_regl(port, OFFSET_MCR, reg->bMCR);
			wr_regl(port, OFFSET_AFT, reg->bAFT);
			wr_regl(port, OFFSET_UCR, reg->bUCR);

			port->suspended = 0;
			// DLAB = 0
			wr_regl(port, OFFSET_LCR, (reg->bLCR & (~Hw7)));
			wr_regl(port, OFFSET_IER, (rd_regl(port, OFFSET_IER) | HwUART_IER_ELSI_EN));

			wr_regl(port, OFFSET_LCR, reg->bLCR);
		}
	}
	
	return 0;
}

#else
#define tcc_serial_suspend NULL
#define tcc_serial_resume  NULL
#endif

static int tcc_serial_probe(struct platform_device *dev)
{
	return tcc_uart_probe(dev, &tcc_uart_inf);
}

static struct platform_driver tcc_serial_drv = {
	.probe		= tcc_serial_probe,
	.remove		= tcc_serial_remove,
	.suspend	= tcc_serial_suspend,
	.resume		= tcc_serial_resume,
	.driver		= {
		.name	= DRV_NAME,
		.owner	= THIS_MODULE,
	},
};

static int tcc_serial_init(struct platform_driver *drv,
			       struct tcc_uart_info *info)
{
	dbg("%s(%p,%p)\n", __func__, drv, info);
	return platform_driver_register(drv);
}

static inline void tcc_serial_exit(void)
{
	platform_driver_unregister(&tcc_serial_drv);
}


/* module initialisation code */

static int __init tcc_serial_modinit(void)
{
	int ret;
	
	ret = uart_register_driver(&tcc_uart_drv);
	if (ret < 0) {
		dbg(KERN_ERR "failed to register UART driver\n");
		return -1;
	}
	//printk("Serial driver %s registered.\n", tcc_uart_drv.driver_name);
	
	tcc_serial_init(&tcc_serial_drv, &tcc_uart_inf);

	return 0;
}

static void __exit tcc_serial_modexit(void)
{
	tcc_serial_exit();

	uart_unregister_driver(&tcc_uart_drv);
}


module_init(tcc_serial_modinit);
module_exit(tcc_serial_modexit);

/************************************************************
 *
 * The following is Console driver 
 *
 ************************************************************/

#ifdef CONFIG_SERIAL_TCC_CONSOLE

static struct uart_port *cons_uart;

static void tcc_serial_console_putchar(struct uart_port *port, int ch)
{
	while (!(rd_regl(port, OFFSET_LSR) & THRE))
		barrier();

    wr_regb(port, OFFSET_THR, ch);

}

void tcc_serial_uart_putchar(struct uart_port *port, int ch)
{
	while (!(rd_regl(port, OFFSET_LSR) & THRE))
		barrier();

    wr_regb(port, OFFSET_THR, ch);
}


static void tcc_console_write(struct console *co, const char *s,
			     unsigned int count)
{
	struct uart_port *port;
	unsigned int t_ier, b_ier; 
#ifdef CONFIG_ARCH_TCC83X
    unsigned int t_IEN, t_IEN_B;
#endif

	port = &tcc_serial_ports[co->index].port;

    t_ier = rd_regl(port, OFFSET_IER);
    b_ier = t_ier;

#ifdef CONFIG_ARCH_TCC83X
        t_IEN = HwIEN;
    t_IEN_B = HwIEN_B;

    if(co->index == 0){ HwIEN &= ~(1<<7); 
    }else if(co->index == 1){ HwIEN &= ~(1<<29); 
    }else if(co->index == 2){ HwIEN_B &= ~(1<<8); 
    }else if(co->index == 3){ HwIEN_B &= ~(1<<9); }
#endif

    wr_regl(port, OFFSET_IER, t_ier & ~ETXI);
    uart_console_write(cons_uart, s, count, tcc_serial_console_putchar);
    wr_regl(port, OFFSET_IER, b_ier);

#ifdef CONFIG_ARCH_TCC83X
    HwIEN = t_IEN; 
    HwIEN_B = t_IEN_B; 
#endif

}

static void __init
tcc_serial_get_options(struct uart_port *port, int *baud,
			   int *parity, int *bits)
{
}


/* tcc_serial_init_ports
 * initialise the serial ports from the machine provided initialisation data.
 */
int tcc_serial_init_ports(struct tcc_uart_port *ourport,
					struct tcc_uart_info *info,
					struct platform_device *pdev)
{
	struct uart_port *port = &ourport->port;
    struct resource *res;
    int ret;

	dbg("tcc_serial_init_ports: initialising ports...\n");
	
	port->iotype	= UPIO_MEM;
	port->flags     = UPF_BOOT_AUTOCONF;
	port->ops	    = &tcc_serial_ops;
	port->fifosize  = FIFOSIZE;
	port->line	    = pdev->id;
	port->dev	    = &pdev->dev;
	port->type      = PORT_TCC;

    res = platform_get_resource(pdev, IORESOURCE_MEM, 0);
	if (res == NULL) {
		printk(KERN_ERR "failed to find memory resource for uart\n");
		return -EINVAL;
	}

	//port->mapbase	= pdev->resource[0].start;
	port->mapbase	= res->start;
	//port->membase	= ioremap_nocache(port->mapbase, MAP_SIZE);
	port->membase	= (unsigned char __iomem *)port->mapbase;
    port->irq       = platform_get_irq(pdev, 0);

    ret = (int)port->line;

	return ret;
}

static int __init
tcc_console_setup(struct console *co, char *options)
{
	struct uart_port *port;
	int baud = BAUDRATE;
	int bits = 8;
	int parity = 'n';
	int flow = 'n';

	dbg("tcc_serial_console_setup: co=%p (%d), %s\n", co, co->index, options);

	/* is this a valid port */

	if (co->index == -1 || co->index >= NR_PORTS)
		co->index = CONSOLE_PORT;

	port = &tcc_serial_ports[co->index].port;

	/* is the port configured? */

	if (port->mapbase == 0x0) {
        port->mapbase = (resource_size_t)&(CONSOLE_BASE);
        port->membase = (unsigned char __iomem *)port->mapbase;
		co->index = CONSOLE_PORT;
		port = &tcc_serial_ports[co->index].port;
	}
    
	cons_uart = port;

	if (options) {
        dbg("uart_parse_options\n");
		uart_parse_options(options, &baud, &parity, &bits, &flow);
    } else {
        dbg("tcc_serial_get_options\n");
		tcc_serial_get_options(port, &baud, &parity, &bits);
    }
	
	dbg("tcc_serial_console_setup: port=%p (%d)\n", port, co->index);
	return uart_set_options(port, co, baud, parity, bits, flow);
}

/* tcc_console_init
 *
 * initialise the console from one of the uart drivers
*/

static struct console tcc_serial_console =
{
	.name		= TCC_SERIAL_NAME,
	.device		= uart_console_device,
	.flags		= CON_PRINTBUFFER | CON_ENABLED,
	.index		= -1,
	.write		= tcc_console_write,
	.setup		= tcc_console_setup,
};

static int tcc_console_init(void)
{
	struct tcc_uart_info *info;
	
	dbg("%s\n", __func__);
	
	info = &tcc_uart_inf;
	
	tcc_serial_console.data = &tcc_uart_drv;
	register_console(&tcc_serial_console);
	
	return 0;
}

console_initcall(tcc_console_init);

#endif /* CONFIG_SERIAL_TCC_CONSOLE */

MODULE_LICENSE("GPL");
MODULE_AUTHOR("linux <linux@telechips.com>");
MODULE_DESCRIPTION("Telechips TCC Serial port driver");

