/*
 * linux/drivers/video/tcc_component.c
 *
 * Author:  <linux@telechips.com>
 * Created: March 18, 2010
 * Description: TCC Component TV-Out Driver
 *
 * Copyright (C) 20010-2011 Telechips 
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, see the file COPYING, or write
 * to the Free Software Foundation, Inc.,
 * 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

#include <linux/module.h>
#include <linux/kernel.h>
#include <linux/errno.h>
#include <linux/string.h>
#include <linux/mm.h>
#include <linux/slab.h>
#include <linux/delay.h>
#include <linux/fb.h>
#include <linux/init.h>
#include <linux/dma-mapping.h>
#include <linux/interrupt.h>
#include <linux/workqueue.h>
#include <linux/wait.h>
#include <linux/platform_device.h>
#include <linux/clk.h>
#include <linux/cpufreq.h>
#include <asm/io.h>
#include <asm/uaccess.h>
#include <asm/div64.h>
#include <asm/mach/map.h>
#ifdef CONFIG_PM
#include <linux/pm.h>
#endif
#include <mach/bsp.h>
#include <mach/tca_ckc.h>
#ifdef CONFIG_HAS_EARLYSUSPEND
#include <linux/earlysuspend.h>
#endif
#include "tcc_component.h"
#include <mach/tcc_component_ioctl.h>
#include "tcc92xxfb.h"
#include <mach/tcc92xxfb_ioctrl.h>
#include <mach/tca_fb_output.h>
#include "tcc_component_cs4954.h"
#include "tcc_component_ths8200.h"


/*****************************************************************************

 VARIABLES

******************************************************************************/

extern char fb_power_state;

/* Debugging stuff */
static int debug = 0;
#define dprintk(msg...)	if (debug) { printk( "tcc_component: " msg); }

#define TCC_LCDC1_USE

static int				Component_LCDC_Num = -1;
static int				Component_Mode = -1;
static PLCDC			pComponent_HwLCDC = NULL;
static PLCDC_CHANNEL	pComponent_HwLCDC_CH = NULL;

#define DEVICE_NAME		"component"
#define MAJOR_ID		206
#define MINOR_ID		1

static struct clk *component_lcdc0_clk;
static struct clk *component_lcdc1_clk;

#if defined(CONFIG_CPU_FREQ_TCC92X) || defined (CONFIG_CPU_FREQ_TCC93XX)
extern struct tcc_freq_table_t gtTvClockLimitTable;
#endif

/*****************************************************************************

 FUNCTIONS

******************************************************************************/

extern char TCC_FB_LCDC_ChangeReg(char NumLcdc, char OnOff);
extern void tca_lcdc_port_setting(char Nlcdc);

/*****************************************************************************
 Function Name : tcc_component_set_lcdc()
******************************************************************************/
int tcc_component_set_lcdc(int lcdc_num)
{
	dprintk("%s - new_lcdc:%d, before_lcdc:%d\n", __func__, lcdc_num, Component_LCDC_Num);

	if(lcdc_num == Component_LCDC_Num)
		return FALSE;
	
	if(lcdc_num)
	{
		Component_LCDC_Num = 1;
		pComponent_HwLCDC = (volatile PLCDC)tcc_p2v(HwLCDC1_BASE);
		pComponent_HwLCDC_CH = (volatile PLCDC_CHANNEL)tcc_p2v(pComponent_HwLCDC->LI2C);
	}
	else
	{
		Component_LCDC_Num = 0;
		pComponent_HwLCDC = (volatile PLCDC)tcc_p2v(HwLCDC0_BASE);
		pComponent_HwLCDC_CH = (volatile PLCDC_CHANNEL)tcc_p2v(pComponent_HwLCDC->LI0C);
	}
	
	return TRUE;
}

/*****************************************************************************
 Function Name : tcc_component_get_spec()
******************************************************************************/
void tcc_component_get_spec(COMPONENT_MODE_TYPE mode, COMPONENT_SPEC_TYPE *spec)
{
	switch(mode)
	{
		case COMPONENT_MODE_NTSC_M:
		case COMPONENT_MODE_NTSC_M_J:
		case COMPONENT_MODE_PAL_M:
		case COMPONENT_MODE_NTSC_443:
		case COMPONENT_MODE_PSEUDO_PAL:
			spec->component_clk = 270000;
			spec->component_bus_width = 8;
			spec->component_lcd_width = 720;
			spec->component_lcd_height = 480;
			spec->component_LPW = 128 - 1; 					// line pulse width
			spec->component_LPC = 720 * 2 - 1; 				// line pulse count (active horizontal pixel - 1)
			spec->component_LSWC = 116 - 1;					// line start wait clock (the number of dummy pixel clock - 1)
			spec->component_LEWC = 32 - 1;					// line end wait clock (the number of dummy pixel clock - 1)

			spec->component_VDB = 0; 						// Back porch Vsync delay
			spec->component_VDF = 0; 						// front porch of Vsync delay

			spec->component_FPW1 = 6 - 1;					// TFT/TV : Frame pulse width is the pulse width of frmae clock
			spec->component_FLC1 = 480 - 1;					// frmae line count is the number of lines in each frmae on the screen
			spec->component_FSWC1 = 30 - 1;					// frmae start wait cycle is the number of lines to insert at the end each frame
			spec->component_FEWC1 = 9 - 1;					// frame start wait cycle is the number of lines to insert at the begining each frame
			spec->component_FPW2 = 6 - 1;					// TFT/TV : Frame pulse width is the pulse width of frmae clock
			spec->component_FLC2 = 480 - 1;					// frmae line count is the number of lines in each frmae on the screen
			spec->component_FSWC2 = 31 - 1;					// frmae start wait cycle is the number of lines to insert at the end each frame
			spec->component_FEWC2 = 8 - 1; 					// frame start wait cycle is the number of lines to insert at the begining each frame
			break;
				
		case COMPONENT_MODE_NTSC_N:
		case COMPONENT_MODE_NTSC_N_J:
		case COMPONENT_MODE_PAL_N:
		case COMPONENT_MODE_PAL_B:
		case COMPONENT_MODE_PAL_G:
		case COMPONENT_MODE_PAL_H:
		case COMPONENT_MODE_PAL_I:
		case COMPONENT_MODE_PSEUDO_NTSC:
			spec->component_clk = 270000;
			spec->component_bus_width = 8;
			spec->component_lcd_width = 720;
			spec->component_lcd_height = 576;
			spec->component_LPW = 128 - 1; 					// line pulse width
			spec->component_LPC = 720 * 2 - 1; 				// line pulse count (active horizontal pixel - 1)
			spec->component_LSWC = 136 - 1;					// line start wait clock (the number of dummy pixel clock - 1)
			spec->component_LEWC = 24 - 1;					// line end wait clock (the number of dummy pixel clock - 1)

			spec->component_VDB = 0; 						// Back porch Vsync delay
			spec->component_VDF = 0; 						// front porch of Vsync delay

			spec->component_FPW1 = 5 - 1;					// TFT/TV : Frame pulse width is the pulse width of frmae clock
			spec->component_FLC1 = 576 - 1;					// frmae line count is the number of lines in each frmae on the screen
			spec->component_FSWC1 = 39 - 1;					// frmae start wait cycle is the number of lines to insert at the end each frame
			spec->component_FEWC1 = 5 - 1;					// frame start wait cycle is the number of lines to insert at the begining each frame
			spec->component_FPW2 = 5 - 1;					// TFT/TV : Frame pulse width is the pulse width of frmae clock
			spec->component_FLC2 = 576 - 1;					// frmae line count is the number of lines in each frmae on the screen
			spec->component_FSWC2 = 40 - 1;					// frmae start wait cycle is the number of lines to insert at the end each frame
			spec->component_FEWC2 = 4 - 1; 					// frame start wait cycle is the number of lines to insert at the begining each frame
			break;

		case COMPONENT_MODE_720P:
			spec->component_clk = 742500;
			spec->component_bus_width = 24;
			spec->component_lcd_width = 1280;
			spec->component_lcd_height = 720;
			#if 1
			spec->component_LPW = 40 - 1; 					// line pulse width
			spec->component_LPC = 1280 - 1; 				// line pulse count (active horizontal pixel - 1)
			spec->component_LSWC = 329 - 1;					// line start wait clock (the number of dummy pixel clock - 1)
			spec->component_LEWC = 1 - 1;					// line end wait clock (the number of dummy pixel clock - 1)
			#else
			spec->component_LPW = 40 - 1; 					// line pulse width
			spec->component_LPC = 1280 - 1; 				// line pulse count (active horizontal pixel - 1)
			spec->component_LSWC = 260 - 1;					// line start wait clock (the number of dummy pixel clock - 1)
			spec->component_LEWC = 70 - 1;					// line end wait clock (the number of dummy pixel clock - 1)
			#endif

			spec->component_VDB = 0; 						// Back porch Vsync delay
			spec->component_VDF = 0; 						// front porch of Vsync delay
				
			spec->component_FPW1 = 6 - 1;					// TFT/TV : Frame pulse width is the pulse width of frmae clock
			spec->component_FLC1 = 720 - 1;					// frmae line count is the number of lines in each frmae on the screen
			spec->component_FSWC1 = 20 - 1;					// frmae start wait cycle is the number of lines to insert at the end each frame
			spec->component_FEWC1 = 4 - 1;					// frame start wait cycle is the number of lines to insert at the begining each frame
			spec->component_FPW2 = 6 - 1;					// TFT/TV : Frame pulse width is the pulse width of frmae clock
			spec->component_FLC2 = 720 - 1;					// frmae line count is the number of lines in each frmae on the screen
			spec->component_FSWC2 = 20 - 1;					// frmae start wait cycle is the number of lines to insert at the end each frame
			spec->component_FEWC2 = 4 - 1; 					// frame start wait cycle is the number of lines to insert at the begining each frame
			break;

		case COMPONENT_MODE_1080I:
			spec->component_clk = 742500;
			spec->component_bus_width = 24;
			spec->component_lcd_width = 1920;
			spec->component_lcd_height = 1080;
			spec->component_LPW = 44 - 1; 					// line pulse width
			spec->component_LPC = 1920 - 1; 				// line pulse count (active horizontal pixel - 1)
			spec->component_LSWC = 148 - 1;					// line start wait clock (the number of dummy pixel clock - 1)
			spec->component_LEWC = 88 - 1;					// line end wait clock (the number of dummy pixel clock - 1)

			spec->component_VDB = 0; 						// Back porch Vsync delay
			spec->component_VDF = 0; 						// front porch of Vsync delay
				
			spec->component_FPW1 = 10 - 1;					// TFT/TV : Frame pulse width is the pulse width of frmae clock
			spec->component_FLC1 = 540 - 1;					// frmae line count is the number of lines in each frmae on the screen
			spec->component_FSWC1 = 10 - 1;					// frmae start wait cycle is the number of lines to insert at the end each frame
			spec->component_FEWC1 = 3 - 1;					// frame start wait cycle is the number of lines to insert at the begining each frame
			spec->component_FPW2 = 10 - 1;					// TFT/TV : Frame pulse width is the pulse width of frmae clock
			spec->component_FLC2 = 540 - 1;					// frmae line count is the number of lines in each frmae on the screen
			spec->component_FSWC2 = 10 - 1;					// frmae start wait cycle is the number of lines to insert at the end each frame
			spec->component_FEWC2 = 2 - 1; 					// frame start wait cycle is the number of lines to insert at the begining each frame
			break;
	}
}

/*****************************************************************************
 Function Name : tcc_component_set_lcd2tv()
******************************************************************************/
void tcc_component_set_lcd2tv(COMPONENT_MODE_TYPE mode)
{
	COMPONENT_SPEC_TYPE spec;
	unsigned int lcd_ctrl = 0;
	unsigned int lcd_peri = 0;
	PDDICONFIG pDDICfg = (PDDICONFIG)tcc_p2v(HwDDI_CONFIG_BASE);

	dprintk("%s\n", __func__);
	
	if(Component_LCDC_Num)
	{
		BITSET(pDDICfg->SWRESET, Hw3);	// LCDC1
		BITCLR(pDDICfg->SWRESET, Hw3);	// LCDC1
		lcd_peri = PERI_LCD1;
	}
	else
	{
		BITSET(pDDICfg->SWRESET, Hw2);	// LCDC0
		BITCLR(pDDICfg->SWRESET, Hw2);	// LCDC0
		lcd_peri = PERI_LCD0;
	}
		
	tcc_component_get_spec(mode, &spec);
	
	#if defined(CONFIG_CPU_FREQ_TCC92X) || defined (CONFIG_CPU_FREQ_TCC93XX)
		tcc_cpufreq_set_limit_table(&gtTvClockLimitTable, TCC_FREQ_LIMIT_TV, 1);
	#endif

	#if 1//defined(CONFIG_DRAM_DDR2)
		tca_ckc_setperi(lcd_peri,ENABLE,spec.component_clk,PCDIRECTPLL2);
	#else
		tca_ckc_setperi(lcd_peri,ENABLE,spec.component_clk,PCDIRECTPLL0);
	#endif

	#if defined(CONFIG_ARCH_TCC92X) && defined(CONFIG_TCC_OUTPUT_STARTER)
	{
		int wait_count = 0;

		BITCLR(pComponent_HwLCDC->LCTRL, HwLCTRL_LEN);
		while(ISSET(pComponent_HwLCDC->LSTATUS, HwLSTATUS_BUSY))
		{
			msleep(1);
			wait_count++;
			if(wait_count > 100)
			{
				break;
			}
		}
	}
	#endif

	//dprintk("LCDC Clock: %d KHz\n", tca_ckc_getperi(PERI_LCD1)/10);

	BITCSET(pComponent_HwLCDC->LCLKDIV, 0x00FF0000, 0<<16);
	BITCSET(pComponent_HwLCDC->LCLKDIV, 0x000000FF, 0);

	BITCSET(pComponent_HwLCDC->LHTIME1, 0x0003FFFF, spec.component_LPC);
	BITCSET(pComponent_HwLCDC->LHTIME1, 0x01FF0000, (spec.component_LPW << 16));
	BITCSET(pComponent_HwLCDC->LHTIME2, 0x000001FF, spec.component_LEWC);
	BITCSET(pComponent_HwLCDC->LHTIME2, 0x01FF0000, (spec.component_LSWC << 16));
	
	BITCSET(pComponent_HwLCDC->LVTIME1, 0x00003FFF, spec.component_FLC1);
	BITCSET(pComponent_HwLCDC->LVTIME1, 0x003F0000, (spec.component_FPW1 << 16));
	BITCSET(pComponent_HwLCDC->LVTIME1, 0x03C00000, (spec.component_VDF << 22));
	BITCSET(pComponent_HwLCDC->LVTIME1, 0xF8000000, (spec.component_VDB << 27));
	
	BITCSET(pComponent_HwLCDC->LVTIME2, 0x01FF0000, (spec.component_FSWC1 << 16));
	BITCSET(pComponent_HwLCDC->LVTIME2, 0x000001FF, spec.component_FEWC1);
	
	BITCSET(pComponent_HwLCDC->LVTIME3, 0x00003FFF, spec.component_FLC2);
	BITCSET(pComponent_HwLCDC->LVTIME3, 0x003F0000, (spec.component_FPW2 << 16));
	
	BITCSET(pComponent_HwLCDC->LVTIME4, 0x01FF0000, (spec.component_FSWC2 << 16));
	BITCSET(pComponent_HwLCDC->LVTIME4, 0x000001FF, spec.component_FEWC2);
	
	BITCSET(pComponent_HwLCDC->LDS, 0x0000FFFF, spec.component_lcd_width);
	BITCSET(pComponent_HwLCDC->LDS, 0xFFFF0000, (spec.component_lcd_height << 16));

	switch(mode)
	{
		case COMPONENT_MODE_NTSC_M:
		case COMPONENT_MODE_NTSC_M_J:
		case COMPONENT_MODE_PAL_M:
		case COMPONENT_MODE_NTSC_443:
		case COMPONENT_MODE_PSEUDO_PAL:
		case COMPONENT_MODE_NTSC_N:
		case COMPONENT_MODE_NTSC_N_J:
		case COMPONENT_MODE_PAL_N:
		case COMPONENT_MODE_PAL_B:
		case COMPONENT_MODE_PAL_G:
		case COMPONENT_MODE_PAL_H:
		case COMPONENT_MODE_PAL_I:
		case COMPONENT_MODE_PSEUDO_NTSC:
			BITCSET(lcd_ctrl, Hw29|Hw28, Hw29|Hw28);				// R2YMD - RGB to YCbCr Conversion Option
			BITCSET(lcd_ctrl, Hw23, Hw23);							// CKG - Clock Gating Enable for Timing Generator
			BITCSET(lcd_ctrl, Hw15, Hw15);							// ID - Inverted Data Enable
			BITCSET(lcd_ctrl, Hw14, Hw14);							// IV - Inverted Vertical Sync
			BITCSET(lcd_ctrl, Hw13, Hw13);							// IH - Inverted Horizontal Sync
			BITCSET(lcd_ctrl, Hw12, Hw12);							// IP - Inverted Pixel Clock
			BITCSET(lcd_ctrl, Hw10, Hw10);							// R2Y - RGB to YCbCr Converter Enable for OUPUT
			BITCSET(lcd_ctrl, Hw7, Hw7);							// TV - TV mode
			BITCSET(lcd_ctrl, Hw3|Hw2|Hw1, Hw3|Hw1);				// OVP
			BITCSET(lcd_ctrl, Hw0, Hw0);							// LEN - LCD Controller Enable
			#ifdef TCC_COMPONENT_CCIR656
			BITCSET(lcd_ctrl, Hw24, Hw24);							// 656 - CCIR 656 Mode
			BITCSET(lcd_ctrl, Hw19|Hw18|Hw17|Hw16, Hw19);			// PXDW - Pixel Data Width
			BITCSET(lcd_ctrl, Hw9, Hw9);							// DP - Double Pixel Data
			#else
			BITCSET(lcd_ctrl, Hw19|Hw18|Hw17|Hw16, Hw18|Hw17);		// PXDW - Pixel Data Width
			#endif
			break;

		case COMPONENT_MODE_720P:
			BITCSET(lcd_ctrl, Hw29|Hw28, Hw29|Hw28);				// R2YMD - RGB to YCbCr Conversion Option
			BITCSET(lcd_ctrl, Hw23, Hw23);							// CKG - Clock Gating Enable for Timing Generator
			#if defined(CONFIG_MACH_TCC9300ST)
			BITCSET(lcd_ctrl, Hw15, Hw15);							// ID - Inverted Data Enable
			BITCSET(lcd_ctrl, Hw14, Hw14);							// IV - Inverted Vertical Sync
//			BITCSET(lcd_ctrl, Hw13, Hw13);							// IH - Inverted Horizontal Sync
			BITCSET(lcd_ctrl, Hw12, Hw12);							// IP - Inverted Pixel Clock
			#else
//			BITCSET(lcd_ctrl, Hw15, Hw15);							// ID - Inverted Data Enable
			BITCSET(lcd_ctrl, Hw14, Hw14);							// IV - Inverted Vertical Sync
			BITCSET(lcd_ctrl, Hw13, Hw13);							// IH - Inverted Horizontal Sync
//			BITCSET(lcd_ctrl, Hw12, Hw12);							// IP - Inverted Pixel Clock
			#endif
			BITCSET(lcd_ctrl, Hw8, Hw8);							// NI - Non-interlace Mode
			BITCSET(lcd_ctrl, Hw3|Hw2|Hw1, Hw3|Hw1);				// OVP
			BITCSET(lcd_ctrl, Hw0, Hw0);							// LEN - LCD Controller Enable
			#ifdef TCC_COMPONENT_16BPP_YCBCR
			BITCSET(lcd_ctrl, Hw10, Hw10);							// R2Y - RGB to YCbCr Converter Enable for OUPUT
			BITCSET(lcd_ctrl, Hw19|Hw18|Hw17|Hw16, Hw19);			// PXDW - Pixel Data Width
			#else
			BITCSET(lcd_ctrl, Hw19|Hw18|Hw17|Hw16, Hw19|Hw18);		// PXDW - Pixel Data Width
			#endif
			break;

		case COMPONENT_MODE_1080I:
			BITCSET(lcd_ctrl, Hw29|Hw28, Hw29|Hw28);				// R2YMD - RGB to YCbCr Conversion Option
			BITCSET(lcd_ctrl, Hw23, Hw23);							// CKG - Clock Gating Enable for Timing Generator
//			BITCSET(lcd_ctrl, Hw15, Hw15);							// ID - Inverted Data Enable
			BITCSET(lcd_ctrl, Hw14, Hw14);							// IV - Inverted Vertical Sync
			BITCSET(lcd_ctrl, Hw13, Hw13);							// IH - Inverted Horizontal Sync
//			BITCSET(lcd_ctrl, Hw12, Hw12);							// IP - Inverted Pixel Clock
			BITCSET(lcd_ctrl, Hw3|Hw2|Hw1, Hw3|Hw1);				// OVP
			BITCSET(lcd_ctrl, Hw0, Hw0);							// LEN - LCD Controller Enable
			#ifdef TCC_COMPONENT_16BPP_YCBCR
			BITCSET(lcd_ctrl, Hw10, Hw10);							// R2Y - RGB to YCbCr Converter Enable for OUPUT
			BITCSET(lcd_ctrl, Hw19|Hw18|Hw17|Hw16, Hw19);			// PXDW - Pixel Data Width
			#else
			BITCSET(lcd_ctrl, Hw19|Hw18|Hw17|Hw16, Hw19|Hw18);		// PXDW - Pixel Data Width
			#endif
			break;

		default:
			break;
	}

	BITCSET(pComponent_HwLCDC->LCTRL, 0xFFFFFFFF, lcd_ctrl);
}

/*****************************************************************************
 Function Name : tcc_component_get_lcdsize()
******************************************************************************/
void tcc_component_get_lcdsize(unsigned int *width, unsigned int *height)
{
	unsigned int lcdsize;
		
	lcdsize = pComponent_HwLCDC->LDS;

	*width = lcdsize & 0x0000FFFF;
	*height = lcdsize >>16;
}

/*****************************************************************************
 Function Name : tcc_component_change_formattype()
******************************************************************************/
COMPONENT_LCDC_IMG_FMT_TYPE tcc_component_change_formattype(TCC_COMPONENT_FORMAT_TYPE g2d_format)
{
	COMPONENT_LCDC_IMG_FMT_TYPE LCDC_fmt = COMPONENT_LCDC_IMG_FMT_MAX;

	switch(g2d_format)
	{
		case GE_YUV444_sp:
			break;

		case GE_YUV440_sp:
			break;
				
		case GE_YUV422_sp: 
			LCDC_fmt = COMPONENT_LCDC_IMG_FMT_YUV422SP;
			break;
				
		case GE_YUV420_sp:
			LCDC_fmt = COMPONENT_LCDC_IMG_FMT_YUV420SP;
			break;

		case GE_YUV411_sp:
		case GE_YUV410_sp:
			break;

		case GE_YUV422_in:
			LCDC_fmt = COMPONENT_LCDC_IMG_FMT_YUV422ITL0;
			break;

		case GE_YUV420_in:
			LCDC_fmt = COMPONENT_LCDC_IMG_FMT_YUV420ITL0;
			break;

		case GE_YUV444_sq:
			break;
		case GE_YUV422_sq:
			LCDC_fmt = COMPONENT_LCDC_IMG_FMT_YUV422SQ;
			break;

		case GE_RGB332:
			LCDC_fmt = COMPONENT_LCDC_IMG_FMT_RGB332;
			break;
				
		case GE_RGB444:
		case GE_ARGB4444:
			LCDC_fmt = COMPONENT_LCDC_IMG_FMT_RGB444;
			break;

		case GE_RGB454:
		case GE_ARGB3454:
			break;

		case GE_RGB555:
		case GE_ARGB1555:
			LCDC_fmt = COMPONENT_LCDC_IMG_FMT_RGB555;
			break;
		case GE_RGB565:
			LCDC_fmt = COMPONENT_LCDC_IMG_FMT_RGB565;
			break;
			
		case GE_RGB666:
		case GE_ARGB4666:
		case GE_ARGB6666:
			LCDC_fmt = COMPONENT_LCDC_IMG_FMT_RGB666;
			break;

		case GE_RGB888:
		case GE_ARGB4888:
		case GE_ARGB8888:
			LCDC_fmt = COMPONENT_LCDC_IMG_FMT_RGB888;
			break;
	}

	return LCDC_fmt;
}

/*****************************************************************************
 Function Name: tcc_component_set_imagebase()
 Parameters
		: nType		: IMG_CH0, IMG_CH1, IMG_CH2
		: nBase0	: Base Address for Y Channel or RGB Channel
					: Valid for 3 Channel
		: nBase1	: Base Address for Cb(U)
					: Valid for IMG_CH0
		: nBase2	: Base Address for Cb(U) Channel
					: Valid for IMG_CH0
******************************************************************************/
void tcc_component_set_imagebase(unsigned int type, unsigned int base0, unsigned int base1, unsigned int base2)
{
	switch (type) 
	{
		case	IMG_CH0 :
			BITCSET(pComponent_HwLCDC->LI0BA0, 0xFFFFFFFF, base0);
			BITCSET(pComponent_HwLCDC->LI0BA1, 0xFFFFFFFF, base1);
			BITCSET(pComponent_HwLCDC->LI0BA2, 0xFFFFFFFF, base2);			
			break;

		case	IMG_CH1 :
			BITCSET(pComponent_HwLCDC->LI1BA0, 0xFFFFFFFF, base0);
			break;

		case	IMG_CH2 :
			BITCSET(pComponent_HwLCDC->LI2BA0, 0xFFFFFFFF, base0);
			break;

		default	:
			break;
	}
}

/*****************************************************************************
 Function Name : tcc_component_get_offset()
******************************************************************************/
void tcc_component_get_offset(COMPONENT_LCDC_IMG_FMT_TYPE fmt, unsigned int width, unsigned int *offset0, unsigned int *offset1)
{
	switch(fmt)
	{
		case COMPONENT_LCDC_IMG_FMT_1BPP:
		case COMPONENT_LCDC_IMG_FMT_2BPP:
		case COMPONENT_LCDC_IMG_FMT_4BPP:
		case COMPONENT_LCDC_IMG_FMT_8BPP:
		case COMPONENT_LCDC_IMG_FMT_RGB332:
			*offset0 = width;
			*offset1 = 0;
			break;
			
		case COMPONENT_LCDC_IMG_FMT_RGB444:
		case COMPONENT_LCDC_IMG_FMT_RGB565:
		case COMPONENT_LCDC_IMG_FMT_RGB555:
			*offset0 = width * 2;
			*offset1 = 0;
			break;

		case COMPONENT_LCDC_IMG_FMT_RGB888:
		case COMPONENT_LCDC_IMG_FMT_RGB666:
			*offset0 = width * 4;
			*offset1 = 0;
			break;

		case COMPONENT_LCDC_IMG_FMT_YUV420SP:
		case COMPONENT_LCDC_IMG_FMT_YUV422SP:
			*offset0 = width;
			*offset1 = width/2;
			break;
		case COMPONENT_LCDC_IMG_FMT_YUV422SQ:
			*offset0 = width * 2;
			*offset1 = 0;
			break;

		case COMPONENT_LCDC_IMG_FMT_YUV420ITL0:
		case COMPONENT_LCDC_IMG_FMT_YUV420ITL1:
		case COMPONENT_LCDC_IMG_FMT_YUV422ITL0:
		case COMPONENT_LCDC_IMG_FMT_YUV422ITL1:
			*offset0 = width;
			*offset1 = width;
			break;
	}
}

/*****************************************************************************
 Function Name: tcc_component_set_imageinfo()
 Parameters
		: nType		: IMG_CH0, IMG_CH1, IMG_CH2
		: nHPos		: Horizontal Position
		: nVPos		: Vertical Position
		: nOffset0	: Horizontal Pixel Offset for Y Channel or RGB Channel
		: nOffset1	: Horizontal Pixel Offset for UV Channel
					: Valid for IMG_CH0
		: nHScale	: Horizontal Scale Factor
		: nVScale	: Vertical Scale Factor
		: nFormat	: Endian / YUV / BPP Setting
					: [3:0]	: BPP Information
					: [6:4]	: YUV Format Information
					: [7]		: Endian Information
******************************************************************************/
unsigned int tcc_component_set_imageinfo (unsigned int flag, unsigned int type, unsigned int width, unsigned int height, unsigned int hpos, unsigned int vpos, unsigned int hoffset0, unsigned int hoffset1, unsigned int hscale, unsigned int vscale)
{
	unsigned int uiCH0Position = 0;
	
	switch (type) 
	{
		case IMG_CH0 :
			if(flag & SET_IMAGE_SIZE)
			{
				BITCSET(pComponent_HwLCDC->LI0S, 0xFFFF0000, (height << 16));
				BITCSET(pComponent_HwLCDC->LI0S, 0x0000FFFF, width);
			}

			if(flag & READ_IMAGE_POSITION)
			{
				uiCH0Position = (unsigned int)pComponent_HwLCDC->LI0P;
			}

			if(flag & SET_IMAGE_POSITION)
			{
				BITCSET(pComponent_HwLCDC->LI0P, 0x0000FFFF, hpos);
				BITCSET(pComponent_HwLCDC->LI0P, 0xFFFF0000, (vpos << 16));
			}	

			if(flag & SET_IMAGE_OFFSET)
			{
				BITCSET(pComponent_HwLCDC->LI0O, 0x0000FFFF, hoffset0);
				BITCSET(pComponent_HwLCDC->LI0O, 0xFFFF0000, (hoffset1 << 16));
			}	

			if(flag & SET_IMAGE_SCALE)
			{
				BITCSET(pComponent_HwLCDC->LI0SR, HwLISCALE_X, hscale);
				BITCSET(pComponent_HwLCDC->LI0SR, HwLISCALE_Y, (vscale << 4));
			}
			break;
			
		case IMG_CH1 :
			if(flag & SET_IMAGE_SIZE)
			{
				BITCSET(pComponent_HwLCDC->LI1S, 0xFFFF0000, (height << 16));
				BITCSET(pComponent_HwLCDC->LI1S, 0x0000FFFF, width);
			}	

			if(flag & SET_IMAGE_POSITION)
			{
				BITCSET(pComponent_HwLCDC->LI1P, 0x0000FFFF, hpos);
				BITCSET(pComponent_HwLCDC->LI1P, 0xFFFF0000, (vpos << 16));
			}	

			if(flag & SET_IMAGE_OFFSET)
			{
				BITCSET(pComponent_HwLCDC->LI1O, 0x0000FFFF, hoffset0);
			}	

			if(flag & SET_IMAGE_SCALE)
			{
				BITCSET(pComponent_HwLCDC->LI1SR, 0x0000000F, hscale);
				BITCSET(pComponent_HwLCDC->LI1SR, 0x000000F0, (vscale << 4));
			}	
			break;
			
		case IMG_CH2 :
			if(flag & SET_IMAGE_SIZE)
			{
				BITCSET(pComponent_HwLCDC->LI2S, 0xFFFF0000, (height << 16));
				BITCSET(pComponent_HwLCDC->LI2S, 0x0000FFFF, width);
			}

			if(flag & SET_IMAGE_POSITION)
			{
				BITCSET(pComponent_HwLCDC->LI2P, 0x0000FFFF, hpos);
				BITCSET(pComponent_HwLCDC->LI2P, 0xFFFF0000, (vpos << 16));
			}

			if(flag & SET_IMAGE_OFFSET)
			{
				BITCSET(pComponent_HwLCDC->LI2O, 0x0000FFFF, hoffset0);
			}

			if(flag & SET_IMAGE_SCALE)
			{
				BITCSET(pComponent_HwLCDC->LI2SR, 0x0000000F, hscale);
				BITCSET(pComponent_HwLCDC->LI2SR, 0x000000F0, (vscale << 4));
			}
			break;

		default	:
			break;
	}
	return uiCH0Position;
}

/*****************************************************************************
 Function Name : tcc_component_set_imagectrl()
******************************************************************************/
void tcc_component_set_imagectrl(unsigned int flag, unsigned int type, COMPONENT_LCDC_IMG_CTRL_TYPE *imgctrl)
{
	unsigned int IMG_Ctrl_reg;
	
	switch (type) 
	{
		case IMG_CH0:
			IMG_Ctrl_reg = pComponent_HwLCDC->LI0C;
			break;

		case IMG_CH1:
			IMG_Ctrl_reg = pComponent_HwLCDC->LI1C;
			break;

		case IMG_CH2:
			IMG_Ctrl_reg = pComponent_HwLCDC->LI2C;
			break;

		default:
			return;
	}

	if(flag & SET_IMAGE_INTL)
	{
		BITCSET(IMG_Ctrl_reg, HwLIC_INTL, (imgctrl->INTL<<31));
	}
	if(flag & SET_IMAGE_AEN)
	{
		BITCSET(IMG_Ctrl_reg, HwLIC_AEN, (imgctrl->AEN<<30));
	}
	if(flag & SET_IMAGE_CEN)
	{
		BITCSET(IMG_Ctrl_reg, HwLIC_CEN, (imgctrl->CEN<<29));
	}
	if(flag & SET_IMAGE_IEN)
	{
		BITCSET(IMG_Ctrl_reg, HwLIC_IEN, (imgctrl->IEN<<28));
	}
	if(flag & SET_IMAGE_SRC)
	{
		BITCSET(IMG_Ctrl_reg, HwLIC_SRC, (imgctrl->SRC<<27));
	}			
	if(flag & SET_IMAGE_AOPT)
	{
		BITCSET(IMG_Ctrl_reg, HwLIC_AOPT, (imgctrl->AOPT<<25));
	}
	if(flag & SET_IMAGE_ASEL)
	{
		BITCSET(IMG_Ctrl_reg, HwLIC_ASEL, (imgctrl->ASEL<<24));
	}
	if(flag & SET_IMAGE_UPD)
	{
		#if defined(CONFIG_ARCH_TCC93XX)
		BITCSET(IMG_Ctrl_reg, HwLIC_UPD, (imgctrl->UPD<<16));
		#endif
	}
	if(flag & SET_IMAGE_PD)
	{
		BITCSET(IMG_Ctrl_reg, HwLIC_PD, (imgctrl->PD<<15));
	}
	if(flag & SET_IMAGE_Y2RMD)
	{
		BITCSET(IMG_Ctrl_reg, HwLIC_Y2RMD, (imgctrl->Y2RMD<<9));
	}
	if(flag & SET_IMAGE_Y2R)
	{
		BITCSET(IMG_Ctrl_reg, HwLIC_Y2R, (imgctrl->Y2R<<8));
	}
	if(flag & SET_IMAGE_BR)
	{
		BITCSET(IMG_Ctrl_reg, HwLIC_BR, (imgctrl->BR<<7));
	}
	if(flag & SET_IMAGE_FMT)
	{
		BITCSET(IMG_Ctrl_reg, HwLIC_FMT, (imgctrl->FMT));
	}

	switch (type) 
	{
		case IMG_CH0:
			pComponent_HwLCDC->LI0C = IMG_Ctrl_reg;
			break;

		case IMG_CH1:
			pComponent_HwLCDC->LI1C = IMG_Ctrl_reg;
			break;

		case IMG_CH2:
			pComponent_HwLCDC->LI2C = IMG_Ctrl_reg;
			break;

		default:
			return;
	}
}

/*****************************************************************************
 Function Name : tcc_component_update()
******************************************************************************/
void tcc_component_update(TCC_COMPONENT_DISPLAY_TYPE *update)
{
	dprintk("%s  \n", __func__);

	COMPONENT_LCDC_IMG_CTRL_TYPE ImgCtrl;
	unsigned int Y_offset, UV_offset;
	unsigned int img_widht = 0, img_height = 0, win_offset_x = 0, win_offset_y = 0;
	unsigned int lcd_w = 0, lcd_h =0;
	unsigned int lcd_ctrl_flag;
	
	tcc_component_get_lcdsize(&lcd_w, &lcd_h);

	//printk("lcd_w : %d, lcd_h %d \n", lcd_w, lcd_h);
	//printk("update_set->pbuffY : %x, update_set->pbuffU : %x, update_set->pbuffV  : %x \n",update_set->pbuffY, update_set->pbuffU, update_set->pbuffV );

	// Set LCD Image channel 0 
	memset((void*)&ImgCtrl, NULL, sizeof(COMPONENT_LCDC_IMG_CTRL_TYPE));

	if(Component_Mode == COMPONENT_MODE_720P)
 		ImgCtrl.INTL = FALSE;
	else
 		ImgCtrl.INTL = TRUE;
	
	ImgCtrl.IEN = update->enable;
	ImgCtrl.FMT = tcc_component_change_formattype(update->fmt);

	if(ImgCtrl.FMT >= COMPONENT_LCDC_IMG_FMT_YUV420SP)
		ImgCtrl.Y2R = TRUE;
	else
		ImgCtrl.Y2R = FALSE;

	ImgCtrl.UPD = TRUE;
	
	tcc_component_set_imagebase(IMG_CH0, (unsigned int)update->pbuffY, (unsigned int)update->pbuffU, (unsigned int)update->pbuffV);
	tcc_component_get_offset(ImgCtrl.FMT, update->frame_pix_width, &Y_offset, &UV_offset);

	if(update->img_pix_width > lcd_w)
	{
		win_offset_x = 0;
		img_widht = lcd_w;
	}
	else
	{
		img_widht = update->img_pix_width;
		win_offset_x = (lcd_w - img_widht)/2;
	}

	if(update->img_pix_height > lcd_h)
	{
		win_offset_y = 0;
		img_height = lcd_h;
	}
	else
	{
		img_height = update->img_pix_height;
		win_offset_y = (lcd_h - img_height)/2; 
	}
		
	if(update->src_off_sx)
		win_offset_x = update->src_off_sx;
	
	if(update->src_off_sy)
		win_offset_y =	update->src_off_sy; 
	
	if(ImgCtrl.INTL)
		win_offset_y = win_offset_y/2;

	dprintk("%s, img_widht:%d, img_height:%d, win_offset_x:%d, win_offset_y:%d\n", __func__, img_widht, img_height, win_offset_x, win_offset_y);
	
	lcd_ctrl_flag = SET_IMAGE_INTL |
					SET_IMAGE_UPD |
					SET_IMAGE_IEN |
					SET_IMAGE_Y2R |
					SET_IMAGE_FMT |
					SET_IMAGE_CEN |
					SET_IMAGE_AEN |
					SET_IMAGE_ASEL;

	tcc_component_set_imageinfo(SET_IMAGE_ALL, IMG_CH0, img_widht, img_height, win_offset_x, win_offset_y, Y_offset, UV_offset, 0, 0);
	tcc_component_set_imagectrl(lcd_ctrl_flag, IMG_CH0, &ImgCtrl);

	BITCSET(pComponent_HwLCDC->LCTRL, HwLCTRL_LEN, HwLCTRL_LEN);
}

/*****************************************************************************
 Function Name : tcc_component_set_interrupt()
******************************************************************************/
void tcc_component_set_interrupt(unsigned int onoff, unsigned int lcdc)
{	 
	unsigned int LCDC_INTbit;
	volatile PPIC pHwPIC = (volatile PPIC)tcc_p2v(HwPIC_BASE);

	dprintk("%s, onoff=%d, lcdc=%d\n", __func__, onoff, lcdc);

	if(lcdc)
		LCDC_INTbit = HwINT0_LCD1;
	else
		LCDC_INTbit = HwINT0_LCD0;

	if(onoff == 1)
	{
		if(pComponent_HwLCDC)
			BITCSET(pComponent_HwLCDC->LIM, 0xFFFFFFFF, HwLSTATUS_RU);

		BITSET(pHwPIC->POL0, LCDC_INTbit);
		BITSET(pHwPIC->SEL0, LCDC_INTbit);
		BITSET(pHwPIC->MODE0, LCDC_INTbit);
		BITSET(pHwPIC->IEN0, LCDC_INTbit);
	}
	else
	{
		BITCLR(pHwPIC->IEN0, LCDC_INTbit);
	}
}

/*****************************************************************************
 Function Name : tcc_component_end()
******************************************************************************/
void tcc_component_end(void)
{
	dprintk("%s, LCDC_Num = %d\n", __func__, Component_LCDC_Num);

	tcc_component_set_interrupt(FALSE, Component_LCDC_Num);
	
	cs4954_reset();

	#if defined(CONFIG_CPU_FREQ_TCC92X) || defined (CONFIG_CPU_FREQ_TCC93XX)
		tcc_cpufreq_set_limit_table(&gtTvClockLimitTable, TCC_FREQ_LIMIT_TV, 0);
	#endif
}

/*****************************************************************************
 Function Name : tcc_component_start()
******************************************************************************/
void tcc_component_start(TCC_COMPONENT_MODE_TYPE mode)
{
	PGPIO pGPIO = (volatile PGPIO)tcc_p2v(HwGPIO_BASE);

	dprintk("%s, mode=%d\n", __func__, mode);

	switch(mode)
	{
		case COMPONENT_NTST_M:
			Component_Mode = COMPONENT_MODE_NTSC_M;
			break;
		case COMPONENT_PAL:
			Component_Mode = COMPONENT_MODE_PAL_B;
			break;
		case COMPONENT_720P:
			Component_Mode = COMPONENT_MODE_720P;
			break;
		case COMPONENT_1080I:
			Component_Mode = COMPONENT_MODE_1080I;
			break;
	}

	if(Component_Mode == COMPONENT_MODE_720P)
	{
		TCC_OUTPUT_LCDC_CtrlLayer(TCC_OUTPUT_COMPONENT, 0, TCC_LCDC_IMG_FMT_RGB565);

		/* PID port of THS9200 */
		#if defined(CONFIG_ARCH_TCC92X)
			#if defined(CONFIG_MACH_TCC8900) || defined(CONFIG_MACH_TCC9200) || defined(CONFIG_MACH_TCC9201)
			BITSET(pGPIO->GPCDAT, Hw28);
			#else
			BITSET(pGPIO->GPCDAT, Hw30);
			#endif
		#elif defined(CONFIG_ARCH_TCC93XX)
			#if defined(CONFIG_MACH_TCC9300ST)
			BITSET(pGPIO->GPDDAT, Hw7);
			#elif defined(CONFIG_MACH_TCC9300CM)
			BITSET(pGPIO->GPCDAT, Hw28);
			#else
			BITSET(pGPIO->GPEDAT, Hw1);
			#endif
		#endif
	}

	tca_lcdc_port_setting(Component_LCDC_Num);
	tcc_component_set_lcd2tv(Component_Mode);

	#ifdef TCC_COMPONENT_IC_CS4954
		cs4954_enable(Component_Mode);
	#endif

	#ifdef TCC_COMPONENT_IC_THS8200
		ths8200_enable(Component_Mode);
	#endif
}

/*****************************************************************************
 Function Name : tcc_component_clock_onoff()
******************************************************************************/
void tcc_component_clock_onoff(char OnOff)
{
	dprintk("%s, Onoff = %d \n", __func__, OnOff);

	if(OnOff)
	{
		if(Component_LCDC_Num)
			clk_enable(component_lcdc1_clk);
		else
			clk_enable(component_lcdc0_clk);
	}
	else
	{
		if(Component_LCDC_Num)
			clk_disable(component_lcdc1_clk);
		else		
			clk_disable(component_lcdc0_clk);
	}

}

/*****************************************************************************
 Function Name : tcc_component_ioctl()
******************************************************************************/
static int tcc_component_ioctl(struct inode *inode, struct file *file, unsigned int cmd, void *arg)
{
	TCC_COMPONENT_START_TYPE start;
	TCC_COMPONENT_MODE_TYPE type;
	TCC_COMPONENT_DISPLAY_TYPE update;			
	
	dprintk("component_ioctl IOCTRL[%d] \n", cmd);      

	switch(cmd)
	{
		case TCC_COMPONENT_IOCTL_START:
			copy_from_user(&start,arg,sizeof(start));
			#if !defined(CONFIG_ARCH_TCC93XX) && !defined(CONFIG_TCC_OUTPUT_STARTER)
				if(start.lcdc == COMPONENT_LCDC_0)
					TCC_FB_LCDC_ChangeReg(1, FALSE);
				else
					TCC_FB_LCDC_ChangeReg(0, FALSE);
			#endif

			TCC_OUTPUT_LCDC_OnOff(TCC_OUTPUT_COMPONENT, start.lcdc, TRUE);

			if(start.lcdc != Component_LCDC_Num)
			{
				tcc_component_set_lcdc(start.lcdc);
				tcc_component_clock_onoff(TRUE);
			}
			tcc_component_start(start.mode);
			break;

		case TCC_COMPONENT_IOCTL_UPDATE:
			copy_from_user(&update,arg,sizeof(update));
			tcc_component_update(&update);									
			break;
			
		case TCC_COMPONENT_IOCTL_END:
			#if !defined(CONFIG_ARCH_TCC93XX) && !defined(CONFIG_TCC_OUTPUT_STARTER)
				if(fb_power_state)	
					TCC_FB_LCDC_ChangeReg(1, 1);
			#endif
			TCC_OUTPUT_LCDC_OnOff(TCC_OUTPUT_COMPONENT, Component_LCDC_Num, FALSE);
			tcc_component_end();
			break;

		default:
			printk(" Unsupported IOCTL!!!\n");      
			break;			
	}

	return 0;
}

/*****************************************************************************
 Function Name : tcc_component_open()
******************************************************************************/
static int tcc_component_open(struct inode *inode, struct file *filp)
{	
	dprintk("%s  \n", __func__);

	tcc_component_clock_onoff(1);

	return 0;
}

/*****************************************************************************
 Function Name : tcc_component_release()
******************************************************************************/
static int tcc_component_release(struct inode *inode, struct file *file)
{
	dprintk("%s  \n", __func__);

	//LCD off
	#if defined(CONFIG_MACH_TCC9300ST)
	if(pComponent_HwLCDC)
		pComponent_HwLCDC->LI0C = 0 | HwLIC_UPD;
	#endif

	tcc_component_clock_onoff(0);
	
	return 0;
}

static struct file_operations tcc_component_fops = 
{
	.owner			= THIS_MODULE,
	.ioctl			= tcc_component_ioctl,
	.open			= tcc_component_open,
	.release		= tcc_component_release,	
};

static struct class *tcc_component_class;

/*****************************************************************************
 Function Name : tcc_component_init()
******************************************************************************/
int __init tcc_component_init(void)
{
	printk("%s  \n", __func__);

	component_lcdc0_clk = clk_get(0, "lcdc0");
	BUG_ON(component_lcdc0_clk == NULL);
	component_lcdc1_clk = clk_get(0, "lcdc1");
	BUG_ON(component_lcdc1_clk == NULL);

	register_chrdev(MAJOR_ID, DEVICE_NAME, &tcc_component_fops);
	tcc_component_class = class_create(THIS_MODULE, DEVICE_NAME);
	device_create(tcc_component_class, NULL, MKDEV(MAJOR_ID, MINOR_ID), NULL, DEVICE_NAME);
	
	return 0;
}

/*****************************************************************************
 Function Name : tcc_component_cleanup()
******************************************************************************/
void __exit tcc_component_cleanup(void)
{
	printk("%s LCDC:%d \n", __func__, Component_LCDC_Num);

	unregister_chrdev(MAJOR_ID, DEVICE_NAME);

	if(component_lcdc0_clk =! NULL)
		clk_put(component_lcdc0_clk);
	if(component_lcdc1_clk =! NULL)
		clk_put(component_lcdc1_clk);

	return;
}

module_init(tcc_component_init);
module_exit(tcc_component_cleanup);

MODULE_AUTHOR("Telechips");
MODULE_DESCRIPTION("Telechips COMPONENT Out driver");
MODULE_LICENSE("GPL");

